Package: ThurMod
Type: Package
Title: Thurstonian CFA and Thurstonian IRT Modeling
Version: 1.1.20
Date: 2026-05-04
Authors@R: person("Markus Thomas", "Jansen", email="mjansen@uni-wuppertal.de", role=c('aut','cre'), comment = c(ORCID = "0000-0002-5162-4409"))
Description: Fit Thurstonian forced-choice models (CFA (simple and factor) and IRT) in R. This package allows for the analysis of item response modeling (IRT) as well as confirmatory factor analysis (CFA) in the Thurstonian framework. Currently, estimation can be performed by 'Mplus' and 'lavaan'. References:
  Brown & Maydeu-Olivares (2011) <doi:10.1177/0013164410375112>;
  Jansen, M. T., & Schulze, R. (in review). The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data.;
  Maydeu-Olivares & Böckenholt (2005) <doi:10.1037/1082-989X.10.3.285>.
VignetteBuilder: knitr
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: stats, utils, mvtnorm, MASS, matrixStats, lavaan
Suggests: rmarkdown, knitr
URL: https://github.com/MarkusTJansen/ThurMod
BugReports: https://github.com/MarkusTJansen/ThurMod/issues
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.3
LazyData: true
Packaged: 2026-05-04 19:10:01 UTC; Jansen
Author: Markus Thomas Jansen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5162-4409>)
Maintainer: Markus Thomas Jansen <mjansen@uni-wuppertal.de>
Repository: CRAN
Date/Publication: 2026-05-04 19:50:02 UTC
Built: R 4.5.2; ; 2026-05-04 22:49:20 UTC; unix

Package: ThurMod
Type: Package
Title: Thurstonian CFA and Thurstonian IRT Modeling
Version: 1.1.20
Date: 2026-05-04
Authors@R: person("Markus Thomas", "Jansen", email="mjansen@uni-wuppertal.de", role=c('aut','cre'), comment = c(ORCID = "0000-0002-5162-4409"))
Description: Fit Thurstonian forced-choice models (CFA (simple and factor) and IRT) in R. This package allows for the analysis of item response modeling (IRT) as well as confirmatory factor analysis (CFA) in the Thurstonian framework. Currently, estimation can be performed by 'Mplus' and 'lavaan'. References:
  Brown & Maydeu-Olivares (2011) <doi:10.1177/0013164410375112>;
  Jansen, M. T., & Schulze, R. (in review). The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data.;
  Maydeu-Olivares & Böckenholt (2005) <doi:10.1037/1082-989X.10.3.285>.
VignetteBuilder: knitr
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: stats, utils, mvtnorm, MASS, matrixStats, lavaan
Suggests: rmarkdown, knitr
URL: https://github.com/MarkusTJansen/ThurMod
BugReports: https://github.com/MarkusTJansen/ThurMod/issues
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.3
LazyData: true
Packaged: 2026-05-04 19:10:01 UTC; Jansen
Author: Markus Thomas Jansen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5162-4409>)
Maintainer: Markus Thomas Jansen <mjansen@uni-wuppertal.de>
Repository: CRAN
Date/Publication: 2026-05-04 19:50:02 UTC
Built: R 4.5.2; ; 2026-05-04 21:51:27 UTC; unix

# Generated by roxygen2: do not edit by hand

export(blocksort)
export(count.combn)
export(count.xblocks)
export(designA)
export(fit.correct)
export(fit.lavaan)
export(fit.mplus)
export(get.scores)
export(get.xblocks)
export(get.xblocks.any)
export(i.name)
export(metablock)
export(mod.matrices)
export(pair.combn)
export(rankA)
export(read.mplus)
export(recode)
export(redundancies)
export(reliabiltyFS)
export(sim.data)
export(syntax.lavaan)
export(syntax.mplus)

### Roxygen-inzable documentation
#' Scores of the data set 'FC' from Mplus.
#'
#' This data set contains synthetic data of 1000 participants on all binary
#' indicators of 15 items and their factor scores. For each paired comparison, 
#' participants had to rank the two alternative items according to their
#' preference. It is assumed that transitivity holds (that is, the data comes
#' from a ranking task). More details can be found in Brown and Maydeu-Olivares
#' (2011), Jansen and Schulze (2023) and Maydeu-Olivares and Böckenholt (2005).
#'
#' @name FC_scores
#' 
#' @docType data
#' 
#' @keywords datasets
#'
#' @usage data(FC_scores)
#'
#' @format A data frame with 1000 observations on 111 variables. For a variable
#' ixiy, the result is the response preferences between item x and item y. It is
#' coded a 1, if item x is preferred over item y, and 0 otherwise. The last six
#' variables are the resulting factor scores and the standard error of factor 
#' scores for three factors.
#' 
#'
#' @references 
#' Brown, A, & Maydeu-Olivares, A. (2011). Item response modeling of forced-choice questionnaires. \emph{Educational and Psychological Measurement}, \emph{71}(3), 460-502. \doi{10.1177/0013164410375112}
#' 
#' Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
#' 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}.
#'
#fit.mplus(1:15,rep(1:3,5),'irt')
#FC_scores <- read.table('myFactorScores.dat')
#save(FC_scores, file="data/FC_scores.rda")
"FC_scores"

get.xblocks.multidim <- function(nrank, blocks, itf, item_not, min, show.warnings){
  warn <- 0
  if(min){
    # If some items should not be used first, separate blocks
    if(!is.null(item_not)){
      tmp <- apply(matrix(blocks%in%item_not,ncol=nrank),1,any)
      not_blocks <- matrix(blocks[tmp,],ncol=nrank)
      not_left <- split_matrix(not_blocks,1,nrank)
      blocks <- blocks[!tmp,]
    }
    
    # Init
    tmp_block <- matrix(nrow=0,ncol=nrank)
    left <- split_matrix(blocks,1,nrank)
    
    # First iteration of connecting
    while(length(left)>=nrank){
      tmp <- left[1:nrank]
      tmp_left <- unlist(tmp)
      left[1:nrank] <- NULL
      tmp_choose <- c()
      tmp_itf <- c()
      for(i in 1:nrank){
        tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
        tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
        tmp3 <- tmptmp[[1]]
        tmp_itf <- c(tmp_itf,itf[tmp3])
        tmp_choose <- c(tmp_choose,tmp3)
      }
      tmp_block <- rbind(tmp_block,matrix(tmp_choose,ncol=nrank))
      left[[(length(left)+1)]] <- tmp_left
    }
    
    # If some items should not be used first, 1. connect them
    if(!is.null(item_not)){
      while(length(not_left)>=nrank){
        tmp <- not_left[1:nrank]
        tmp_left <- unlist(tmp)
        not_left[1:nrank] <- NULL
        whole_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),all))
        tmp_choose <- c()
        tmp_itf <- c()
        warn <- 0
        
        if(any(whole_not)){
          for(i in 1:length(tmp)){
            tmp2 <- unique(c(which(!tmp[[i]]%in%item_not),which(itf[tmp[[i]]]%in%tmp_itf)))
            tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
            tmp3 <- tmptmp[[1]]
            warn <- tmptmp[[2]]
            tmp_itf <- c(tmp_itf,itf[tmp3])
            tmp_choose <- c(tmp_choose,tmp3)
          }
        }else{
          for(i in 1:length(tmp)){
            tmp2 <- unique(c(which(tmp[[i]]%in%item_not),which(itf[tmp[[i]]]%in%tmp_itf)))
            tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
            tmp3 <- tmptmp[[1]]
            warn <- tmptmp[[2]]
            tmp_itf <- c(tmp_itf,itf[tmp3])
            tmp_choose <- c(tmp_choose,tmp3)
          }
        }

        tmp_choose <- matrix(tmp_choose,ncol=nrank)
        tmp_block <- rbind(tmp_block,tmp_choose)
        if(warn>0&show.warnings){
          warning(paste0("The addition of one mixed key block was unavoidable. This concerns block ",nrow(tmp_block),'.'))
        }
        not_left[[(length(not_left)+1)]] <- tmp_left
      }
      # 2. add them to the rest
      left <- c(not_left,left)
      
      # 3. connect blocks with as least different keyed blocks as possible
      while(length(left)>=nrank){
        tmp <- left[1:nrank]
        tmp_left <- unlist(tmp)
        left[1:nrank] <- NULL
        any_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),any))
        whole_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),all))
        tmp_choose <- c()
        tmp_itf <- c()
        warn <- 0
        
        if(any(whole_not)&all(any_not)){
          
            for(i in 1:length(tmp)){
              tmp2 <- unique(c(which(!tmp[[i]]%in%item_not),which(itf[tmp[[i]]]%in%tmp_itf)))
              tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
              tmp3 <- tmptmp[[1]]
              warn <- tmptmp[[2]]
              tmp_itf <- c(tmp_itf,itf[tmp3])
              tmp_choose <- c(tmp_choose,tmp3)
          }
        }else{
          for(i in 1:length(tmp)){
            tmp2 <- unique(c(which(tmp[[i]]%in%item_not),which(itf[tmp[[i]]]%in%tmp_itf)))
            tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
            tmp3 <- tmptmp[[1]]
            warn <- tmptmp[[2]]
            tmp_itf <- c(tmp_itf,itf[tmp3])
            tmp_choose <- c(tmp_choose,tmp3)
          }
          
          tmp_block <- rbind(tmp_block,matrix(tmp_choose,ncol=nrank))
          if(warn>0&show.warnings){
            warning(paste0("The addition of one mixed key block was unavoidable. This concerns block ",nrow(tmp_block),'.'))
          }
          left[[(length(left)+1)]] <- tmp_left
        }
      }
    }
    
    # Last block
    tmp_choose <- c()
    tmp_itf <- c()
    warn <- 0
    if(length(left)!=1){
      missing <- nrank-length(left)
      tmp <- left
      # If some items should not be used first
      if(!is.null(item_not)){
        any_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),any))
        whole_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),all))
        if(any(whole_not)==T){
          for(i in 1:length(tmp)){
            tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
            tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
            tmp3 <- tmptmp[[1]]
            warn <- tmptmp[[2]]
            tmp_itf <- c(tmp_itf,itf[tmp3])
            tmp_choose <- c(tmp_choose,tmp3)
          }
        }else{
          for(i in 1:length(tmp)){
            tmp2 <- unique(c(which(tmp[[i]]%in%item_not),which(itf[tmp[[i]]]%in%tmp_itf)))
            tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
            tmp3 <- tmptmp[[1]]
            warn <- tmptmp[[2]]
            tmp_itf <- c(tmp_itf,itf[tmp3])
            tmp_choose <- c(tmp_choose,tmp3)
          }
        }
        num_items <- rev(order(unlist(lapply(left,length))))
        for(i in 1:missing){
          j <- ifelse(i%%length(left)==0,length(left),i%%length(left))
          tmp2 <- unique(c(which(left[[j]]%in%item_not),which(itf[left[[j]]]%in%tmp_itf)))
          tmp_choose[length(left)+i] <- sample(unlist(left[num_items[j]])[-c(which(unlist(left[num_items[j]])%in%c(tmp_choose)),tmp2)], size=1)
        }
        tmp_choose <- matrix(tmp_choose,ncol=nrank)
        tmp_block <- rbind(tmp_block,tmp_choose)
        left <- unlist(left)
      }else{#else
        for(i in 1:length(tmp)){
          tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
          tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
          tmp3 <- tmptmp[[1]]
          tmp_itf <- c(tmp_itf,itf[tmp3])
          tmp_choose <- c(tmp_choose,tmp3)
        }
        num_items <- rev(order(unlist(lapply(left,length))))
        for(i in 1:missing){
          j <- ifelse(i%%length(left)==0,length(left),i%%length(left))
          
          tmp2 <- which(itf[tmp[[j]]]%in%tmp_itf)
          tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
          tmp3 <- tmptmp[[1]]
          tmp_itf <- c(tmp_itf,itf[tmp3])
          tmp_choose <- c(tmp_choose,tmp3)
        }
        
        tmp_choose <- matrix(tmp_choose,ncol=nrank)
        tmp_block <- rbind(tmp_block,tmp_choose)
        if(warn>0&show.warnings){
          warning(paste0("The addition of one mixed key block was unavoidable. This concerns block ",nrow(tmp_block),'.'))
        }
        left <- unlist(left)
      }
    }
  }else{
    # Init
    tmp_block <- matrix(nrow=0,ncol=nrank)
    left <- split_matrix(blocks,1,nrank)
    # First iteration of connecting
    while(length(left)>=nrank){
      tmp <- left[1:nrank]
      tmp_left <- unlist(tmp)
      left[1:nrank] <- NULL
      tmp_choose <- c()
      tmp_itf <- c()
      for(i in 1:nrank){
        tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
        tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
        tmp3 <- tmptmp[[1]]
        tmp_itf <- c(tmp_itf,itf[tmp3])
        tmp_choose <- c(tmp_choose,tmp3)
      }
      tmp_block <- rbind(tmp_block,matrix(tmp_choose,ncol=nrank))
      left[[(length(left)+1)]] <- tmp_left
    }
    
    # Last block

    if(length(left)!=1){
      tests <- TRUE
      while(any(tests)){
        tmp_choose <- c()
        tmp_itf <- c()
        missing <- nrank-length(left)
        tmp <- left
        for(i in 1:length(tmp)){
          tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
          tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
          tmp3 <- tmptmp[[1]]
          tmp_itf <- c(tmp_itf,itf[tmp3])
          tmp_choose <- c(tmp_choose,tmp3)
        }
        num_items <- rev(order(unlist(lapply(left,length))))
        for(i in 1:missing){
          j <- ifelse(i%%length(left)==0,length(left),i%%length(left))
          tmp2 <- which(itf[tmp[[j]]]%in%tmp_itf)
          tmptmp <- choose.item(tmp,tmp2,i,T,itf,tmp_itf,warn)
          tmp3 <- tmptmp[[1]]
          tmp_itf <- c(tmp_itf,itf[tmp3])
          tmp_choose <- c(tmp_choose,tmp3)
        }
        tests <- c(any(apply(apply(pair.combn(blocksort(rbind(blocks,tmp_block))),1,function(x) x%in%pair.combn(blocksort(matrix(tmp_choose,ncol=nrank)))[1,]),2,all)),
                   any(apply(apply(pair.combn(blocksort(rbind(blocks,tmp_block))),1,function(x) x%in%pair.combn(blocksort(matrix(tmp_choose,ncol=nrank)))[2,]),2,all)),
                   any(apply(apply(pair.combn(blocksort(rbind(blocks,tmp_block))),1,function(x) x%in%pair.combn(blocksort(matrix(tmp_choose,ncol=nrank)))[3,]),2,all)))
      }

      tmp_choose <- matrix(tmp_choose,ncol=nrank)
      tmp_block <- rbind(tmp_block,tmp_choose)
      left <- unlist(left)
    }
  }
  return(tmp_block)
}

# get orders from 01 format

pord <- function(data,block){
  tmp0 <- utils::combn(block,2)
  tmp0 <- table(c(tmp0[1,which(data==1)],tmp0[2,which(data==0)],block))
  tmp1 <- order(tmp0, decreasing = TRUE)
  return(tmp1)
}

# Genuine Likelihood
# Estimates Genuine Likelihood based on (Yousfi, 2019)

bglh <- function(fs,nrank,thres_b,lo_b,res_b,matT,ord_b,ord_o,alg=mvtnorm::TVPACK(),log=T){
  means <- thres_b[ord_o] + (matT%*%(lo_b[ord_b,]%*% fs))[-nrank]
  vars <- (tcrossprod(matT%*%res_b[ord_b,ord_b], matT))[-nrank,-nrank]
  
  out <- mvtnorm::pmvnorm(lower=rep(0,nrank-1), upper=rep(Inf,nrank-1), mean=means, sigma=vars, algorithm=alg)
  
  ## out: likelihood for one block and one person
  if (log==TRUE) {
    return(ifelse(out<=0, -10^100, log(out))) #small number
  } else {
    return(ifelse(out<=0, 1e-100, out)) #number close to 0
  }
}

### Roxygen-izable Documentation ----
#' Determine all paired comparisons
#' 
#' This function returns a matrix containing all paired comparisons defined by
#' a design.
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param unique Logical. Should only unique paired comparisons be returned?
#' 
#' 
### Outputs ---- 
#' @return Returns a matrix with all paired comparisons defined by a design.
#' 
### Examples ----
#' @examples
#' 
#' #' # Define 30 items divided by three triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' # Get all blocks
#' pair.combn(blocks)
#'     
#' @export

### Function definition ----


pair.combn <- function(blocks,unique=TRUE){
  pair_combn <- NULL
  if(is.null(nrow(blocks))){
    blocks <- t(blocks)
  }
  for(i in 1:nrow(blocks)){
    pair_combn <- rbind(pair_combn,t(utils::combn(blocks[i,],2)))
  }
  if(unique){
    pair_combn <- unique(pair_combn[order(pair_combn[,1],pair_combn[,2]),])
  }else{
    pair_combn <- pair_combn[order(pair_combn[,1],pair_combn[,2]),]
  }
  return(pair_combn)
}

split_matrix <- function(mat, r=NULL, c=NULL){
  if(is.null(r)){
    r <- nrow(mat)
  }
  if(is.null(c)){
    c <- ncol(mat)
  }
  rg <- (row(mat)-1)%/%r+1
  cg <- (col(mat)-1)%/%c+1
  rci <- (rg-1)*max(cg) + cg
  N <- prod(dim(mat))/r/c
  cv <- lapply(1:N, function(x) mat[rci==x])
  return(cv)
}

### Roxygen-izable Documentation ----
#' Get extra blocks in a Thurstonian design, that links as few blocks as possible.
#' 
#' The function creates extra blocks in a Thurstonian design, that links as few
#' initial blocks as possible, with the number of blocks determined by
#' `count.xblocks`. This is only useful for comparisons between linked and
#' partially linked block designs (Jansen & Schulze, 2023). 
#' 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
##' @param multidim Logical. Should the items within each linking block be
##' forced to be multidimensional?
#' 
### Details ---- 
#' @details The main strategy of the function is to create extra blocks that
#' link as few blocks as possible, with the number of blocks determined by
#' `count.xblocks`. Therefore, first all combinations of additional blocks with
#' the first two blocks are created. If more extra blocks are needed the
#' function uses block three, four, etc..
#' 
### Outputs ---- 
#' @return The result is a matrix where the rows correspond to the specific
#' extra blocks.
#' 
### Examples ----
#' @examples
#' 
#' # Define a matrix of blocks
#' blocks <- matrix(1:15,ncol=3, byrow=TRUE)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' # Get the extra blocks for a partially linked design
#' get.xblocks.any(blocks, itf, FALSE)
#'
#' @references
#' Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
#' 
#' @export

### Function definition ----

get.xblocks.any <- function(blocks,itf,multidim){
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  any.block <- matrix(nrow=0,ncol=nrank)
  nxblocks <- count.xblocks(blocks)
  i <- 2
  while(nrow(any.block)!=nxblocks){
    tmp <- t(utils::combn(c(blocks[1:i,]),nrank))
    for(j in 1:i){
      tmp <- tmp[-which(apply(apply(tmp,1,function(x) x%in%blocks[j,]),2,all)),]
    }
    if(multidim){
      itf_mat <- matrix(itf[tmp],ncol=nrank)
      tmp <- tmp[-which(apply(itf_mat,1,function(x) length(unique(x)))!=nrank),]
    }
    if(nxblocks<=nrow(tmp)){
      any.block <- tmp[sample(1:nrow(tmp),nxblocks),]
    }
    i <- i+1
  }
  return(any.block)
}

### Roxygen-izable Documentation ----
#' Recode variables.
#' 
#' 
#' 
#' @author Markus Thomas Jansen
#' 
#' 
#' 
### Inputs ----
#' @param var A variable/column of a data matrix/ data frame containing data to be recoded.
#' @param vals A vector containing the original scores. If cat is TRUE, these are thresholds.
#' @param ct A vector containing the desired scores.
#' @param cat Logical. should the data be categorized?
#' 
#' ### Outputs ---- 
#' @return Returns a vector with the recoded input values.
#' 
### Deatils ----

#' @details This function is a simple recode function. It is possible to recode any value to any other value, as long as the class of values within a vector are equal. If an interval of numerical values should be categorized, for N categories, N-1 thresholds must be given. The first N-1 categories are constructed by using all values that are smaller or equal (<=) than the thresholds, the last category is constructed to be all values larger than the last threshold.

### Examples ----
#' @examples
#' #numerical
#' v <- rep(c(1:5),10)
#' v_r <- recode(v, c(1:5), c(5:1))
#'
#' # character
#' v <- rep(c('a','b','d','e'),10)
#' v_r <- recode(v, c('a','b','d','e'), c('apple','1','dummy',TRUE))
#' 
#' # interval
#' set.seed(1)
#' v <- sample(runif(20,0,6))
#' 
#' # recode x <= 1.3 into 1
#' # recode x <= 2.6 into 2
#' # recode x <= 3.9 into 3
#' # recode x <= 5.2 into 4
#' # recode x > 5.1 into 5
#' v_r <- recode(v,c(1.3,2.6,3.9,5.2),c(1:5),cat=TRUE)
#' 
#' @export


### Function definition ----
recode <- function(var=var, vals=c(1,2,3,4,5), ct=c(5,4,3,2,1),cat=FALSE){
  varnew <- var
  if(cat){
    done <- c()
    for (i in 1:length(vals)){
      score <- setdiff(which(var<=vals[i]),done)
      varnew  <- replace(varnew, score, values=ct[i])
      done <- c(done,score)
    }
    score <- setdiff(which(var>vals[length(vals)]),done)
    varnew  <- replace(varnew, score, values=ct[length(ct)])
  }else{
    for (i in 1:length(vals)) {
      score <- which(var==vals[i])
      varnew  <- replace(varnew, score, values=ct[i])
    }
  }
  return(varnew)
}

### Roxygen-izable Documentation ----
#' Performs lavaan estimation of the given model.
#' 
#' This function writes a lavaan syntax given the specifications of the
#' Thurstonian forced choice model. Additionally it runs the code (given lavaan
#' is installed) and returns the results.
#' 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' @param data A matrix or data frame including the binary indicators as columns
#' and respondents as rows.
#' @param estimator Which estimator should be used? All estimators that are
#' available in `lavaan` can be used. Defaults to `'ULSMV'`.
#' @param rename_list A list with two vectors to rename the objects in the syntax.
#' Vector one is the original names, vector two the new names. Defaults to `NULL`.
#' 
### Outputs ---- 
#' @return Returns a lavaan object containing the specified results, after model
#' analysis.
#' 
#' @details The syntax currently is able to perform model analysis for the
#' latent utility model (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
#' unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
#' the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
#' model types can be performed with all types of forced choice designs (full,
#' block, partially linked block, linked block). For an overview and review see
#' Jansen and Schulze (2023a,2023b).
#' 
#' The standard naming procedure ixiy, for the comparison of items x and y, 
#' can be changed by specifying the `rename_list` argument. The first vector of
#' the schould be the vector of original names, for example 
#' `c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
#' the second vector should contain the new names, for example 
#' `c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.
#' 
### Examples ----
#' @examples
#' 
#' # read and save data set FC
#' data(FC12)
#' 
#' # set seed and define blocks
#' blocks <- matrix(c(5,2,1,4,7,6,3,8,10,12,9,11), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,4)
#' 
#' # Create and run syntax
#' \donttest{fit <- fit.lavaan(blocks, itf, 'irt', FC, estimator = 'ULSMV')}
#' 
#' @references 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}
#'
#' Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
#'  
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.
#' 
#' Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 
#' 
#'     
#' @export

fit.lavaan <- function(blocks,itf,model,data=NULL,estimator='ULSMV',rename_list=NULL){
  
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- matrix(blocks,nrow=1)
  }
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  modelsyn <- syntax.lavaan(blocks,itf,model,rename_list)
  
  if(model=='irt'){
    results <- lavaan::lavaan(modelsyn, data = data, ordered = T, 
                               auto.fix.first = FALSE, auto.th = TRUE, parameterization = "theta", estimator=estimator)
  }else if(model=='lmean'|model=='simple2'|model=='simple3'|model=='simple5'){
    results <- lavaan::lavaan(modelsyn, data = data, ordered = T, 
                              auto.fix.first = FALSE, auto.var = TRUE, int.lv.free = TRUE, parameterization = "theta", estimator=estimator)
  }else{
    results <- lavaan::lavaan(modelsyn, data = data, ordered = T, 
                              auto.fix.first = FALSE, auto.var = TRUE, auto.th = TRUE, parameterization = "theta", estimator=estimator)
  }
  
  return(results)

}

### Roxygen-izable Documentation ----
#' Correct degree of freedom and fit indices in Thurstonian block models
#' 
#' Correct fit indices (RMSEA and CFI) by correcting the degrees of freedom
#' after estimation a Thurstonian model.
#' 
#' 
### Inputs ----
#'
#' @param n The number of respondents.
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param chi2_mod The \eqn{\chi^2}{chi^2} value of the estimated model.
#' @param df_mod The degrees of freedom of the estimated model.
#' @param chi2_base The \eqn{\chi^2}{chi^2} value of the baseline model.
#' @param df_base The degrees of freedom of the baseline model.
#' 
### Outputs ---- 
#' @return Returns a vector containing corrected degrees of freedom, and the
#' corrected RMSEA and CFI values.
#' 
### Details
#' @details If a ranking design is used (variances of binary indicators is zero)
#' there are redundancies among the thresholds and tetrachoric correlations to 
#' be estimated. This is the case, whenever the number of items per block is 
#' larger than two. In these cases the degrees of freedom must be corrected by
#' subtracting the redundancies. For more details see Jansen and Schulze (2023)
#' and Maydeu-Olivares (1999).
#' 
### Examples ----
#' @examples
#' 
#' # Define 30 items divided by ten triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' # Assume the model yield the following fit, with 426 respondents
#' # chi2_mod = 224.456, df_mod = 59, chi2_base = 1056.566, df_base = 90
#' 
#' fit.correct(426,blocks,224.456,59,1056.566,90)
#' 
#' # The corrected values are rmsea = 0.0917892; cfi =  0.8184749
#'
#' @references 
#' Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
#' 
#' Maydeu-Olivares, A. (1999). Thurstonian modeling of ranking data via mean and covariance structure analysis. \emph{Psychometrika}, \emph{64}(3), 325-340. \doi{10.1007/BF02294299}
#' 
#' @export

fit.correct <- function(n,blocks,chi2_mod,df_mod,chi2_base,df_base){
  df_corr <- df_mod-redundancies(blocks)
  names(df_corr) <- 'df_corr'
  rmsea_corr <- sqrt(max(((chi2_mod-df_corr)/(n-1))/df_corr,0))
  cfi_corr <- min(1-(chi2_mod-df_corr)/(chi2_base-df_base),1)
  return(c(df=df_corr,rmsea=rmsea_corr,cfi=cfi_corr))
}

### Roxygen-izable Documentation ----
#' Determine the number of redundancies
#' 
#' This function determines the number of redundancies among the tetrachoric
#' correlations and thresholds.
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param warn Logical. Should warning messages be returned? Defaults to `TRUE`.
#' 
#' 
### Outputs ---- 
#' @return Returns an integer of the number of redundancies.
#' 
#' @details If a ranking design is used (variances of binary indicators is zero)
#' there are redundancies among the thresholds and tetrachoric correlations to 
#' be estimated. This is the case, whenever the number of items per block is 
#' larger than two. In these cases the degrees of freedom must be corrected by
#' subtracting the redundancies. For more details see Jansen and Schulze (2023)
#' and Maydeu-Olivares (1999).
#' 
### Examples ----
#' @examples
#' 
#' # Define 30 items divided by ten triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,10)
#' 
#' # Determine the redundancies
#' redundancies(blocks)
#' 
#' @references 
#' Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
#' 
#' Maydeu-Olivares, A. (1999). Thurstonian modeling of ranking data via mean and covariance structure analysis. \emph{Psychometrika}, \emph{64}(3), 325-340. \doi{10.1007/BF02294299}
#' 
#' @export


redundancies <- function(blocks,warn=TRUE){
  #message('Calculation gblocks...\n')
  gblocks <- metablock(blocks)
  nrank <- dim(blocks)[2]
  
  # original blocks for each gblock
  tmp <- lapply(gblocks, function(x) blocks[which(apply(matrix(blocks%in%x, ncol=ncol(blocks)),1,all)),])
  
  # number of paired comparisons if pairs are unique and not unique
  #message('Testing for block design...\n')
  tmp0 <- unlist(lapply(lapply(tmp,function(x) pair.combn(x)),function(x) ifelse(is.null(nrow(x)),2,nrow(x))))
  tmp1 <- unlist(lapply(tmp,function(x) ifelse(is.null(nrow(x)),2,nrow(pair.combn(x,unique = FALSE)))))
  
  # number of pairs that are possible with all items per gblock
  tmp2 <- unlist(lapply(gblocks,function(x) ifelse(is.null(nrow(pair.combn(x))),1,nrow(pair.combn(x)))))
  
  # Is each comparison considered only once? 
  tmp3 <- tmp0==tmp1
  
  # If comparisons are considered multiple times, are all comparisons considered?
  tmp4 <- tmp0==tmp2
  
  # If any of the two above is true, then calculation of redundancies is easy, else:
  tmp5 <- tmp0[!(tmp3|tmp4)]
  
  if(length(tmp5)!=0){
    if(max(tmp5)>70&warn==T){
      stop('The number of items is larger than 70, but it is not fully linked. It is possible that the calculation of redundancies takes a long time, or is not possible due to computer ressources. I you want to try again use warn=FALSE.')
    }
  }
  
  gpairs <- lapply(tmp,function(x) pair.combn(x))
  red <- 0
  #message('Caculating redundancies per block...\n')
  for(i in 1:length(gblocks)){
    if(tmp4[i]){
      nitems <- length(unique(c(gblocks[[i]])))
      red <- red+nitems*(nitems-1)*(nitems-2)/6
    }else if(tmp3[i]){
      red <- red+nrow(tmp[[i]])*nrank*(nrank-1)*(nrank-2)/6
    }else{

      # Calculation via transitivities
      pairs <- gpairs[[i]]
      all_redun <- t(utils::combn(unique(c(pairs)),3))
      tmp0 <- apply(pairs,1,function(x) paste0(x, collapse = ','))
      tmp1 <- apply(all_redun,1,function(x) paste0(x[c(1,2)], collapse = ','))
      tmp2 <- apply(all_redun,1,function(x) paste0(x[c(1,3)], collapse = ','))
      tmp3 <- apply(all_redun,1,function(x) paste0(x[c(2,3)], collapse = ','))
      tmp <- cbind(tmp1%in%tmp0,tmp2%in%tmp0,tmp3%in%tmp0)
      red <- length(which(apply(tmp,1,all)))
    }
  }
  red <- ifelse(length(red)==0,0,red)
  return(red)
}

### Roxygen-izable Documentation ----
#' Determine the number of extra blocks
#' 
#' This function determines the minimal number of extra blocks needed in order
#' to link all blocks.
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' 
### Details ---- 
#' @details The matrix of blocks must be constructed so that the number of
#' columns corresponds to the number of items per block. The number of rows
#' corresponds to the number of blocks for the specific measure. If $p$ is the
#' number of blocks, and $k$ is the number of items per block (e.g. $k=3$ for
#' triplets), then the number of extra blocks can be determined by (see also
#' Jansen & Schulze, 2023)
#' \deqn{\lceil\frac{p-1}{k-1}\rceil}
#' 
### Outputs ---- 
#' @return An integer corresponding to the number of extra blocks needed.
#' 
### Examples ----
#' @examples
#' 
#' # Define a matrix of blocks
#' blocks <- matrix(1:15,ncol=3, byrow=TRUE)
#' 
#' # Determine the number of extra blocks needed
#' count.xblocks(blocks)
#' 
#' @references 
#' Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
#'
#' @export


### Function definition ----
count.xblocks <- function(blocks){
  return(ceiling((nrow(blocks)-1)/(ncol(blocks)-1)))
}

### Roxygen-izable Documentation ----
#' Create data based on Thurstonian model equations
#' 
#' Simulates a data set of paired comparisons or ranking data based a Thurstonian
#' latent utility model.
#' 
### Inputs ----
#'
#' @param nfactor The number of factors. Defaults to 1.
#' @param nitem The number of items.
#' @param nperson The number of data points (= respondents) to simulate.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model The model class from which to simulate. Options are 'simple', 'factor', 'uc' and 'irt'. Defaults to 'factor'.
#' @param variables A vector containing the names of paired comparison variables
#' to return. If `NULL` (default), all variables are returned.
#' @param ints A vector defining the latent intercepts of item pairs. 
#' @param lmu A vector defining the latent means of items. 
#' @param ivarcov A matrix defining the variance-covariance matrix of the items.
#' @param loadings A vector defining the loadings of items.
#' @param varcov A matrix defining the variance-covariance matrix of the traits.
#' @param graded Logical. Should a graded preference model be simulated? Defaults to `FALSE`.
#' @param ncat Numerical. The number of categories to specify for graded preference models. If graded = `FALSE`, ncat is 2.
#' @param thres A vector of thresholds to categorize the latent difference response. If no thresholds are given, they are simulated from the distribution of the latent items. If graded = `FALSE`, the thresholds are all 0.
#' @param transitive Logical. Should the data be transitive? If `TRUE`, ranking
#' data is simulated, else paired comparison data is simulated. Defaults to `TRUE`.
#' @param var A vector containing the variances for each paired comparison. Defaults to 0.
#' @param eta A nfactor-dimensional matrix with factor scores. Defaults to `NULL`. If `NULL` factor scores are simulated from multivariate normal distribution.
#' @param fvalues Logical. Should simulated factor values be returned? Defaults to `FALSE`.
#' @param sim Logical. Should the simulated data be returned? Defaults to `TRUE`.
#' 
#' @details The syntax currently is able to simulate data from the
#' latent utility model (`'simple'` and `'factor'`; Maydeu-Olivares & Böckenholt, 2005) the 
#' unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
#' the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010).
#' 
#' 
### Outputs ---- 
#' @return Returns a list containing the true factor scores and the data, or a
#' matrix containing the data.
#' 
#' 
### Examples ----
#' @examples
#' 
#' nfactor <- 3
#' nitem <- 15
#' nperson <- 1000
#' itf <- rep(1:3,5)
#' varcov <- diag(1,3)
#' 
#' # latent utility means
#' set.seed(69)
#' lmu <- runif(nitem, -1, 1)
#' loadings <- runif(nitem, 0.30, 0.95)
#' 
#' FC <- sim.data(nfactor=nfactor, nitem=nitem, nperson=nperson, itf=itf, 
#' varcov = varcov, lmu = lmu, loadings=loadings)
#'     
#' @export

### Function definition ----
sim.data <- function(nfactor=1,nitem,nperson,itf,model='factor',variables=NULL, ints=NULL, lmu=NULL, ivarcov=NULL, loadings=NULL, varcov=NULL, graded = FALSE, ncat=NULL, thres = NULL, transitive = TRUE, var = 0, eta=NULL,fvalues=FALSE, sim=TRUE){  
  if(model%in%c('simple')){
    if(is.null(lmu)){
      stop('For simple Thurstonian models for Cases II, III and V you must specify latent item means (object lmu).')
    }
    if(is.null(ivarcov)){
      stop('For simple Thurstonian models for Cases II, III and V you must specify a covariance matrix of latent items (object ivarcov). For Case II variances and covarainces of latent items are free. For Case III variances are free and covarainces are fixed to zero. For Case V variances are all equal and covarainces are fixed to 0.')
    }
  }
  if(model=='factor'){
    if(is.null(nfactor)){
      stop('For Thurstonian factor models you must specify the number of factors (object nfactor).')
    }
    if(is.null(lmu)){
      stop('For Thurstonian factor models you must specify latent item means (object lmu).')
    }
    if(is.null(varcov)){
      stop('For Thurstonian factor models you must specify a covariance matrix of latent traits (object varcov).')
    }
    if(is.null(loadings)){
      stop('For Thurstonian factor models you must specify the laodings of items on latent traits (object loadings).')
    }
  }
  if(model%in%c('uc','irt')){
    if(is.null(nfactor)){
      stop('For Thurstonian irt models or unconstrained threshold models you must specify the number of factors (object nfactor).')
    }
    if(is.null(ints)){
      stop('For Thurstonian irt models or unconstrained threshold models you must specify latent intercepts of item pairs (object ints).')
    }
    if(is.null(varcov)){
      stop('For Thurstonian irt models or unconstrained threshold you must specify a covariance matrix of latent traits (object varcov).')
    }
    if(is.null(loadings)){
      stop('For Thurstonian irt models or unconstrained threshold you must specify the laodings of items on latent traits (object loadings).')
    }
  }
  if(graded){
    if(is.null(thres)&is.null(lmu)){
      stop('For Thurstonian graded preference models you must specify either a covariance matrix of latent items (object ivarcov) and the latent item means (object lmu), or the thresholds (object thres).')
    }
    if(is.null(ivarcov)&is.null(thres)){
      stop('For Thurstonian graded preference models you must specify either a covariance matrix of latent items (object ivarcov) and the latent item means (object lmu), or the thresholds (object thres).')
    }
    if(is.null(ncat)&is.null(thres)){
      stop('For Thurstonian graded preference models you must specify either the thresholds (object thres) or the number of categories to simulate (object ncat).')
    }
  }
  
  # Init parameters
  npair <- nitem*(nitem-1)/2
  
  if(!is.null(loadings)){
    # Set loadings, means, factor scores and latent variances (of items)
    # loadings
    tmp_l <- loadings
    lambda <- matrix(0, ncol=nfactor, nrow = nitem)
    
    for(i in 1:nitem){
      lambda[i,itf[i]] <- tmp_l[i]
    }
  }
  
  # design
  A <- designA(nitems=nitem)
  
  # simulate latent difference response
  if(transitive==FALSE){
    eps <- t(MASS::mvrnorm(nperson, rep(0,npair), diag(var,npair))) # variance pair comparison
  }else{
    eps <- 0 # variance pair comparison; 0 for rankings
  }
  
  if(sim==T){
    if(model%in%c('simple')){
      # Case II Covs_t free; Var_t free
      # Case III Covs_t = 0; Var_t free
      # Case V Covs_t = 0; Var_t (all equal)
      t <- t(MASS::mvrnorm(nperson, lmu, ivarcov)) #eq 7 (MO&B, 2010) 
      lresp <- A%*%t
    }
    # factor scores
    # make eta
    if(is.null(eta)){
      eta <- t(MASS::mvrnorm(nperson, rep(0,nfactor), varcov))
    }
    if(dim(eta)[1]!=nfactor){
      eta <- t(eta)
    }
    
    if(model=='factor'){
      # latent variances (items)
      psi <- t(MASS::mvrnorm(nperson, rep(0,nitem), diag(1-c(tmp_l^2),nitem)))
      pgam <- A%*%lmu
      At <- A%*%(lambda%*%eta+psi)
      lresp <- apply(At,2,function(x) x+pgam) # eq 11 (MO&B, 2010)
    }
    if(model%in%c('uc','irt')){
      # latent variances (items)
      psi <- t(MASS::mvrnorm(nperson, rep(0,nitem), diag(1-c(tmp_l^2),nitem)))
      pgam <- -ints
      At <- A%*%(lambda%*%eta+psi)
      lresp <- apply(At,2,function(x) x+pgam) # eq 21 (MO&B, 2010)
    }
    
    # Add variance of pairs if transitive = FALSE
    lresp <- lresp+eps
    
    if(graded){
      if(is.null(thres)){
        pairs <- t(utils::combn(1:nitem,2))
        gvars <- apply(pairs,1,function(x) ivarcov[x[1],x[1]]+ivarcov[x[2],x[2]]-2*ivarcov[x[1],x[2]])
        pmeans <- apply(pairs,1,function(x) lmu[x[2]]-lmu[x[1]]) # pair means
        thresholds <- sapply(1:length(pmeans),function(x) sort(stats::rnorm(ncat-1, pmeans[x], gvars[x])))
        
        tmp <- ncat/2==floor(ncat/2)
        
        if(tmp){
          thresholds <- apply(thresholds,2, function(x) x-x[ncat/2])
        }else{
          thresholds <- apply(thresholds,2, function(x) x-stats::runif(1,x[floor(ncat/2)],x[ceiling(ncat/2)]))
        }
      }
      if(is.vector(thres)){
        thresholds <- matrix(rep(thres,npair), ncol=length(thres),byrow = T)
      }
      resp <- sapply(1:dim(thresholds)[2],function(x) recode(t(lresp)[,x],thresholds[,x],1:ncat,cat=TRUE))
    }else{
      resp <- t(ifelse(lresp<0,0,1))
    }

    resp <- as.data.frame(resp)
    names(resp) <- i.name(1:nitem)
    
    if(!is.null(variables)){
      tmp <- which(i.name(1:nitem)%in%variables)
      resp <- resp[,tmp]
    }
    
  }else{
    resp <- 'You chose to not simulate the data.'
  }
  
  if(fvalues==T){
    return(list(resp,eta))
  }else{
    return(resp)
  }
  
}

# Genuine Likelihood
# Estimates Genuine Likelihood based on (Yousfi, 2019)
# Likelihood per person

glh <- function(fs, dat, nrank, blocks, thres_bs, lo, res, matT, ord, ord_o, estimator="MAP", mp, sp, alg, log) {
  like <- 0
  
  #loop over all blocks (also additional)
  for (k in 1:nrow(blocks)) {
    b <- blocks[k,]
    like <- like + bglh(fs=fs,nrank=nrank,thres_b=c(thres_bs[,k],-thres_bs[,k]),lo_b=lo[b,],res_b=res[b,b],
                      matT=matT,ord_b=ord[,k],ord_o=ord_o[,k],alg=alg,log=log)
  }
  if(estimator=="MAP") {
    return(mvtnorm::dmvnorm(x=fs, mean=mp, sigma=sp, log=TRUE) + like)
  } else {
    return(like)
  }
}

### Roxygen-izable Documentation ----
#' Create model matrices for Thurstonian modeling
#' 
#' This function creates and returns model matrices of Thurstonian model equations.
#' 
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' 
### Outputs ---- 
#' @return Returns a list of elements containing model matrix information.
#' 
### Examples ----
#' @examples
#' 
#' # set seed and define blocks
#' set.seed(1)
#' blocks <- matrix(sample(1:15,15), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' mod.matrices(blocks,itf,'irt')
#'     
#' @export


### Function definition ----

mod.matrices <- function(blocks,itf,model){
  
  if(model=='simple2'){
    model <- 'simple'
    case <- 2
  }
  if(model=='simple3'){
    model <- 'simple'
    case <- 3
  }
  if(model=='simple5'){
    model <- 'simple'
    case <- 5
  }
  
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- blocks <- matrix(blocks,nrow=1)
  }
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  # Find number of factors, blocks and items
  nblock <- dim(blocks)[1]
  nitem <- length(unique(c(blocks)))
  nfactor <- max(itf)
  
  # Pair combinations
  pair_combn <- t(utils::combn(1:nitem,2))
  
  # Define items to factors for lambda (loadings)
  lambda_raw <- matrix(0,nrow=nitem,ncol=nfactor)
  for(i in 1:nfactor){
    lambda_raw[,i] <- ifelse(itf==i,i,0)
  }
  
  # Create names of pairs
  pair_names <- paste0('i',pair_combn[,1],'i',pair_combn[,2])
  
  # For each block: is it uni-dimensional?
  blocks_lambda <- matrix(itf[blocks],ncol=nrank)
  unidim <- apply(blocks_lambda,1,function(x) min(x)==max(x))
  
  # Create names of pairs for blocks
  pair_names_bs <- i.name(blocksort(blocks))
  pair_names_b <- i.name(blocks)
  
  tmp <- which(pair.combn(blocks)[,1]>pair.combn(blocks)[,2])
  
  if(length(tmp)!=0){
    tmp1 <- pair_names_b[tmp]
    tmp2 <- sub('^i.+i','i',tmp1)
    tmp3 <- tmp1
    for(j in 1:length(tmp)){
      tmp3 <- paste0(tmp2[j],sub(paste0(tmp2[j],'$'),'',tmp1[j]))
      pair_names_bs[which(pair_names_bs%in%tmp3)] <- pair_names_b[tmp[j]]
    }
  }
  pair_combn_b <- pair.combn(blocks)
  
  
  # MODEL
  if(model!='simple'){
    traits <- paste0('Trait',1:nfactor)
  }
  
  # Create Matrix A
  A <- designA(blocks)
  Arref <- rref(A)
  Arank <- dim(A)[1]-length(which(apply(abs(Arref),1,sum)==0))
  
  # Find global blocks
  gblocks <- metablock(blocks)
  unidim_gblocks <- sapply(gblocks,function(x) min(itf[x])==max(itf[x]))
  
  # Latent utility model and the unconstrained threshold model
  if(model%in%c('lmean','uc','simple')){
    # Part 1 (primary factors)
    t <- paste0('t',1:nitem)
    A_sign <- ifelse(A==0,'',ifelse(A==1,'+','-'))
    
    At <- matrix(NA, nrow=nrow(A_sign), ncol=nitem)
    At <- t(apply(A_sign,1,function(x)sub('\\+','',sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,t)))))))
    At <- sub('^-','',matrix(paste0(At,' BY ',pair_names_b,'@',ifelse(A_sign=='+','',A_sign),'1;\n'), ncol=nitem))
    At <- ifelse(A_sign=='','',At)
    At_text <- paste0(apply(At,2,function(x)paste0(x,collapse = '')), collapse = '')
    
    # Part 2 (secondary factors)
    if(model!='simple'){
      if(nfactor==1&Arank==(nitem-1)){
        #input <- paste0(input, 'Trait1 BY\nt1@1\n', paste0(paste0('t',2:nitem,'*1'),collapse='\n'), ';\n\n')
      }else{
        A_fac <- rep('',nfactor)
        
        itf_gblocks <- lapply(gblocks,function(x) itf[x])
        items_fac <- list()
        items_fac_uni <- list()
        for(i in 1:nfactor){
          item_fac <- c()
          item_fac_uni <- c()
          for(j in 1:length(gblocks)){
            if(unidim_gblocks[j]){
              item_fac_uni <- c(item_fac_uni,gblocks[[j]][which(itf_gblocks[[j]]==i)[1]])
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)[-1]])
            }else{
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)])
            }
          }
          items_fac[[i]] <- sort(item_fac)
          if(all(is.na(item_fac_uni))){
            items_fac_uni[[i]] <- NA
          }else{
            items_fac_uni[[i]] <- sort(item_fac_uni)
          }
          
          A_fac[[i]] <- paste0(traits[i], ' BY\n')
          A_fac[[i]] <- paste0(paste0(traits[i], ' BY\n'),paste0('t',unname(items_fac[[i]]),'*1', '\n', collapse = ''))
          
          #for(j in 1:length(gblocks)){
          #if(unidim_gblocks[j]){
          #if(length(items_fac_uni[[i]])>0){
          if(all(!is.na(items_fac_uni[[i]]))){
            A_fac[[i]] <- paste0(A_fac[[i]],paste0('t',unname(items_fac_uni[[i]]),'@1', '\n', collapse = ''))
          }
          #}
          #}
          #}
        }
      }
    }
  }
  
  # IRT model
  if(model%in%c('irt')){
    # Create matrix lambda
    lambda <- ifelse(lambda_raw==0,0,paste0('L',row(lambda_raw)))
    A_sign <- ifelse(A==0,0,ifelse(A==1,'+','_'))
    A_lambda <- A%*%ifelse(lambda_raw==0,0,1)
    Lambda <- matrix(NA, nrow=nrow(A_sign), ncol=ncol(lambda))
    
    for(i in 1:nfactor){
      Lambda[,i] <- apply(A_sign,1,function(x) paste0(sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,lambda[,i])))), collapse =''))
    }
    tmp <- grep('^_L.+\\+',Lambda)
    tmp1 <- Lambda[tmp]
    Lambda[tmp] <- paste0(sub('^_L.+\\+','',tmp1),sub('\\+L.+','',tmp1))
    Lambda <- sub('\\+','',Lambda)
    Lambda <- sub('^_L','L_',Lambda)
    
    A_mplus <- NULL
    for(i in 1:nfactor){
      A_mplus <- paste0(A_mplus,'\n\n',traits[i],' BY \n',
                        paste0(ifelse(Lambda[,i]=='','',
                                      paste0(pair_names_b,'*',A_lambda[,i],' (',Lambda[,i],')\n')),collapse = ''),';')
    }
    
    if(nrank==2){
      
      Psi <- ifelse(row(diag(nitem))==col(diag(nitem)),paste0('1/2'),0)
      Psi <- ifelse(Psi==0,'',Psi) # neu
      A_sign <- ifelse(A==0,0,ifelse(A==1,'+','-')) # alt
      
      
      APsi <- matrix(NA, nrow=length(pair_names_b), ncol=nitem)
      for(i in 1:nitem){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[j,],Psi[,i])
          tmp <- sub('^-$','',sub('^\\+$','',tmp))
          tmp <- sub('^-0.*','',sub('^\\+0.*','',sub('^0.*','',tmp)))
          APsi[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsi <- sub('^\\+','',APsi)
      
      APsiA <- matrix(NA, nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:length(pair_names_b)){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[i,],APsi[j,])
          #tmp <- sub('\\+','',sub('^0.*','',tmp))
          tmp <- sub('^0.*','',tmp)
          APsiA[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsiA <- ifelse(APsiA=='+-','',ifelse(APsiA=='-+','',APsiA))
      APsiA <- gsub('-\\+','',APsiA)
      APsiA <- gsub('\\+-','-',APsiA)
      APsiA <- gsub('--','+',APsiA)
      
      APsiA <- sub('-$','',APsiA)#
      APsiA <- sub('\\+$','',APsiA)#
      APsiA <- sub('^\\+','',APsiA)
      
      APsiA <- sub('1/2+1/2','1',APsiA,fixed = TRUE)
      
      if(length(gblocks)!=nblock){
        # Define new parameters
        unidim_item <- Lambda[grep('^L.*_L',Lambda)]
        tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
        if(length(which(tmp%in%Lambda))!=0){
          tmp <- tmp[-which(tmp%in%Lambda)]
        }
        
        AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
        
        tmp <- c(AddL,tmp)
        Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
        
        Add_unidim <- NULL
        for(i in 1:nrow(blocks)){
          tmp <- blocks[i,]%in%sub('L','',Add_param)
          if(all(tmp)){
            Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
          }
        }
        
        # Constraints
        conL <- rep('',nitem)
        for(i in 1:nitem){
          if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
            if(paste0('L_',i)%in%Lambda){
              conL[i] <- paste0('L_',i,' = ','- L',i)
            }
          }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
            conL[i] <- paste0('L_',i,' = ','- L',i)
          }
        }
        
        Add_param <- paste0(paste0(Add_param, collapse = '\n'),';')
        unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' = ', sub('_','-',unidim_item),';\n',collapse = ''))
        conL <- paste0(ifelse(conL=='','',paste0(conL,';\n')), collapse = '')
      }else{
        Add_param <- NULL
      }
    }
    if(nrank>2){
      
      # Create Matrix Psi for irt models
      Psi <- ifelse(row(diag(nitem))==col(diag(nitem)),paste0('e',1:nitem),0)
      Psi <- ifelse(Psi==0,'',Psi) # neu
      A_sign <- ifelse(A==0,0,ifelse(A==1,'+','-')) # alt
      
      APsi <- matrix(NA, nrow=length(pair_names_b), ncol=nitem)
      for(i in 1:nitem){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[j,],Psi[,i])
          tmp <- sub('^-$','',sub('^\\+$','',tmp))
          tmp <- sub('^-0.*','',sub('^\\+0.*','',sub('^0.*','',tmp)))
          APsi[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsi <- sub('^\\+','',APsi)
      
      APsiA <- matrix(NA, nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:length(pair_names_b)){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[i,],APsi[j,])
          tmp <- sub('^0.*','',tmp)
          APsiA[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsiA <- ifelse(APsiA=='+-','',ifelse(APsiA=='-+','',APsiA))
      APsiA <- gsub('-\\+','',APsiA)
      APsiA <- gsub('\\+-','-',APsiA)
      APsiA <- gsub('--','+',APsiA)
      
      APsiA <- sub('-$','',APsiA)#
      APsiA <- sub('\\+$','',APsiA)#
      APsiA <- sub('^\\+','',APsiA)
      
      APsiA_under  <- sub('-e','e_',APsiA)
      APsiA_sign <- ifelse(APsiA=='','',ifelse(substr(APsiA,1,1)=='-','-',''))
      
      
      # Define new parameters
      unidim_item <- Lambda[grep('^L.*_L',Lambda)]
      tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
      if(length(which(tmp%in%Lambda))!=0){
        tmp <- tmp[-which(tmp%in%Lambda)]
      }
      
      AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
      
      tmp <- c(AddL,tmp)
      Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
      
      Add_unidim <- NULL
      for(i in 1:nrow(blocks)){
        tmp <- blocks[i,]%in%sub('L','',Add_param)
        if(all(tmp)){
          Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
        }
      }
      
      # Constraints
      conL <- rep('',nitem)
      for(i in 1:nitem){
        if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
          if(paste0('L_',i)%in%Lambda){
            conL[i] <- paste0('L_',i,' = ','- L',i)
          }
        }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
          conL[i] <- paste0('L_',i,' = ','- L',i)
        }
      }
      
      Add_param <- paste0(paste0(Add_param, collapse = '\n'),';')
      unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' = ', sub('_','-',unidim_item),';\n',collapse = ''))
      conL <- paste0(ifelse(conL=='','',paste0(conL,';\n')), collapse = '')
      
      
      # Pair's uniqueness is equal to sum of two utility uniquenesses
      if(design=='full'){
      }else if(design=='block'){
        uniquenesses <- c()
        for(i in 1:nrow(blocks)){
          items <- blocks[i,]
          whitems <- which(pair_combn_b[,1]%in%items&pair_combn_b[,2]%in%items)
          tmp <- pair_combn_b[whitems,1][which(pair_combn_b[whitems,1]%in%pair_combn_b[whitems,2])]
          
          tmp <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2],' = ' ,ifelse(pair_combn_b[whitems,1]%in%tmp,'-e_','e'),pair_combn_b[whitems,1],
                        ifelse(pair_combn_b[whitems,2]%in%tmp,' - e_',' + e'),pair_combn_b[whitems,2],';')
          tmp0 <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2])
          for(i in length(tmp0):1){
            if(any(grepl(tmp0[i],uniquenesses))){
              tmp <- tmp[-i]
            }
          }
          uniquenesses <- c(uniquenesses,tmp)
        }
        uniquenesses <- unique(uniquenesses[order(nchar(uniquenesses),uniquenesses)])
      }
      
    }
  }
  
  if(model=='irt'){
    return(list(lambda=Lambda, 
                resvar=paste0(pair_names_b,'*2 ','(e',pair_combn_b[,1],'e',pair_combn_b[,2],');', collapse = '\n'),
                Psi=Psi,
                APsi=APsi,
                APsiA=APsiA,
                npair=length(pair_names_b),
                naddparam=length(unlist(strsplit(gsub(';','',Add_param),'\n'))),
                addparam=strsplit(gsub(';','',Add_param),'\n'),
                pair_names_b=sub('i','I',pair_names_b)))
  }else if(model=='lmean'|model=='uc'){
    lambda_read <- strsplit(A_fac,'\n')
    #if(connected|xtra){
    #  for(i in 1:nfactor){
    #    lambda_read[[i]] <- lambda_read[[i]][1:(length(which(itf==i))+1)]
    #  }
    #}
    return(list(lambda=lambda_read,
                npair=length(pair_names_b),
                pair_names_b=sub('i','I',pair_names_b)))
  }else{
    #lambda_read <- strsplit(A_fac,'\n')
    #if(connected|xtra){
    #  for(i in 1:nfactor){
    #    lambda_read[[i]] <- lambda_read[[i]][1:(length(which(itf==i))+1)]
    #  }
    #}
    return(list(npair=length(pair_names_b),
                pair_names_b=sub('i','I',pair_names_b)))
  }
}

### Roxygen-izable Documentation ----
#' Performs Mplus estimation of the given model.
#' 
#' This function writes the Mplus syntax given the specifications of a
#' Thurstonian forced choice design. Additionally it runs the code
#' (given Mplus is installed) and returns the results.
#' 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' @param input_path Path to save the Mplus input file. Defaults to `'myFC_model.inp'`.
#' @param output_path Path to the Mplus output file. Defaults to `'myFC_model.out'`.
#' @param data_path Path of the data file for Mplus. Defaults to `'myDataFile.dat'`.
#' @param fscore_path Path to save the file of factor scores. Defaults to
#' `'myFactorScores.dat'`.
#' @param title Title of the Mplus model. Defaults to `'myFC_model'`.
#' @param ID Logical. Should a ID variable be included? The ID must be the first 
#' variable in the data set. Defaults to `FALSE`.
#' @param byblock Logical. Should the order in Mplus variable statement be the same
#' as in the blocks. Defaults to `TRUE`.
#' @param estimator Which estimator should be used? All Estimators that are
#' available in Mplus can be used. Defaults to `'ULSMV'`.
#' @param data_full Logical. Are the data considered to be from a full design?
#' Defaults to `FALSE`.
#' @param standardized Logical. Should standardized values be computed? Defaults
#' to `TRUE`.
#' @param rename_list A list with two vectors to rename the objects in the syntax.
#' Vector one is the original names, vector two the new names. Defaults to `NULL`.
#' @param ... Further arguments passed to function `read.mplus`.
#' 
### Outputs ---- 
#' @return Returns a list containing the specified results, after model analysis.
#' 
#' @details The syntax currently is able to perform model analysis for the
#' latent utility models (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
#' unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
#' the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
#' model types can be performed with all types of forced choice designs (full,
#' block, partially linked block, linked block). For an overview and review see
#' Jansen and Schulze (2023a,2023b).
#' 
#' The function writes and saves the Mplus input files, keeps the output files 
#' and reads the results specified for the function `read.mplus`.
#' 
#' The standard naming procedure ixiy, for the comparison of items x and y, 
#' can be changed by specifying the `rename_list` argument. The first vector of
#' the schould be the vector of original names, for example 
#' `c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
#' the second vector should contain the new names, for example 
#' `c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.
#' 
### Examples ----
#' @examples
#' 
#' # read and save data set FC
#' data(FC)
#' write.table(FC,paste0(tempdir(),'/','my_data.dat'),quote=FALSE, sep=" ",
#' col.names = FALSE, row.names = FALSE)
#' 
#' # set seed and define blocks
#' set.seed(1)
#' blocks <- matrix(sample(1:15,15), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' # perform analysis 
#' \dontrun{
#' fit.mplus(blocksort(blocks),itf,'irt',data_path = 'mydata.dat', data_full = TRUE,
#' input_path = paste0(tempdir(),'/','myFC_model'))
#' }
#' 
#' @references 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}
#'
#' Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
#'  
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.
#' 
#' Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 
#' 
#'     
#' @export


fit.mplus <- function(blocks,itf,model,input_path,output_path=NULL,data_path='myDataFile.dat',fscore_path='myFactorScores.dat',title='myFC_model',
                      ID=FALSE, byblock=TRUE,estimator='ULSMV',data_full=FALSE,standardized=TRUE,rename_list=NULL,...){
  if(is.null(input_path)){
    stop('You need to specify a Mplus input file in object input_path.')
  }
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- matrix(blocks,nrow=1)
  }
  if(is.null(output_path)){
    output_path <- sub('.inp','.out',input_path)
  }
  
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  syntax.mplus(blocks,itf,model,input_path,data_path,fscore_path,title,
               ID,byblock,estimator,data_full,standardized,rename_list)
  
  path <- paste0('mplus ',input_path)
  
  system(path, wait=TRUE,show.output.on.console=FALSE)
  
  if(model=='irt'|model=='uc'){
    results <- read.mplus(blocks,itf,model,output_path,intercept = F,...)
  }else if(model=='lmean'|model=='simple2'|model=='simple3'|model=='simple5'){
    results <- read.mplus(blocks,itf,model,output_path,threshold = F,loading=F,cor=F,...)
  }
  
  return(results)

}

get.xblocks.random <- function(nrank, blocks, item_not){
  warn <- 0
  
  # If some items should not be used first, separate blocks
  if(!is.null(item_not)){
    tmp <- apply(matrix(blocks%in%item_not,ncol=nrank),1,any)
    not_blocks <- matrix(blocks[tmp,],ncol=nrank)
    not_left <- split_matrix(not_blocks,1,nrank)
    blocks <- blocks[!tmp,]
  }
  
  # Init
  tmp_block <- matrix(nrow=0,ncol=nrank)
  if(length(blocks)>0){
    left <- split_matrix(blocks,1,nrank)
  }else{
    left <- list()
  }
  

  
  # First iteration of connecting
  while(length(left)>=nrank){
    tmp <- left[1:nrank]
    tmp_left <- unlist(tmp)
    left[1:nrank] <- NULL
    tmp_choose <- matrix(unlist(lapply(tmp, sample,size=1)),ncol=nrank,byrow = T)
    tmp_block <- rbind(tmp_block,tmp_choose)
    left[[(length(left)+1)]] <- tmp_left
  }
  
  # If some items should not be used first, 1. connect them
  if(!is.null(item_not)){
    while(length(not_left)>=nrank){
      tmp <- not_left[1:nrank]
      tmp_left <- unlist(tmp)
      not_left[1:nrank] <- NULL
      tmp_choose <- c()
      for(i in 1:length(tmp)){
        tmp2 <- which(!tmp[[i]]%in%item_not)
        tmptmp <- choose.item(tmp,tmp2,i,F,NULL, NULL,0)
        tmp3 <- tmptmp[[1]]
        tmp_choose <- c(tmp_choose,tmp3)
      }
      tmp_choose <- matrix(tmp_choose,ncol=nrank)
      tmp_block <- rbind(tmp_block,tmp_choose)
      not_left[[(length(not_left)+1)]] <- tmp_left
    }
    # 2. add them to the rest
    left <- c(not_left,left)
    
    # 3. connect blocks with as least different keyed blocks as possible
    while(length(left)>=nrank){
      tmp <- left[1:nrank]
      tmp_left <- unlist(tmp)
      left[1:nrank] <- NULL
      any_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),any))
      whole_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),all))
      tmp_choose <- c()
      if(any(whole_not)&all(any_not)){
          for(i in 1:length(tmp)){
            tmp2 <- which(!tmp[[i]]%in%item_not)
            tmptmp <- choose.item(tmp,tmp2,i,F,NULL, NULL,0)
            tmp3 <- tmptmp[[1]]
            tmp_choose <- c(tmp_choose,tmp3)
        }
      }else{
        for(i in 1:length(tmp)){
          tmp2 <- which(tmp[[i]]%in%item_not)
          tmptmp <- choose.item(tmp,tmp2,i,F,NULL, NULL,0)
          tmp3 <- tmptmp[[1]]
          tmp_choose <- c(tmp_choose,tmp3)
        }
      }
      tmp_block <- rbind(tmp_block,matrix(tmp_choose,ncol=nrank))
      left[[(length(left)+1)]] <- tmp_left
    }
  }
  
  # Last block
  if(length(left)!=1){
    tmp <- left
    tmp_choose <- c()
    missing <- nrank-length(left)
    # If some items should not be used first
    if(!is.null(item_not)){
      any_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),any))
      whole_not <- unlist(lapply(lapply(tmp, function(x) x%in%item_not),all))
      if(any(whole_not)==T){
        for(i in 1:length(left)){
          tmp3 <- sample(tmp[[i]],size=1)
          tmp_choose <- c(tmp_choose,tmp3)
        }
      }else{
        for(i in 1:length(left)){
          tmp2 <- which(tmp[[i]]%in%item_not)
          tmptmp <- choose.item(tmp,tmp2,i,F,NULL, NULL,0)
          tmp3 <- tmptmp[[1]]
          tmp_choose <- c(tmp_choose,tmp3)
        }
      }
      num_items <- rev(order(unlist(lapply(left,length))))
      for(i in 1:missing){
        j <- ifelse(i%%length(left)==0,length(left),i%%length(left))
        tmp2 <- which(left[[j]]%in%item_not)
        tmp_choose[length(left)+i] <- sample(unlist(left[num_items[j]])[-which(unlist(left[num_items[j]])%in%c(item_not,tmp_choose))], size=1)
      }
      tmp_choose <- matrix(tmp_choose,ncol=nrank)
      tmp_block <- rbind(tmp_block,tmp_choose)
      left <- unlist(left)
    }else{# else
      tmp_choose <- matrix(unlist(lapply(left, sample,size=1)),ncol=nrank-missing,byrow = T)
      tmp <- left
      num_items <- rev(order(unlist(lapply(left,length))))
      for(i in 1:missing){
        j <- ifelse(i%%length(left)==0,length(left),i%%length(left))
        tmp_choose[length(left)+i] <- sample(unlist(left[num_items[j]])[-which(unlist(left[num_items[j]])%in%c(tmp_choose))], size=1)
      }
      tmp_choose <- matrix(tmp_choose,ncol=nrank)
      tmp_block <- rbind(tmp_block,tmp_choose)
      left <- unlist(left)
    }
  }
  return(tmp_block)
}

### Roxygen-izable Documentation ----
#' Find all general blocks
#' 
#' This function creates meta interlinked blocks within a Thurstonian
#' design.
#' 
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' 
#' 
### Outputs ---- 
#' @return Returns a list of items that form meta interlinked blocks.
#'
### Details
#' @details This function creates meta interlinked blocks of a block design.
#' These are blocks, that have at least one link from each of its items to
#' any other of its items. If there is not such a link between every item,
#' there are at least two meta blocks.
#' 
### Examples ----
#' @examples
#' 
#' # Define 30 items divided by ten triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' # Add one block to link the first three blocks.
#' blocks <- rbind(blocks,c(1,2,3))
#' 
#' # Find mata blocks
#' metablock(blocks)
#'   
#' @export


metablock <- function(blocks){
  A <- designA(blocks)
  Arref <- rref(A)
  Arank <- dim(A)[1]-length(which(apply(abs(Arref),1,sum)==0))
  
  # Find global blocks
  tmp <- which(apply(Arref,2,sum)<0)
  gblock <- list()
  for(i in 1:length(tmp)){
    tmp0 <- which(Arref[,tmp[i]]==-1)
    if(is.null(dim(Arref[tmp0,]))){
      tmp0 <- which(Arref[tmp0,]==1)
    }else{
      tmp0 <- apply(Arref[tmp0,],1,function(x) which(x==1))
    }
    gblock[[i]] <- unname(c(tmp0,tmp[i]))
  }
  return(gblock)
}

eps <- function (x = 1){
  stopifnot(is.numeric(x))
  x <- max(abs(x))
  if (x < .Machine$double.xmin) {
    e <- .Machine$double.xmin
  }
  else {
    e <- 2^floor(log2(x)) * .Machine$double.eps
  }
  return(e)
}

### Roxygen-izable Documentation ----
#' Calculate reliability estimate for factor scores.
#' 
#' This function returns a reliability estimate for factor scores estimated via
#' the Thurstonian IRT models.
#' 
#' 
### Inputs ----
#' @param scores A matrix with factor scores. Rows are respondents, columns are
#' factors/traits.
#' @param scores_se A matrix with the standard errors of the factor scores. 
#' Rows are respondents, columns are factors/traits. Factors/traits must be in
#' the same order as for `scores`.
#' @param method Can be one of 'div' or 'sub'. See details. Defaults to 'sub'.
#'   
### Outputs ---- 
#' @return Returns the reliability value. If method = 'irt', the a plot with the
#' reliability depending on the factorscore is returned. 
#' 
#' @details The function returns the empirical reliability of factor scores. If
#' \eqn{\sigma^2} is the sample variance of the estimated scores and \eqn{\bar{sigma}^2_{error}}
#' is the average if the squared scores, that is
#' 
#' \deqn{\bar{sigma}^2_{error}=\frac{1}{N}\sum^N_i=1 se_{scores}^2}
#' 
#' then the subtraction method, a classical reliability estimate similar to
#' classical test theory is returned using 'sub' yields
#' \deqn{\frac{\sigma^2-\bar{sigma}^2_{error}}{\sigma^2}}
#' for the reliability of the scores. If 'div' is chosen, and alternative
#' division based approach is used.
#' \deqn{\frac{\sigma^2}{\sigma^2+\bar{sigma}^2_{error}}}.
#' 
#' If 'irt' is chosen, a plot returning the standard error of the scores with
#' the scores is returned per factor.
#' 
### Examples ----
#' @examples
#' 
#' # read and save data set FC
#' data(FC_scores)
#' 
#' # get reliability
#' reliabiltyFS(FC_scores[,c(106,108,110)],FC_scores[,c(107,109,111)])
#' 
#' @export

reliabiltyFS <- function(scores, scores_se, method = 'sub'){
  
  if(!is.matrix(scores)&!is.data.frame(scores)){
    scores <- matrix(scores)
    scores_se <- matrix(scores_se)
  }
  x <- apply(scores,2,stats::var)
  y <- apply(scores_se^2,2,mean)
  rels <- (x-y)/x
  relsa <- x/(x+y) # alternative (Brown & Maydeu-Olivares, 2018)
  
  names(rels) <- names(relsa) <- paste0('F',1:length(rels))
  if(method=='sub'){
    return(rels)
  }
  else if(method == 'div'){
    return(relsa)
  }
  else if(method == 'irt'){
    pl <- list()
    for(i in 1:dim(scores)[2]){
      pl[[i]] <- plot(scores[,i],scores_se[,i], xlab = expression(theta), ylab=paste0('standard error F',i))
    }
    return(pl)
  }
}

### Roxygen-izable Documentation ----
#' Determine the rank of the design matrix defined by the blocks.
#' 
#' This function determines the rank of the fundamental design matrix defined
#' by the blocks.
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' 
### Outputs ---- 
#' @return Returns the rank of the design matrix as an integer.
#' 
### Examples ----
#' @examples
#' 
#' # Define nine items divided by three triplets as blocks
#' blocks <- matrix(c(1:9), ncol = 3)
#' 
#' # Determine the rank of the design matrix
#' rankA(blocks)
#'     
#' @export


rankA <- function(blocks){
  A <- designA(blocks)
  Arref <- rref(A)
  Arank <- dim(A)[1]-length(which(apply(abs(Arref),1,sum)==0))
  return(Arank)
}

### Roxygen-izable Documentation ----
#' Create Mplus syntax for Thurstonian forced choice designs.
#' 
#' This function writes and saves the Mplus syntax given the specifications of a
#' Thurstonian forced choice design.
#' 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' @param input_path Path to save the Mplus input file. Defaults to `'myFC_model.inp'`.
#' @param data_path Path of the data file for Mplus. Defaults to `'myDataFile.dat'`.
#' @param fscore_path Path to save the file of factor scores. Defaults to
#' `'myFactorScores.dat'`.
#' @param title Title of the Mplus model. Defaults to `'myFC_model'`.
#' @param ID Logical. Should a ID variable be included? The ID must be the first 
#' variable in the data set. Defaults to `FALSE`.
#' @param byblock Logical. Should the order in Mplus variable statement be the same
#' as in the blocks. Defaults to `TRUE`.
#' @param estimator Which estimator should be used? All Estimators that are
#' available in Mplus can be used. Defaults to `'ULSMV'`.
#' @param data_full Logical. Are the data considered to be from a full design?
#' Defaults to `FALSE`.
#' @param standardized Logical. Should standardized values be computed? Defaults
#' to `TRUE`.
#' @param rename_list A list with two vectors to rename the objects in the syntax.
#' Vector one is the original names, vector two the new names. Defaults to `NULL`. 
#' 
### Outputs ---- 
#' @return Returns a description of the user-specified model. Typically, the
#' model is described using the lavaan model syntax. See `lavaan::model.syntax`
#' for more information. 
#' 
#' @details The syntax currently is able to perform model analysis for the
#' latent utility models (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
#' unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
#' the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
#' model types can be performed with all types of forced choice designs (full,
#' block, partially linked block, linked block). For an overview and review see
#' Jansen and Schulze (2023a,2023b).
#' 
#' The function writes and saves the Mplus input files.
#' 
#' The standard naming procedure ixiy, for the comparison of items x and y, 
#' can be changed by specifying the `rename_list` argument. The first vector of
#' the schould be the vector of original names, for example 
#' `c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
#' the second vector should contain the new names, for example 
#' `c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.
#' 
#' ### Outputs ---- 
#' @return Saves a list Mplus input file at 'input_path'.
#' 
### Examples ----
#' @examples
#' 
#' # read and save data set FC
#' data(FC)
#' write.table(FC,paste0(tempdir(),'/','my_data.dat'),quote=FALSE, sep=" ",
#' col.names = FALSE, row.names = FALSE)
#' 
#' # set seed and define blocks
#' set.seed(1)
#' blocks <- matrix(sample(1:15,15), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' # Create and save Mplus syntax
#' syntax.mplus(blocks,itf,'lmean',data_path = 'my_data.dat', data_full = TRUE,
#' input_path = paste0(tempdir(),'/','myFC_model'))
#' 
#' @references 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}
#'
#' Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
#'  
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.
#' 
#' Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 
#' 
#' @export

syntax.mplus <- function(blocks,itf,model,input_path='myFC_model.inp',data_path='myDataFile.dat',fscore_path='myFactorScores.dat',title='myFC_model',
                         ID=FALSE, byblock=TRUE,estimator='ULSMV',data_full=FALSE,standardized=TRUE,rename_list=NULL){
  if(is.null(input_path)){
    stop('You need to specify a Mplus input file in object input_path.')
  }
  if(model=='simple2'){
    model <- 'simple'
    case <- 2
  }
  if(model=='simple3'){
    model <- 'simple'
    case <- 3
  }
  if(model=='simple5'){
    model <- 'simple'
    case <- 5
  }
  
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- matrix(blocks,nrow=1)
  }
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  # Find number of factors, blocks and items
  nblock <- dim(blocks)[1]
  nitem <- length(unique(c(blocks)))
  nfactor <- max(itf)

  # Pair combinations
  pair_combn <- t(utils::combn(1:nitem,2))
  
  # Define items to factors for lambda (loadings)
  lambda_raw <- matrix(0,nrow=nitem,ncol=nfactor)
  for(i in 1:nfactor){
    lambda_raw[,i] <- ifelse(itf==i,i,0)
  }
  
  # Create names of pairs
  pair_names <- paste0('i',pair_combn[,1],'i',pair_combn[,2])
  
  # For each block: is it uni-dimensional?
  blocks_lambda <- matrix(itf[blocks],ncol=nrank)
  unidim <- apply(blocks_lambda,1,function(x) min(x)==max(x))
  
  # Create names of pairs for blocks
  pair_names_bs <- i.name(blocksort(blocks))
  pair_names_b <- i.name(blocks)
  
  tmp <- which(pair.combn(blocks)[,1]>pair.combn(blocks)[,2])
  
  if(length(tmp)!=0){
    tmp1 <- pair_names_b[tmp]
    tmp2 <- sub('^i.+i','i',tmp1)
    tmp3 <- tmp1
    for(j in 1:length(tmp)){
      tmp3 <- paste0(tmp2[j],sub(paste0(tmp2[j],'$'),'',tmp1[j]))
      pair_names_bs[which(pair_names_bs%in%tmp3)] <- pair_names_b[tmp[j]]
    }
  }
  
  pair_combn_b <- pair.combn(blocks)
  
  # Begin syntax
  # Init parameters for syntax (combine with init parameters above)
  title <- title
  estimator <- estimator
  
  # write mplus input file
  # TITLE
  input <- paste0('TITLE: ',title, '\n\n')
  
  # DATA
  input <- paste0(input, 'DATA: FILE IS ',data_path, ';\n\n')
  
  # VARIABLE
  input <- paste0(input, 'VARIABLE: NAMES ARE ', '\n')
  if(ID){
    input <- paste0(input, 'ID', '\n')
  }

  if(data_full){
    tmp <- which(!pair_names_b%in%pair_names)
    if(length(tmp)!=0){
      tmp1 <- pair_names_b[tmp]
      tmp2 <- sub('^i.+i','i',tmp1)
      for(i in 1:length(tmp)){
        tmp3 <- paste0(tmp2[i],sub(paste0(tmp2[i],'$'),'',tmp1[i]))
        pair_names[which(pair_names%in%tmp3)] <- pair_names_b[tmp[i]]
      }
    }
    
    input <- paste0(input, paste(pair_names,collapse = '\n'), ';\n')
  }else if(byblock){
    tmp <- apply(blocks,1, function(y) apply(t(utils::combn(y,2)),1,function(x) paste0('i',x[1],'i',x[2])))
    pair_names_bb <- paste0(tmp)
    input <- paste0(input, paste(pair_names_bb,collapse = '\n'), ';\n')
  }else{
    input <- paste0(input, paste(pair_names_bs,collapse = '\n'), ';\n')
  }
  
  if(design=='full'|!data_full){
    input <- paste0(input, '\n','USEVARIABLES ARE ALL',';\n')
  }else{
    input <- paste0(input, '\n','USEVARIABLES ARE ','\n')
    input <- paste0(input, paste(pair_names_b,collapse = '\n'), ';\n')
  }
  
  input <- paste0(input, 'CATEGORICAL ARE ALL;\n\n')
  
  if(ID){
    input <- paste0(input, 'IDVARIABLE IS ID;', '\n')
  }
  
  # ANALYSIS
  input <- paste0(input, 'ANALYSIS:\n', 'ESTIMATOR IS ', estimator, ';\n', 'ITERATIONS = 10000;\n','PARAMETERIZATION IS THETA;\n\n')
  
  # MODEL
  input <- paste0(input, 'MODEL:\n')
  if(model!='simple'){
    traits <- paste0('Trait',1:nfactor)
  }
  
  # Create Matrix A
  A <- designA(blocks)
  Arref <- rref(A)
  Arank <- dim(A)[1]-length(which(apply(abs(Arref),1,sum)==0))
  
  # Find global blocks
  gblocks <- metablock(blocks)
  unidim_gblocks <- sapply(gblocks,function(x) min(itf[x])==max(itf[x]))
  
  # Latent utility model and the unconstrained threshold model
  if(model%in%c('lmean','uc','simple')){
    # Part 1 (primary factors)
    input <- paste0(input, '! Model of primary factors:\n')
    t <- paste0('t',1:nitem)
    A_sign <- ifelse(A==0,'',ifelse(A==1,'+','-'))
    
    At <- matrix(NA, nrow=nrow(A_sign), ncol=nitem)
    At <- t(apply(A_sign,1,function(x)sub('\\+','',sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,t)))))))
    At <- sub('^-','',matrix(paste0(At,' BY ',pair_names_b,'@',ifelse(A_sign=='+','',A_sign),'1;\n'), ncol=nitem))
    At <- ifelse(A_sign=='','',At)
    At_text <- paste0(apply(At,2,function(x)paste0(x,collapse = '')), collapse = '')
    
    input <- paste0(input, paste(At_text, collapse='\n'), '\n\n')
    
    # For rankings: Fix variances of indicators to 0
    input <- paste0(input, '! Fix the variances of all indicators to 0:\n')
    input <- paste0(input, pair_names_b[1],'-',pair_names_b[length(pair_names_b)],'@0',';\n\n')
    #### Whats with paired comparisons?
    
    # Part 2 (secondary factors)
    if(model!='simple'){
      input <- paste0(input, '! Model of secondary factors:\n')
      if(nfactor==1&Arank==(nitem-1)){
        input <- paste0(input, 'Trait1 BY\nt1@1\n', paste0(paste0('t',2:nitem,'*1'),collapse='\n'), ';\n\n')
      }else{
        A_fac <- rep('',nfactor)
        
        itf_gblocks <- lapply(gblocks,function(x) itf[x])
        items_fac <- list()
        items_fac_uni <- list()
        for(i in 1:nfactor){
          item_fac <- c()
          item_fac_uni <- c()
          for(j in 1:length(gblocks)){
            if(unidim_gblocks[j]){
              item_fac_uni <- c(item_fac_uni,gblocks[[j]][which(itf_gblocks[[j]]==i)[1]])
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)[-1]])
            }else{
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)])
            }
          }
          items_fac[[i]] <- sort(item_fac)
          if(all(is.na(item_fac_uni))){
            items_fac_uni[[i]] <- NA
          }else{
            items_fac_uni[[i]] <- sort(item_fac_uni)
          }
          
          
          A_fac[[i]] <- paste0(traits[i], ' BY\n')
          A_fac[[i]] <- paste0(paste0(traits[i], ' BY\n'),paste0('t',unname(items_fac[[i]]),'*1', '\n', collapse = ''))
          
          #for(j in 1:length(gblocks)){
          #if(unidim_gblocks[j]){
          #if(length(items_fac_uni[[i]])>0&any(gblocks[[j]]%in%items_fac_uni[[i]])){
          if(all(!is.na(items_fac_uni[[i]]))){
            A_fac[[i]] <- paste0(A_fac[[i]],paste0('t',unname(items_fac_uni[[i]]),'@1', '\n', collapse = ''))
          }
          #}
          #}
          #}
        }
        input <- paste0(input, paste0(A_fac,';', collapse = '\n'),'\n\n')
      }
      # Fix variances of factors to 1
      input <- paste0(input, '! Fix variances of all factors to 1:\n')
      input <- paste0(input, paste0(traits[1],'-',traits[nfactor], '@1', collapse = ' '),';', '\n\n')
      
      # Correlations between factors
      input <- paste0(input, '! Define the correlations between factors:\n')
      if(nfactor>1){
        inp_tmp <- paste0('Trait',t(utils::combn(1:nfactor,2))[,1], ' WITH ', 'Trait', t(utils::combn(1:nfactor,2))[,2],';', collapse = '\n')
        input <- paste0(input, inp_tmp, '\n\n')
      }
      # Fix uniqueness of one item "per block"
      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      #####
      if(nrank==2){
        input <- paste0(input, '! Fix uniquenesses of all items to 1:\n')
        input <- paste0(input, paste0('t',1:nitem,collapse = '@1;\n'), '@1;\n')
      }else{
        input <- paste0(input, '!Fix uniquenesses of one item per meta block:\n')
        for(i in 1:length(gblocks)){
          tmp <- min(gblocks[[i]])
          input <- paste0(input,'t',tmp,'@1;\n')
        }
      }
    }

    # Fixing for the latent utility model
    if(model=='lmean'|model=='simple'){
      input <- paste0(input, '!Fix all thresholds to 0 to estimate means:\n')
      input <- paste0(input, '\n[',pair_names_b[1],'$1-',utils::tail(pair_names_b, n=1),'$1@0]',';\n\n')
      if(design=='full'){
        input <- paste0(input, '! Fix one mean to 0 and free all other means:\n')
        input <- paste0(input, '[t1@0]',';\n')
        input <- paste0(input, '[t2-t',nitem,']',';\n\n')
      }
      if(design=='block'){
        input <- paste0(input, '! Fix one mean per grand block to 0 and free all other means:\n')
        input <- paste0(input, '[t1-t',nitem,']',';\n')
        # This is not necessary but maybe more helpful for application. Fix the lowest item per block
        #####
        for(i in 1:length(gblocks)){
          tmp <- min(gblocks[[i]])
          input <- paste0(input,'\n[t',tmp,'@0];')
        }
      }
    }
    
    # Additional Fixing for the Thurstone cases
    if(model=='simple'){
      if(case==5){
        input <- paste0(input,'\n','! fix factor variances to 1\n', 't1-',paste0('t',nitem),'@1;\n\n')
        
        input <- paste0(input, '! fix all covariances between factors to zero:\n')
        for(i in 1:(nitem-2)){
          input <- paste0(input,'\n',paste0('t',i),' WITH ',paste0('t',i+1),'-',paste0('t',nitem),'@0;')
        }
        input <- paste0(input,'\n',paste0('t',nitem-1),' WITH ',paste0('t',nitem),'@0;')
      }
      if(case==3){
        input <- paste0(input,'\n','! free factor variances except the last one\n', 't1-',paste0('t',nitem-1),'*;\n',paste0('t',nitem),'@1;\n\n')
        
        input <- paste0(input, '! fix all covariances between factors to zero:\n')
        for(i in 1:(nitem-2)){
          input <- paste0(input,'\n',paste0('t',i),' WITH ',paste0('t',i+1),'-',paste0('t',nitem),'@0;')
        }
        input <- paste0(input,'\n',paste0('t',nitem-1),' WITH ',paste0('t',nitem),'@0;')
      }
      if(case==2){
        input <- paste0(input,'\n','! free factor variances except the first and last one\n', 't1@1;\nt2-',paste0('t',nitem-1),'*;\n',paste0('t',nitem),'@1;\n\n')
        
        input <- paste0(input, '! all Factor covariances are free except these involving the last factor:\n')
        for(i in 1:(nitem-3)){
          input <- paste0(input,'\n',paste0('t',i),' WITH ',paste0('t',i+1),'-',paste0('t',nitem-1),'*;')
        }
        input <- paste0(input,'\n',paste0('t',nitem-2),' WITH ',paste0('t',nitem-1),'*;')
        input <- paste0(input,'\n',paste0('t',nitem),' WITH ',paste0('t1-t',nitem-1),'@0;')
      }
    }
    
    if(standardized){
      input <- paste0(input, '\n\nOUTPUT: STDYX;\n\n')
    }
  }
  
  # IRT model
  if(model%in%c('irt')){
    # Create matrix lambda
    lambda <- ifelse(lambda_raw==0,0,paste0('L',row(lambda_raw)))
    A_sign <- ifelse(A==0,0,ifelse(A==1,'+','_'))
    A_lambda <- A%*%ifelse(lambda_raw==0,0,1)
    Lambda <- matrix(NA, nrow=nrow(A_sign), ncol=ncol(lambda))
    
    for(i in 1:nfactor){
      Lambda[,i] <- apply(A_sign,1,function(x) paste0(sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,lambda[,i])))), collapse =''))
    }
    tmp <- grep('^_L.+\\+',Lambda)
    tmp1 <- Lambda[tmp]
    Lambda[tmp] <- paste0(sub('^_L.+\\+','',tmp1),sub('\\+L.+','',tmp1))
    Lambda <- sub('\\+','',Lambda)
    Lambda <- sub('^_L','L_',Lambda)
    
    A_mplus <- NULL
    for(i in 1:nfactor){
      A_mplus <- paste0(A_mplus,'\n\n',traits[i],' BY \n',
                        paste0(ifelse(Lambda[,i]=='','',
                                      paste0(pair_names_b,'*',A_lambda[,i],' (',Lambda[,i],')\n')),collapse = ''),';')
    }
    
    # Fix variances of factors to 1
    input <- paste0(input, A_mplus,'\n\n', '! Fix variances of all factors to 1:\n')
    input <- paste0(input, paste0(traits[1],'-',traits[nfactor], '@1', collapse = ' '),';', '\n\n')
    
    
    # Correlations between factors
    input <- paste0(input, '! Define the correlations between factors:\n')
    if(nfactor>1){
      inp_tmp <- paste0('Trait',t(utils::combn(1:nfactor,2))[,1], ' WITH ', 'Trait', t(utils::combn(1:nfactor,2))[,2],';', collapse = '\n')
      input <- paste0(input, inp_tmp, '\n\n')
    }
    
    if(nrank==2){
      input <- paste0(input, '! Fix uniquenesses of all indicators to 1:\n')
      input <- paste0(input, paste(pair_names_b,collapse = '@2;\n'), '@2;\n')
      
      # Create Matrix Psi for irt models
      input <- paste0(input, '\n\n! Define structured uniquenesses:\n')
      Psi <- ifelse(row(diag(nitem))==col(diag(nitem)),paste0('e',1:nitem),0)
      Psi <- ifelse(Psi==0,'',Psi) # neu
      A_sign <- ifelse(A==0,0,ifelse(A==1,'+','-')) # alt
      
      
      APsi <- matrix(NA, nrow=length(pair_names_b), ncol=nitem)
      for(i in 1:nitem){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[j,],Psi[,i])
          tmp <- sub('^-$','',sub('^\\+$','',tmp))
          tmp <- sub('^-0.*','',sub('^\\+0.*','',sub('^0.*','',tmp)))
          APsi[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsi <- sub('^\\+','',APsi)
      
      APsiA <- matrix(NA, nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:length(pair_names_b)){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[i,],APsi[j,])
          #tmp <- sub('\\+','',sub('^0.*','',tmp))
          tmp <- sub('^0.*','',tmp)
          APsiA[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsiA <- ifelse(APsiA=='+-','',ifelse(APsiA=='-+','',APsiA))
      APsiA <- gsub('-\\+','',APsiA)
      APsiA <- gsub('\\+-','-',APsiA)
      APsiA <- gsub('--','+',APsiA)
      
      
      APsiA <- sub('-$','',APsiA)#
      APsiA <- sub('\\+$','',APsiA)#
      APsiA <- sub('^\\+','',APsiA)
      
      
      APsiA_under  <- sub('-e','e_',APsiA)
      APsiA_sign <- ifelse(APsiA=='','',ifelse(substr(APsiA,1,1)=='-','-',''))
      
      
      # Correlations in Psi
      APsiA_text <- matrix('', nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:(nrow(APsiA)-1)){
        for(j in (i+1):nrow(APsiA)){
          APsiA_text[j,i] <- paste0(pair_names_b[i],' WITH ', pair_names_b[j], '@',APsiA_sign[j,i],'1;')
        }
      }
      APsiA_text <- ifelse(APsiA=='','',APsiA_text)
      APsiA_text <- paste0(ifelse(APsiA_text=='','',paste0(APsiA_text,'\n')), collapse = '')
      
      input <- paste0(input,'\n\n',APsiA_text,'\n\n')
      
      if(length(gblocks)!=nblock){
        # Model constraints
        input <- paste0(input, '! Set model constraints for loadings:\n')
        input <- paste0(input, '\n','MODEL CONSTRAINT:\nNEW\n')
        
        # Define new parameters
        unidim_item <- Lambda[grep('^L.*_L',Lambda)]
        tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
        if(length(which(tmp%in%Lambda))!=0){
          tmp <- tmp[-which(tmp%in%Lambda)]
        }
        
        AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
        
        tmp <- c(AddL,tmp)
        Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
        
        Add_unidim <- NULL
        for(i in 1:nrow(blocks)){
          tmp <- blocks[i,]%in%sub('L','',Add_param)
          if(all(tmp)){
            Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
          }
        }
        
        # Constraints
        conL <- rep('',nitem)
        for(i in 1:nitem){
          if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
            if(paste0('L_',i)%in%Lambda){
              conL[i] <- paste0('L_',i,' = ','- L',i)
            }
          }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
            conL[i] <- paste0('L_',i,' = ','- L',i)
          }
        }
        
        Add_param <- paste0(paste0(Add_param, collapse = '\n'),';')
        unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' = ', sub('_','-',unidim_item),';\n',collapse = ''))
        conL <- paste0(ifelse(conL=='','',paste0(conL,';\n')), collapse = '')
        
        input <- paste0(input,Add_param,'\n\n',unidim_item,'\n',conL,'\n\n')
        
        # Fix one loading in each unidim block if this block is not connected with blocks of another factor or of already fixed blocks
        # This is not necessary but maybe more helpful for application. Fix the lowest item per block
        input <- paste0(input, '! Fix one loading per uni-dimensional grand block:\n')
        for(i in 1:length(gblocks)){
          if(unidim_gblocks[i]){
            input <- paste0(input, paste0('L',min(gblocks[[i]]),'=1;\n', '\n'))
          }
        }
        
        input <- paste0(input,'\n')
      }
    }
    if(nrank>2){
      # Declare uniqunesses
      input <- paste0(input, '! Declare uniquenesses:\n')
      input <- paste0(input,paste0(pair_names_b,'*2 ','(e',pair_combn_b[,1],'e',pair_combn_b[,2],');', collapse = '\n'))
      
      # Create Matrix Psi for irt models
      input <- paste0(input, '\n\n! Define structured uniquenesses:\n')
      Psi <- ifelse(row(diag(nitem))==col(diag(nitem)),paste0('e',1:nitem),0)
      Psi <- ifelse(Psi==0,'',Psi) # neu
      A_sign <- ifelse(A==0,0,ifelse(A==1,'+','-')) # alt

      
      APsi <- matrix(NA, nrow=length(pair_names_b), ncol=nitem)
      for(i in 1:nitem){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[j,],Psi[,i])
          tmp <- sub('^-$','',sub('^\\+$','',tmp))
          tmp <- sub('^-0.*','',sub('^\\+0.*','',sub('^0.*','',tmp)))
          APsi[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsi <- sub('^\\+','',APsi)
      
      APsiA <- matrix(NA, nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:length(pair_names_b)){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[i,],APsi[j,])
          #tmp <- sub('\\+','',sub('^0.*','',tmp))
          tmp <- sub('^0.*','',tmp)
          APsiA[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsiA <- ifelse(APsiA=='+-','',ifelse(APsiA=='-+','',APsiA))
      APsiA <- gsub('-\\+','',APsiA)
      APsiA <- gsub('\\+-','-',APsiA)
      APsiA <- gsub('--','+',APsiA)

      
      APsiA <- sub('-$','',APsiA)#
      APsiA <- sub('\\+$','',APsiA)#
      APsiA <- sub('^\\+','',APsiA)
      
      
      APsiA_under  <- sub('-e','e_',APsiA)
      APsiA_sign <- ifelse(APsiA=='','',ifelse(substr(APsiA,1,1)=='-','-',''))
      
      
      # Correlations in Psi
      APsiA_text <- matrix('', nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:(nrow(APsiA)-1)){
        for(j in (i+1):nrow(APsiA)){
          APsiA_text[j,i] <- paste0(pair_names_b[i],' WITH ', pair_names_b[j], '*',APsiA_sign[j,i],'1 (',
                                    APsiA_under[j,i],');')
        }
      }
      APsiA_text <- ifelse(APsiA=='','',APsiA_text)
      APsiA_text <- paste0(ifelse(APsiA_text=='','',paste0(APsiA_text,'\n')), collapse = '')
      
      input <- paste0(input,'\n\n',APsiA_text,'\n\n')
      
      # Model constraints
      input <- paste0(input, '! Set model constraints for loadings and uniquenesses:\n')
      input <- paste0(input, '\n','MODEL CONSTRAINT:\nNEW\n')
      
      # Define new parameters
      unidim_item <- Lambda[grep('^L.*_L',Lambda)]
      tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
      if(length(which(tmp%in%Lambda))!=0){
        tmp <- tmp[-which(tmp%in%Lambda)]
      }
      
      AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
      
      tmp <- c(AddL,tmp)
      Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
      
      Add_unidim <- NULL
      for(i in 1:nrow(blocks)){
        tmp <- blocks[i,]%in%sub('L','',Add_param)
        if(all(tmp)){
          Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
        }
      }
      
      # Constraints
      conL <- rep('',nitem)
      for(i in 1:nitem){
        if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
          if(paste0('L_',i)%in%Lambda){
            conL[i] <- paste0('L_',i,' = ','- L',i)
          }
        }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
          conL[i] <- paste0('L_',i,' = ','- L',i)
        }
      }
      
      Add_param <- paste0(paste0(Add_param, collapse = '\n'),';')
      unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' = ', sub('_','-',unidim_item),';\n',collapse = ''))
      conL <- paste0(ifelse(conL=='','',paste0(conL,';\n')), collapse = '')
      
      input <- paste0(input,Add_param,'\n\n',unidim_item,'\n',conL,'\n\n')
      
      
      # Fix one loading in each unidim block if this block is not connected with blocks of another factor or of already fixed blocks
      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      input <- paste0(input, '! Fix one loading per uni-dimensional grand block:\n')
      for(i in 1:length(gblocks)){
        if(unidim_gblocks[i]){
          input <- paste0(input, paste0('L',min(gblocks[[i]]),'=1;\n', '\n'))
        }
      }
      
      input <- paste0(input,'\n')
      
      # Uniquenesses relating to the same item are equal
      input <- paste0(input, '! Set uniquenesses relation to the same item as equal:\n')
      tmp <- rep(NA,nitem)
      for(i in 1:nitem){
        tmp[i] <- paste0('e',i)%in%APsiA[lower.tri(APsiA)]&paste0('-e',i)%in%APsiA[lower.tri(APsiA)]
        if(tmp[i]){
          input <- paste0(input,paste0('e_',i),' = ',paste0('- e',i),';\n')
        }
      }
      
      input <- paste0(input,'\n')
      
      # Pair's uniqueness is equal to sum of two utility uniquenesses
      input <- paste0(input, '! A pairs uniqueness is equal to the sum of the items uniquenesses:\n')
      if(design=='full'){
        input <- paste0(input,paste0('e',pair_combn[,1],'e',pair_combn[,2],' = e',pair_combn[,1],' + e',pair_combn[,2],';', collapse = '\n'),'\n\n')
      }else if(design=='block'){
        uniquenesses <- c()
        for(i in 1:nrow(blocks)){
          items <- blocks[i,]
          whitems <- which(pair_combn_b[,1]%in%items&pair_combn_b[,2]%in%items)
          tmp <- pair_combn_b[whitems,1][which(pair_combn_b[whitems,1]%in%pair_combn_b[whitems,2])]
          
          tmp <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2],' = ' ,ifelse(pair_combn_b[whitems,1]%in%tmp,'-e_','e'),pair_combn_b[whitems,1],
                        ifelse(pair_combn_b[whitems,2]%in%tmp,' - e_',' + e'),pair_combn_b[whitems,2],';')
          tmp0 <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2])
          for(i in length(tmp0):1){
            if(any(grepl(tmp0[i],uniquenesses))){
              tmp <- tmp[-i]
            }
          }
          uniquenesses <- c(uniquenesses,tmp)
          ##sapply(paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2]),function(x) grepl(x,uniquenesses))
        }
        uniquenesses <- unique(uniquenesses[order(nchar(uniquenesses),uniquenesses)])
        input <- paste0(input,paste0(uniquenesses, collapse = '\n'),'\n', collapse = '\n')
      }

      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      for(i in 1:length(gblocks)){
        tmp2 <- min(gblocks[[i]])
        tmp3 <- max(gblocks[[i]])
        if(paste0('e',tmp2)%in%APsiA_under[lower.tri(APsiA)]){
          input <- paste0(input,'\n','e',tmp2,' = 1;')
        }else{
          input <- paste0(input,'\n','e_',tmp2,' = -1;')
        }
        
      }
    }
    
    if(standardized){
      input <- paste0(input, '\n\nOUTPUT: STDYX;')
    }
    input <- paste0(input, "\n\nSAVEDATA:\nFILE IS ",fscore_path,";\nSAVE = FSCORES;\nFORMAT IS free;\n")
  }
  if(!is.null(rename_list)){
    tmp <- ceiling(log10(length(rename_list[[1]])))
    elem <- c()
    for(i in 1:tmp){
      tmp0 <- paste0('el3m3nt',paste0(rep('0',tmp-i),collapse = ''),(10^(i-1)):((10^i)-1))
      elem <- c(elem,tmp0)
    }
    elem <- elem[1:length(rename_list[[1]])]
    
    for(i in 1:length(rename_list[[1]])){
      input <- gsub(rename_list[[1]][i],elem[i],input)
    }
    for(i in 1:length(rename_list[[1]])){
      input <- gsub(elem[i],rename_list[[2]][i],input)
    }
  }
  cat(input,file=input_path)
}

### Roxygen-inzable documentation
#' Raw ranking data of $N=15$ items from three factors/traits (Thurstonian modeling)
#'
#' This data set contains synthetic raw data of 1000 participants on a ranking
#' task on 15 items. 
#'
#' @name FC_raw
#' 
#' @docType data
#' 
#' @keywords datasets
#'
#' @usage data(FC_raw)
#'
#' @format A data frame with 1000 observations on 15 variables.
#data(FC)
#FC_raw <- t(sapply(1:1000, function(y) pord(FC[y,],1:15)))
#colnames(FC_raw) <- paste0('item',1:15)
#save(FC_raw, file="data/FC_raw.rda")
"FC_raw"

### Roxygen-izable Documentation ----
#' Reads results from Mplus output file.
#' 
#' This function reads and returns results from an Mplus output file.
#' 
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' @param output_path Path to the Mplus output file. Defaults to `'myFC_model.out'`.
#' @param convergence Logical. Should a message for convergence be returned?
#' Defaults to `TRUE`.
#' @param fit.stat Logical. Should fit statistics be returned? Defaults to `TRUE`.
#' @param loading Logical. Should loading estimates be returned? Defaults to `TRUE`.
#' @param cor Logical. Should latent correlation estimates be returned?
#' Defaults to `TRUE`.
#' @param intercept Logical. Should intercepts be returned? Does only work for
#' `model = 'lmean'`. Defaults to `TRUE`.
#' @param threshold Logical. Should thresholds be returned? Does only work for
#' `model = 'uc'` or `'irt'`. Defaults to `TRUE`.
#' @param resvar Logical. Should residual variances be returned? Defaults to `TRUE`.
#' @param standardized Logical. Should standardized values be returned? Defaults
#' to `FALSE`.
#' 
### Outputs ---- 
#' @return Returns a list containing the specified results, after model analysis, by reading
#' the results from the 'output_path'.
#' 
### Examples ----
#' @examples
#' 
#' # read and save data set FC
#' data(FC)
#' 
#' write.table(FC,paste0(tempdir(),'/','my_data.dat'),quote=FALSE, sep=" ",
#' col.names = FALSE, row.names = FALSE)
#' 
#' 
#' # set seed and define blocks
#' set.seed(1)
#' blocks <- matrix(sample(1:15,15), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' # perform analysis 
#' \dontrun{
#' fit.mplus(blocksort(blocks),itf,'irt',data_path = 'mydata.dat', data_full = TRUE,
#' input_path = paste0(tempdir(),'/','myFC_model'))
#' 
#' 
#' # After estimation
#' read.mplus(blocks,itf,'irt',output_path = paste0(tempdir(),'/','myFC_model.out'))
#' }
#' @export


### Function definition ----
read.mplus <- function(blocks,itf,model,output_path,convergence=TRUE,fit.stat=TRUE,loading=TRUE, cor=TRUE, intercept=TRUE, threshold=TRUE, resvar=TRUE,
                       standardized=FALSE){
  
  if(is.null(output_path)){
    stop('You need to specify a Mplus output file in object output_path.')
  }
  
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- matrix(blocks,nrow=1)
  }
  
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  # Find number of factors, blocks and items
  nblock <- dim(blocks)[1]
  nitem <- length(unique(c(blocks)))
  nfactor <- max(itf)
  
  
  results <- list()
  MPoutput <- readLines(output_path)
  #MPoutput <- readLines('myFC_model3.out')
  mod <- mod.matrices(blocks,itf,model)
  if(convergence){
    if (!any(grepl('MODEL FIT',MPoutput))) {
      if (any(grepl('NO CONVERGENCE.',MPoutput))|any(grepl('THE MODEL MAY NOT BE IDENTIFIED.',MPoutput))) {
        convergence <- 0
      }else{
        convergence <- 0
      }}else {
        convergence <- 1
      }
    results$converge <- convergence
  }
  
  if(fit.stat){
    locator <- c('^SRMR',rep('^CFI/TLI',2),rep('^Chi-Square Test of Model Fit$',3),
                 rep('^Chi-Square Test of Model Fit for the Baseline Model',3),rep('^RMSEA',2))
    offset <- c(2,2,3,2,3,4,2,3,4,2,4)
    name <- c('srmr','cfi','tli','chisq','df','pvalue','blchisq','bldf','blpvalue','rmsea','rmsea_p')
    results$fit <- valuesMPout.fit(locator,offset,name,TRUE,MPoutput,convergence)
  }
  
  if(loading){
    lmloc <- c()
    lmoff <- c()
    lmname <- c()
    if(model=='irt'){
      mod$lambda <- ifelse(mod$lambda=='',NA,mod$lambda)
      lmlength <- dim(mod$lambda)[1]-matrixStats::colCounts(mod$lambda,value=NA)
      for(i in 1:nfactor){
        lmoff <- c(lmoff,1:lmlength[i])
        lmloc <- c(lmloc,rep(paste0('^ TRAIT',i,'\\s+BY'),lmlength[i]))
        #tmp <- mod$lambda[-which(is.na(mod$lambda[,i])),i]
        tmp <- mod$lambda[,i]
        
        #mod$lambda[ifelse(length(which(is.na(mod$lambda[,i])))==0,NA,-which(is.na(mod$lambda[,i]))),i]
        
        lmname <- c(lmname,tmp)
      }
      if(!standardized){
        lmloc <- c(lmloc,rep('^New/Additional Parameters',mod$naddparam))
        lmoff <- c(lmoff,1:mod$naddparam)
        lmname <- c(lmname,unlist(mod$addparam))
        if(length(which(is.na(lmname)))!=0){
          lmname <- lmname[-which(is.na(lmname))]
        }
      }
    }else if(model=='lmean'|model=='uc'){
      for(i in 1:nfactor){
        lmoff <- c(lmoff,1:(length(mod$lambda[[i]])-1))
        lmloc <- c(lmloc,rep(paste0('^ TRAIT',i,'\\s+BY'),length(mod$lambda[[i]])-1))
        tmp <- sub('\\*.','',mod$lambda[[i]][-1])
        lmname <- c(lmname,tmp)
      }
    }
    results$loadings <- valuesMPout(lmloc,lmoff,lmname,MPoutput,convergence, standardized)
  }
  
  if(cor&nfactor>1){
    corloc <- c()
    coroff <- c()
    for(i in 1:(nfactor-1)){
      tmp <- rep(paste0('^ TRAIT',i,'\\s+WITH'),nfactor-i)
      corloc <- c(corloc,tmp)
      coroff <- c(coroff,1:(nfactor-i))
    }
    corname <- paste0('TRAIT',utils::combn(1:nfactor,2)[1,],'TRAIT',utils::combn(1:nfactor,2)[2,])
    results$cor <- valuesMPout(corloc,coroff,corname,MPoutput,convergence, standardized)
  }
  
  
  if(model=='simple2'|model=='simple3'|model=='simple5'){
    corloc <- c()
    coroff <- c()
    for(i in 1:(nitem-2)){
      tmp <- rep(paste0('^ T',i,'\\s+WITH'),nitem-i)
      corloc <- c(corloc,tmp)
      coroff <- c(coroff,1:(nitem-i))
    }
    if(model=='simple2'){
      tmp <- rep(paste0('^ T',nitem,'\\s+WITH'),1)
    }else{
      tmp <- rep(paste0('^ T',nitem-1,'\\s+WITH'),1)
    }
    
    corloc <- c(corloc,tmp)
    coroff <- c(coroff,1)
    
    corname <- paste0('T',utils::combn(1:nitem,2)[1,],'T',utils::combn(1:nitem,2)[2,])
    results$cor <- valuesMPout(corloc,coroff,corname,MPoutput,convergence, standardized)
  }
  
  
  if(intercept){
    if(model!='lmean'&model!='simple2'&model!='simple3'&model!='simple5'){
      warning('Intercepts can only be retrieved from a latent mean model!')
    }
    if(model=='lmean'){
      intloc <- c(rep('^ Intercepts',nitem))
    }else{
      intloc <- c(rep('^ Means',nitem))
    }
    intoff <- c(1:nitem)
    intname <- paste0('T',c(1:nitem))
    results$int <- valuesMPout(intloc,intoff,intname,MPoutput,convergence, standardized)
  }
  
  if(threshold){
    threshloc <- c(rep('^ Thresholds',mod$npair))
    threshoff <- c(1:mod$npair)
    threshname <- paste0(mod$pair_names_b,'$1')#paste0("I$", c(1:mod$npair))
    results$thres <- valuesMPout(threshloc,threshoff,threshname,MPoutput,convergence, standardized)
  }
  
  if(resvar){
    if(model=='lmean'|model=='uc'){
      varloc <- c(rep('^ Residual Variances',(mod$npair+nitem)))
      varoff <- c(1:(mod$npair+nitem))
      varname <- c(mod$pair_names_b,paste0('T$',c(1:nitem)))
    } else if(model=='irt'){
      varloc <- c(rep('^ Residual Variances',mod$npair))
      varoff <- c(1:mod$npair)
      varname <- mod$pair_names_b#paste0("I$", c(1:mod$npair))
    }else if(model=='simple2'|model=='simple3'|model=='simple5'){
      varloc <- c(rep('^ Variances',(nitem)))
      varloc <- c(varloc,rep('^ Residual Variances',(mod$npair)))
      varoff <- c(1:(nitem))
      varoff <- c(varoff,1:(mod$npair))
      varname <- c(paste0('T$',c(1:nitem)),mod$pair_names_b)
    }
    results$resvar <- valuesMPout(varloc,varoff,varname,MPoutput,convergence, standardized)
  }
  
  if(model=='irt'){
    if (dim(blocks)[2] == 2) {
      uniq <- replicate(max(blocks), c(2, 0, 999, 999), simplify = FALSE)
      names(uniq) <- paste0("e", 1:max(blocks))
    }else{
      # Uniquenesses
      APsiA <- mod$APsiA
      APsiA[upper.tri(APsiA,diag=TRUE)] <- ''
      off <- apply(APsiA,2,function(x) mod$npair-length(which(x=="")))
      loc <- mod$pair_names_b
      
      tmp <- c()
      tmpo <- c()
      for(i in 1:length(mod$pair_names_b)){
        tmp <- c(tmp,rep(loc[i],off[i]))
        if(off[i]!=0){
          tmpo <- c(tmpo,1:off[i])
        }
      }
      
      loc <- gsub('i','I',paste0('^ ',tmp))
      offset <- tmpo
      APsiA_under  <- sub('-e','e_',APsiA)
      APsiA_under <- APsiA_under[lower.tri(APsiA_under)]
      APsiA_under <- ifelse(APsiA_under=='',NA,APsiA_under)
      name <- APsiA_under[!is.na(APsiA_under)]
      
      uniq <- valuesMPout(loc, offset, name, MPoutput,convergence, standardized)
      
      
      uniq <- uniq[-which(names(uniq)%in%names(which(table(gsub('_','',names(uniq)))>1)))]
      index <- grep('_',names(uniq))
      uniq[index] <- lapply(index, function(x) c(-1*uniq[[x]][1],uniq[[x]][2],-1*uniq[[x]][3],uniq[[x]][4]))
      
      names(uniq) <- gsub('_','',names(uniq))
    }
    results$uniq <- uniq
  }
  
  return(results)
}

valuesMPout.fit <- function(locator,offset,name,rmsea_conf,MPoutput,convergence){
  output <- list()
  for (i in 1:(length(locator))) {
    tmp <- gsub('[*]','',gsub('\\s+','',gsub(' <= .05','',gsub('[A-Za-z-]+','',MPoutput[grep(locator[i],MPoutput)+offset[i]]))))
    if(convergence==1){
      output[[name[i]]] <- as.numeric(tmp)
    }else{
      output[[name[i]]] <- NA
    }
  }
  if(rmsea_conf&convergence==1){
    tmp <- strsplit(gsub('\\s+',' ',MPoutput[grep('^RMSEA',MPoutput)+3]),' ')[[1]]
    output['rmsea.ci.lower'] <- as.numeric(tmp[5])
    output['rmsea.ci.upper'] <- as.numeric(tmp[6])
  }else{
    output['rmsea.ci.lower'] <- NA
    output['rmsea.ci.upper'] <- NA
  }

  return(output)
}

### Roxygen-izable Documentation ----
#' Create the Thurstonian design matrix for paired comparison and ranking data.
#' 
#' Creates the Thurstonian design matrix for paired comparison and ranking data,
#' given by blocks or the number of items.
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param nitems The number of items that are included in the design. 
#' 
### Outputs ---- 
#' @return Returns a design matrix which includes all paired comparisons
#' derivable from the blocks.
#' 
### Details
#' @details Each Thurstonian design can be defined by blocks of at least two
#' items. The function determines the fundamental design matrix A of the
#' Thurstonian design, including all possible paired comparisons that can be
#' derived by the blocks. For further information of the importance of the
#' design matrix, see Jansen and Schulze (2023a,2023b).
#' 
### Examples ----
#' @examples
#' 
#' # Define a matrix of blocks
#' blocks <- matrix(1:15,ncol=3, byrow=TRUE)
#' 
#' # Get the design matrix
#' loading_Matrix <- designA(blocks)
#' 
#' @references 
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Manuscript submitted.
#' 
#' Jansen, M. T., & Schulze, R. (2023b). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
#' 
#' @export


### Function definition ----

designA <- function(blocks=NULL, nitems=max(unique(blocks))){
  if(is.null(blocks)&nitems<2){
    stop('If blocks is not defined, nitems must be defined.')
  }
  if(nitems>1&is.null(blocks)){
    pairs <- t(utils::combn(1:nitems,2))
  }else{
    pairs <- pair.combn(blocks)
  }
  
  A_tmp <- c()
  for(i in 1:nrow(pairs)){
    tmp <- rep(0,nitems)
    tmp[pairs[i,1]] <- 1
    tmp[pairs[i,2]] <- -1
    A_tmp <- rbind(A_tmp,unname(tmp))
  }
  return(A_tmp)
}

### Roxygen-izable Documentation ----
#' Get extra blocks in a Thurstonian design, that links all blocks.
#' 
#' The function creates extra blocks for a Thurstonian design, that links all
#' initial blocks with as few extra blocks as possible. The number of extra
#' blocks is determined by `count.xblocks` (see Jansen & Schulze, 2023). 
#' 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
##' @param multidim Logical. Should the items within each linking block be
##' forced to be multidimensional?
#' @param item_not The items that are differently keyed compared to the majority
#' of items.
#' @param min Logical. Should a minimal number of blocks contain mixed keyed items?
#' @param show.warnings Logical. Should warnings be shown?
#' 
### Outputs ---- 
#' @return The result is a matrix where the rows correspond to the specific
#' extra blocks.
#' 
### Examples ----
#' @examples
#' 
#' # Define a matrix of blocks
#' blocks <- matrix(1:15,ncol=3, byrow=TRUE)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#'  
#' # Get the extra blocks for a completely linked design
#' get.xblocks(blocks, itf, FALSE)
#' 
#' @references
#' Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
#'      
#' @export


### Function definition ----

get.xblocks <- function(blocks, itf, multidim, item_not=NULL,min=FALSE,show.warnings=FALSE){
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  if(nrow(blocks)<nrank){
    if(nrank%in%c(2,3,4)){
      number <- c('two','three','four')
      name <- c('dyads','triplets','quads')
      stop(paste0('For ', name[nrank-1],' at least ' , number[nrank-1],' ', name[nrank-1],' are needed!'))
    }else{
      stop('Not enough blocks. Please use more.')
    }
  }
  if(multidim){
    if(length(unique(itf))<nrank){
      stop('If multidimensional blocks should be created, you need at least as many factors as items per block.')
    }
  }
  nrank <- ncol(blocks)
  if(multidim){
    tmp_blocks <- get.xblocks.multidim(nrank,blocks,itf,item_not,min,show.warnings)
  }else{
    tmp_blocks <- get.xblocks.random(nrank,blocks,item_not)
  }
  return(tmp_blocks)
}

### Roxygen-inzable documentation
#' Paired comparisons of $N=15$ items from one factor/trait (Thurstonian modeling)
#'
#' This data set contains synthetic data of 1000 participants on all binary
#' indicators of 15 items. For each paired comparison, participants had to rank
#' the two alternative items according to their preference. It is assumed that 
#' transitivity holds (that is, the data comes from a ranking task). More
#' details can be found in Brown and Maydeu-Olivares (2011), Jansen and Schulze
#' (2023) and Maydeu-Olivares and Böckenholt (2005).
#'
#' @name FC
#' 
#' @docType data
#' 
#' @keywords datasets
#'
#' @usage data(FC)
#'
#' @format A data frame with 1000 observations on 105 variables. For a variable
#' ixiy, the result is the response preferences between item x and item y. It is
#' coded a 1, if item x is preferred over item y, and 0 otherwise.
#' 
#'
#' @references 
#' Brown, A, & Maydeu-Olivares, A. (2011). Item response modeling of forced-choice questionnaires. \emph{Educational and Psychological Measurement}, \emph{71}(3), 460-502. \doi{10.1177/0013164410375112}
#' 
#' Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
#' 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}.
#'
#set.seed(69)
#FC <- sim.data(3,15,1000,rep(1:3,5),lmu=runif(15,-1,1),varcov=diag(3),loadings=runif(15,.3,.9))
#save(FC, file="../data/FC.rda")
#write.table(FC,'myDataFile.dat',quote=FALSE, sep=" ", col.names = FALSE, row.names = FALSE)
"FC"

### Roxygen-izable Documentation ----
#' Create lavaan syntax for Thurstonian forced choice analysis
#' 
#' This function writes a lavaan syntax given the specifications of the
#' Thurstonian forced choice model. 
#' 
### Inputs ----
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param model A descriptor for the model. Can be one of `'lmean'`,
#' `'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
#' the `'simple'` statement defines the Thurstone case.
#' @param rename_list A list with two vectors to rename the objects in the syntax.
#' Vector one is the original names, vector two the new names. Defaults to `NULL`.
#' 
### Outputs ---- 
#' @return Returns a description of the user-specified model. Typically, the
#' model is described using the lavaan model syntax. See `lavaan::model.syntax`
#' for more information. 
#' 
#' @details The syntax currently is able to perform model analysis for the
#' latent utility models (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
#' unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
#' the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
#' model types can be performed with all types of forced choice designs (full,
#' block, partially linked block, linked block). For an overview and review see
#' Jansen and Schulze (2023a,2023b).
#' 
#' The standard naming procedure ixiy, for the comparison of items x and y, 
#' can be changed by specifying the `rename_list` argument. The first vector of
#' the schould be the vector of original names, for example 
#' `c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
#' the second vector should contain the new names, for example 
#' `c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.
#' 
### Examples ----
#' @examples
#' 
#' # read data set FC
#' data(FC)
#' 
#' # set seed and define blocks
#' set.seed(1)
#' blocks <- matrix(sample(1:15,15), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,5)
#' 
#' # Create lavaan model syntax
#' syntax.lavaan(blocks,itf,'irt')
#' 
#' 
#' @references 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}
#'
#' Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
#'  
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.
#' 
#' Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 
#' 
#' @export




syntax.lavaan <- function(blocks,itf,model,rename_list=NULL){
  
  if(model=='simple2'){
    model <- 'simple'
    case <- 2
  }
  if(model=='simple3'){
    model <- 'simple'
    case <- 3
  }
  if(model=='simple5'){
    model <- 'simple'
    case <- 5
  }
  
  if(!is.matrix(blocks)&!is.vector(blocks)){
    stop('Blocks must be given as a matrix. Only for full designs a vector is possible.')
  }
  if(is.vector(blocks)){
    blocks <- blocks <- matrix(blocks,nrow=1)
  }
  design <- ifelse(dim(blocks)[1]==1,'full','block')
  
  # Find number of items per block
  nrank <- dim(blocks)[2]
  
  # Find number of factors, blocks and items
  nblock <- dim(blocks)[1]
  nitem <- length(unique(c(blocks)))
  nfactor <- max(itf)

  # Pair combinations
  pair_combn <- t(utils::combn(1:nitem,2))
  
  # Define items to factors for lambda (loadings)
  lambda_raw <- matrix(0,nrow=nitem,ncol=nfactor)
  for(i in 1:nfactor){
    lambda_raw[,i] <- ifelse(itf==i,i,0)
  }
  
  # Create names of pairs
  pair_names <- paste0('i',pair_combn[,1],'i',pair_combn[,2])
  
  # For each block: is it uni-dimensional?
  blocks_lambda <- matrix(itf[blocks],ncol=nrank)
  unidim <- apply(blocks_lambda,1,function(x) min(x)==max(x))
  
  # Create names of pairs for blocks
  pair_names_b <- i.name(blocks)
  pair_combn_b <- pair.combn(blocks)
  
  # Begin syntax

  
  # write lavaan model
  # MODEL
  if(model!='simple'){
    traits <- paste0('Trait',1:nfactor)
  }
  
  
  # Create Matrix A
  A <- designA(blocks)
  Arref <- rref(A)
  Arank <- dim(A)[1]-length(which(apply(abs(Arref),1,sum)==0))
  
  # Find global blocks
  gblocks <- metablock(blocks)
  unidim_gblocks <- sapply(gblocks,function(x) min(itf[x])==max(itf[x]))
  
  # Latent utility model and the unconstrained threshold model
  if(model%in%c('lmean','uc','simple')){
    # Part 1 (primary factors)
    input <- paste0('# Model of primary factors:\n')
    t <- paste0('t',1:nitem)
    A_sign <- ifelse(A==0,'',ifelse(A==1,'+','-'))
    
    At <- matrix(NA, nrow=nrow(A_sign), ncol=nitem)
    At <- t(apply(A_sign,1,function(x)sub('\\+','',sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,t)))))))
    At <- sub('^-','',matrix(paste0(At,' =~ ',ifelse(A_sign=='+','',A_sign),'1*',pair_names_b,'\n'), ncol=nitem))
    At <- ifelse(A_sign=='','',At)
    At_text <- paste0(apply(At,2,function(x)paste0(x,collapse = '')), collapse = '')
    
    input <- paste0(input, paste(At_text, collapse='\n'), '\n\n')
    
    # For rankings: Fix variances of indicators to 0
    input <- paste0(input, '# Fix the variances of all indicators to 0:\n')
    input <- paste0(input, paste0(pair_names_b,' ~~ 0*',pair_names_b, collapse = '\n'))
    #### Whats with paired comparisons?
  
    # Part 2 (secondary factors)
    if(model!='simple'){
      input <- paste0(input, '\n\n # Model of secondary factors:\n')
      if(nfactor==1&Arank==(nitem-1)){
        input <- paste0(input, 'Trait1 =~ 1 * t1 +', paste0(paste0('start(1) * t',2:nitem),collapse=' + \n'), '\n\n')
      }else{
        A_fac <- rep('',nfactor)
        
        itf_gblocks <- lapply(gblocks,function(x) itf[x])
        items_fac <- list()
        items_fac_uni <- list()
        for(i in 1:nfactor){
          item_fac <- c()
          item_fac_uni <- c()
          for(j in 1:length(gblocks)){
            if(unidim_gblocks[j]){
              item_fac_uni <- c(item_fac_uni,gblocks[[j]][which(itf_gblocks[[j]]==i)[1]])
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)[-1]])
            }else{
              item_fac <- c(item_fac,gblocks[[j]][which(itf_gblocks[[j]]==i)])
            }
          }
          items_fac[[i]] <- sort(item_fac)
          if(all(is.na(item_fac_uni))){
            items_fac_uni[[i]] <- NA
          }else{
            items_fac_uni[[i]] <- sort(item_fac_uni)
          }
          
          A_fac[[i]] <- paste0(traits[i], ' =~ ')
          A_fac[[i]] <- paste0(paste0(traits[i], ' =~ '),paste0('start(1) * t',unname(items_fac[[i]]), collapse = ' + '))
          
          #for(j in 1:length(gblocks)){
          #if(unidim_gblocks[j]){
          #if(length(items_fac_uni[[i]])>0){
          #A_fac[[i]] <- paste0(A_fac[[i]],paste0('1 * t',unname(items_fac_uni[[i]]), '\n', collapse = '+'))
          if(all(!is.na(items_fac_uni[[i]]))){
            A_fac[[i]] <- paste0(A_fac[[i]],' + ',paste0('1 * t',unname(items_fac_uni[[i]]), '\n', collapse = ' + \n'))
          }
          #}
          #}
          #}
        }
        input <- paste0(input, paste0(A_fac,collapse = '\n'),'\n\n')
      }
      
      # Fix variances of factors to 1
      input <- paste0(input, '# Fix variances of all factors to 1:\n')
      input <- paste0(input, paste0(traits,' ~~ 1 * ',traits, collapse = '\n'), '\n')
      
      input <- paste0(input, '# Define the correlations between factors:\n')
      if(nfactor>1){
        inp_tmp <- paste0('Trait',t(utils::combn(1:nfactor,2))[,1], ' ~~ ', 'Trait', t(utils::combn(1:nfactor,2))[,2], collapse = '\n')
        input <- paste0(input, inp_tmp, '\n\n')
      }
      # Fix uniqueness of one item "per block"
      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      #####
      if(nrank==2){
        input <- paste0(input, '# Fix uniquenesses of all items to 1:\n')
        input <- paste0(input, paste0('t',1:nitem, '~~ 1*t',1:nitem,collapse = '\n'), '\n')
      }else{
        input <- paste0(input, '#Fix uniquenesses of one items per grand block:\n')
        for(i in 1:length(gblocks)){
          tmp <- min(gblocks[[i]])
          input <- paste0(input,'t',tmp,'~~ 1*t',tmp,'\n')
        }
      }
    }

    # Fixing for the latent utility model
    if(model=='lmean'|model=='simple'){
      input <- paste0(input, '#Fix all thresholds to 0 to estimate means:\n')
      input <- paste0(input, paste0(pair_names_b,'|0*t1', collapse='\n'),'\n\n')
      if(model!='simple'){
        input <- paste0(input, '#Fix all means of traits to 0:\n')
        input <- paste0(input, paste0(traits,'~0', collapse='\n'),'\n\n')
      }

      if(design=='full'){
        input <- paste0(input, '# Fix one mean to 0 and free all other means via int.lv.free = TRUE:\n')
        input <- paste0(input, 't1~0','\n\n')
      }
      if(design=='block'){
        input <- paste0(input, '# Fix one mean per grand block to 0 and free all other means via int.lv.free = TRUE:\n')

        tmptmp <- c()
        for(i in 1:length(gblocks)){
          tmp <- min(gblocks[[i]])
          tmptmp <- c(tmptmp,tmp)
        }
        
        input <- paste0(input, paste0('t',tmptmp,' ~ 0', collapse='\n'))

      }
    }
    
    #input <- ''
    # Additional Fixing for the Thurstone cases
    if(model=='simple'){
      if(case==5){
        input <- paste0(input,'\n','# fix factor variances to 1\n',paste0('t',1:nitem,' ~~ 1*','t',1:nitem, collapse = '\n'),'\n\n')
        input <- paste0(input, '# fix all covariances between factors to zero:\n')
        
        inp_tmp <- paste0('t',t(utils::combn(1:nitem,2))[,1], ' ~~ 0*', 't', t(utils::combn(1:nitem,2))[,2], collapse = '\n')
        input <- paste0(input, inp_tmp, '\n\n')
      }
      if(case==3){
        input <- paste0(input,'\n','# free factor variances except the last one\n',paste0('t',nitem),' ~~ 1*\n',paste0('t',nitem),'\n\n')
        
        input <- paste0(input, '# fix all covariances between factors to zero:\n')
        
        inp_tmp <- paste0('t',t(utils::combn(1:nitem,2))[,1], ' ~~ 0*', 't', t(utils::combn(1:nitem,2))[,2], collapse = '\n')
        input <- paste0(input, inp_tmp, '\n\n')
      }
      if(case==2){
        input <- paste0(input,'\n','# free factor variances except the first and last one\n', paste0('t',c(1,nitem),' ~~ 1*','t',c(1,nitem),collapse = '\n'),'\n\n')
        
        input <- paste0(input, '# all Factor covariances are free except these involving the last factor:\n')
        tmp <- t(utils::combn(1:nitem,2))
        tmp1 <- tmp[which(tmp[,2]!=nitem),]
        inp_tmp1 <- paste0('t',tmp1[,1], ' ~~ ', 't', tmp1[,2], collapse = '\n')
        
        tmp2 <- tmp[which(tmp[,2]==nitem),]
        inp_tmp2 <- paste0('t',tmp2[,1], ' ~~ 0*', 't', tmp2[,2], collapse = '\n')
        input <- paste0(input, inp_tmp1, '\n\n', inp_tmp2, '\n\n')
      }
    }
    
    
    #if(standardized){
    #  input <- paste0(input, '\n\nOUTPUT: STDYX;\n\n')
    #}
  }
  
  # IRT model
  if(model%in%c('irt')){
    # Create matrix lambda
    lambda <- ifelse(lambda_raw==0,0,paste0('L',row(lambda_raw)))
    A_sign <- ifelse(A==0,0,ifelse(A==1,'+','_'))
    A_lambda <- A%*%ifelse(lambda_raw==0,0,1)
    Lambda <- matrix(NA, nrow=nrow(A_sign), ncol=ncol(lambda))
    
    for(i in 1:nfactor){
      Lambda[,i] <- apply(A_sign,1,function(x) paste0(sub('^_0.*','',sub('^\\+0.*','',sub('^0.*','',paste0(x,lambda[,i])))), collapse =''))
    }
    tmp <- grep('^_L.+\\+',Lambda)
    tmp1 <- Lambda[tmp]
    Lambda[tmp] <- paste0(sub('^_L.+\\+','',tmp1),sub('\\+L.+','',tmp1))
    Lambda <- sub('\\+','',Lambda)
    Lambda <- sub('^_L','L_',Lambda)
    
    A_mplus <- NULL
    for(i in 1:nfactor){
      tmp <- ifelse(Lambda[,i]=='','',
                    paste0('start(',A_lambda[,i],') * ',Lambda[,i],' * ',pair_names_b))
      tmp <- tmp[which(tmp!='')]
      A_mplus <- paste0(A_mplus,'\n',traits[i],' =~ ',
                        paste0(tmp,collapse = ' + '),'')
    }
    
    # Fix variances of factors to 1
    input <- paste0(A_mplus,'\n\n', '# Fix variances of all factors to 1:\n')
    input <- paste0(input, paste0(traits,' ~~ 1 * ',traits, collapse = '\n'), '\n')
    
    
    # Correlations between factors
    input <- paste0(input, '# Define the correlations between factors:\n')
    if(nfactor>1){
      inp_tmp <- paste0('Trait',t(utils::combn(1:nfactor,2))[,1], ' ~~ ', 'Trait', t(utils::combn(1:nfactor,2))[,2], collapse = '\n')
      input <- paste0(input, inp_tmp, '\n\n')
    }
    
    if(nrank==2){
      input <- paste0(input, '# Fix uniquenesses of all indicators to 1:\n')
      input <- paste0(input, paste0(pair_names_b,'~~1*',pair_names_b,collapse = '\n'), '\n')
      if(length(gblocks)!=nblock){
        # Model constraints
        input <- paste0(input, '# Set model constraints for loadings:\n')
        #input <- paste0(input, '\n','MODEL CONSTRAINT:\nNEW\n')
        
        # Define new parameters
        unidim_item <- Lambda[grep('^L.*_L',Lambda)]
        tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
        if(length(which(tmp%in%Lambda))!=0){
          tmp <- tmp[-which(tmp%in%Lambda)]
        }
        
        AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
        
        tmp <- c(AddL,tmp)
        Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
        
        Add_unidim <- NULL
        for(i in 1:nrow(blocks)){
          tmp <- blocks[i,]%in%sub('L','',Add_param)
          if(all(tmp)){
            Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
          }
        }
        
        # Constraints
        conL <- rep('',nitem)
        for(i in 1:nitem){
          if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
            if(paste0('L_',i)%in%Lambda){
              conL[i] <- paste0('L_',i,' == ','- L',i)
            }
          }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
            conL[i] <- paste0('L_',i,' == ','- L',i)
          }
        }
        #pair_names_b_mat <- ifelse(Lambda=="","",pair_names_b)
        
        
        #Add_param_tmp_pos <- paste0(Add_param,' := -',gsub('L','L_',Add_param), collapse = '\n')
        #Add_param_tmp_neg <- paste0(gsub('L','L_',Add_param),' := -',Add_param, collapse = '\n')
        unidim_item_lavx <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
        conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')
        
        
        tmp <- sub('_.*$','',unidim_item)
        tmp1 <- sub('^.*_','',unidim_item)
        unidim_item_tmp <- unidim_item
        
        y <- c(paste0('L_',1:nitem)[which(paste0('L_',1:nitem)%in%Lambda)])
        y <- sub('_','',y)
        y <- y[which(y%in%AddL)]
        
        
        mins <- c()
        for(i in 1:length(gblocks)){
          if(unidim_gblocks[i]){
            mins <- c(mins, min(gblocks[[i]]))
          }
        }
        
        known <- c(paste0('L',1:nitem))
        known <- known[which(!known%in%AddL)]
        known <- unique(c(known,y))
        if(length(mins)!=0){
          known <- unique(c(known,paste0('L',mins)))
        }
        
        conL_neg <- ''
        if(length(y)!=0){
          conL_neg <- paste0(y,' := -',gsub('L','L_',y), collapse = '\n')
        }
        
        
        
        unidim_item_lav <- ''
        lknown <- length(known)
        if(lknown!=nitem){
          
          x <- which(tmp%in%AddL|tmp1%in%AddL)
          
          
          tmp <- tmp[x]
          tmp1 <- tmp1[x]
          unidim_item_tmp <- unidim_item_tmp[x]
          
          
          tmp2 <- which(!Add_param%in%known) #which(!Add_param%in%c(tmp1,paste0('L',mins)))
          
          # change order for fixing parameters where needed
          tmp1s <- c()
          tmps <- c()
          if(length(tmp2)>0){
            for(i in 1:length(tmp2)){
              w <- which(tmp==Add_param[tmp2[i]]) # which(tmp==paste0('L',tmp2[i]))
              if(length(w)!=0){
                x <- tmp1[w]
                b <- table(tmp1)[names(table(tmp1))%in%x]
                z <- names(which(b==max(b)))[1]
                
                a <- w[which(x==z)]
                
                u <- tmp[a]
                v <- tmp1[a]
                tmp1[a] <- u
                tmp[a] <- v
                tmp1s <- c(tmp1s,z)
                tmps <- c(tmps,u)
              }
            }
          }
          
          while(!all(tmp%in%known)){
            x <- which(tmp%in%known)
            known <- unique(c(known,tmp1[x]))
            tmp <- c(tmp[x],tmp[-x])
            tmp1 <- c(tmp1[x],tmp1[-x])
            unidim_item_tmp <- c(unidim_item_tmp[x],unidim_item_tmp[-x])
          }
          
          i <- 2
          while(!is.na(tmp1[i])){
            x <- tmp1[i]
            if(x%in%tmp1[1:(i-1)]){
              tmp <- tmp[-i]
              tmp1 <- tmp1[-i]
              unidim_item_tmp <- unidim_item_tmp[-i]
            }else{
              i <- i+1
            }
          }
          b <- which(tmp1%in%tmps&tmp%in%tmp1s)
          if(length(b)==0){
            b <- 0
          }
        }
        
        if(length(tmp1)!=0&lknown!=nitem){
          
          x <- AddL[which(!AddL%in%y)]
          x <- which(tmp1%in%x)
          
          tmp <- tmp[x]
          tmp1 <- tmp1[x]
          unidim_item_tmp <- unidim_item_tmp[x]
          
          unidim_sign <- rep('-',length(tmp1))
          unidim_sign[b] <- '+'
          
          unidim_item_lav <- paste0(tmp1,' := ', tmp, unidim_sign,unidim_item_tmp,'\n',collapse = '')
        }
        
        mins <- ifelse(length(mins)==0,'',paste0('\n L',mins,':=1', collapse = '\n'))
        
        #Add_param_pre <- paste0(paste0(Add_param,' =~ 0', collapse = '\n'),'\n')
        #Add_param_post <- paste0(Add_param," ~ NA*1 + label('",Add_param,"')*1 +0?1", collapse = '\n')
        #unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
        #conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')
        
        #input <- paste0(input,Add_param,'\n\n',unidim_item,'\n',conL,'\n\n')
        
        #input <- paste0(input,Add_param_pre,'\n\n',Add_param_post,'\n\n',unidim_item,'\n',conL,'\n\n')
        
        #if(unidim_item_lav==''){
        #  input <- paste0(input,'\n\n',mins,'\n\n',conL_neg,'\n\n',unidim_item_lavx,'\n\n',
        #                  conL,'\n\n')
        #}else{
        input <- paste0(input,'\n\n',mins,'\n\n',conL_neg,'\n\n',unidim_item_lav,'\n',unidim_item_lavx,'\n\n',
                        '\n',conL,'\n\n')
        #}
        
        
        
        #Add_param <- paste0(Add_param,' == -',gsub('L','L_',Add_param), collapse = '\n')
        #unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
        #conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')
        
        
        
        #input <- paste0(input,unidim_item,'\n',Add_param,'\n',conL,'\n\n')
        
        
        # Fix one loading in each unidim block if this block is not connected with blocks of another factor or of already fixed blocks
        # This is not necessary but maybe more helpful for application. Fix the lowest item per block
        input <- paste0(input, '# Fix one loading per uni-dimensional grand block:\n')
        for(i in 1:length(gblocks)){
          if(unidim_gblocks[i]){
            input <- paste0(input, paste0('L',min(gblocks[[i]]),'==1\n', '\n'))
          }
        }
        
        input <- paste0(input,'\n')
      }
    }
    if(nrank>2){
      # Declare uniqunesses
      input <- paste0(input, '# Declare uniquenesses:\n')
      input <- paste0(input,paste0(pair_names_b,' ~~ ','e',pair_combn_b[,1],'e',pair_combn_b[,2],' * ',pair_names_b, collapse = '\n'))
      
      # Create Matrix Psi for irt models
      input <- paste0(input, '\n\n# Define structured uniquenesses:\n')
      Psi <- ifelse(row(diag(nitem))==col(diag(nitem)),paste0('e',1:nitem),0)
      Psi <- ifelse(Psi==0,'',Psi) # neu
      A_sign <- ifelse(A==0,0,ifelse(A==1,'+','-')) # alt
      
      APsi <- matrix(NA, nrow=length(pair_names_b), ncol=nitem)
      for(i in 1:nitem){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[j,],Psi[,i])
          tmp <- sub('^-$','',sub('^\\+$','',tmp))
          tmp <- sub('^-0.*','',sub('^\\+0.*','',sub('^0.*','',tmp)))
          APsi[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsi <- sub('^\\+','',APsi)
      
      APsiA <- matrix(NA, nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:length(pair_names_b)){
        for(j in 1:length(pair_names_b)){
          tmp <- paste0(A_sign[i,],APsi[j,])
          #tmp <- sub('\\+','',sub('^0.*','',tmp))
          tmp <- sub('^0.*','',tmp)
          APsiA[j,i] <- paste0(tmp, collapse ='')
        }
      }
      APsiA <- ifelse(APsiA=='+-','',ifelse(APsiA=='-+','',APsiA))
      APsiA <- gsub('-\\+','',APsiA)
      APsiA <- gsub('\\+-','-',APsiA)
      
      APsiA <- gsub('--','+',APsiA)
      
      APsiA <- sub('-$','',APsiA)#
      APsiA <- sub('\\+$','',APsiA)#
      APsiA <- sub('^\\+','',APsiA)
      
      APsiA_under  <- sub('-e','e_',APsiA)
      APsiA_sign <- ifelse(APsiA=='','',ifelse(substr(APsiA,1,1)=='-','-',''))
      
      
      # Correlations in Psi
      APsiA_text <- matrix('', nrow=length(pair_names_b), ncol=length(pair_names_b))
      for(i in 1:(nrow(APsiA)-1)){
        for(j in (i+1):nrow(APsiA)){
          APsiA_text[j,i] <- paste0(pair_names_b[i],' ~~ start(',APsiA_sign[j,i],'1) * ',APsiA_under[j,i],' * ', pair_names_b[j])
        }
      }
      
      APsiA_text <- ifelse(APsiA=='','',APsiA_text)
      APsiA_text <- paste0(ifelse(APsiA_text=='','',paste0(APsiA_text,'\n')), collapse = '')
      
      input <- paste0(input,'\n\n',APsiA_text,'\n\n')
      
      # Model constraints
      input <- paste0(input, '# Set model constraints for loadings and uniquenesses:\n')
      
      # Define new parameters
      unidim_item <- Lambda[grep('^L.*_L',Lambda)]
      tmp <- unique(c(sub('_.*','',unidim_item),sub('^.*_','',unidim_item)))
      if(length(which(tmp%in%Lambda))!=0){
        tmp <- tmp[-which(tmp%in%Lambda)]
      }
      
      AddL <- c(paste0('L',1:nitem)[which(!paste0('L',1:nitem)%in%Lambda)])
      
      tmp <- c(AddL,tmp)
      Add_param <- unique(c(sort(tmp[which(nchar(tmp)==2)]),sort(tmp[which(nchar(tmp)==3)]),sort(tmp[which(nchar(tmp)==4)])))
      
      Add_unidim <- NULL
      for(i in 1:nrow(blocks)){
        tmp <- blocks[i,]%in%sub('L','',Add_param)
        if(all(tmp)){
          Add_unidim <- c(Add_unidim ,paste0('L',blocks[i,]))
        }
      }
      
      # Constraints
      conL <- rep('',nitem)
      for(i in 1:nitem){
        if(paste0('L',i)%in%Lambda|paste0('L',i)%in%Add_param){
          if(paste0('L_',i)%in%Lambda){
            conL[i] <- paste0('L_',i,' == ','- L',i)
          }
        }else if(i%in%sub('L','',Add_param[-which(Add_param%in%Add_unidim)])){
          conL[i] <- paste0('L_',i,' == ','- L',i)
        }
      }
      
      #pair_names_b_mat <- ifelse(Lambda=="","",pair_names_b)
      
      
      #Add_param_tmp_pos <- paste0(Add_param,' := -',gsub('L','L_',Add_param), collapse = '\n')
      #Add_param_tmp_neg <- paste0(gsub('L','L_',Add_param),' := -',Add_param, collapse = '\n')
      unidim_item_lavx <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
      conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')

      
      tmp <- sub('_.*$','',unidim_item)
      tmp1 <- sub('^.*_','',unidim_item)
      unidim_item_tmp <- unidim_item
      
      y <- c(paste0('L_',1:nitem)[which(paste0('L_',1:nitem)%in%Lambda)])
      y <- sub('_','',y)
      y <- y[which(y%in%AddL)]
      
      
      mins <- c()
      for(i in 1:length(gblocks)){
        if(unidim_gblocks[i]){
          mins <- c(mins, min(gblocks[[i]]))
        }
      }
      
      known <- c(paste0('L',1:nitem))
      known <- known[which(!known%in%AddL)]
      known <- unique(c(known,y))
      if(length(mins)!=0){
        known <- unique(c(known,paste0('L',mins)))
      }
      
      conL_neg <- ''
      if(length(y)!=0){
        conL_neg <- paste0(y,' := -',gsub('L','L_',y), collapse = '\n')
      }
      
      
      
      unidim_item_lav <- ''
      lknown <- length(known)
      if(lknown!=nitem){
        
        x <- which(tmp%in%AddL|tmp1%in%AddL)
        
        
        tmp <- tmp[x]
        tmp1 <- tmp1[x]
        unidim_item_tmp <- unidim_item_tmp[x]
        
        
        tmp2 <- which(!Add_param%in%known) #which(!Add_param%in%c(tmp1,paste0('L',mins)))
        
        # change order for fixing parameters where needed
        tmp1s <- c()
        tmps <- c()
        if(length(tmp2)>0){
          for(i in 1:length(tmp2)){
            w <- which(tmp==Add_param[tmp2[i]]) # which(tmp==paste0('L',tmp2[i]))
            if(length(w)!=0){
              x <- tmp1[w]
              b <- table(tmp1)[names(table(tmp1))%in%x]
              z <- names(which(b==max(b)))[1]
              
              a <- w[which(x==z)]
              
              u <- tmp[a]
              v <- tmp1[a]
              tmp1[a] <- u
              tmp[a] <- v
              tmp1s <- c(tmp1s,z)
              tmps <- c(tmps,u)
            }
          }
        }

        while(!all(tmp%in%known)){
          x <- which(tmp%in%known)
          known <- unique(c(known,tmp1[x]))
          tmp <- c(tmp[x],tmp[-x])
          tmp1 <- c(tmp1[x],tmp1[-x])
          unidim_item_tmp <- c(unidim_item_tmp[x],unidim_item_tmp[-x])
        }
        
        i <- 2
        while(!is.na(tmp1[i])){
          x <- tmp1[i]
          if(x%in%tmp1[1:(i-1)]){
            tmp <- tmp[-i]
            tmp1 <- tmp1[-i]
            unidim_item_tmp <- unidim_item_tmp[-i]
          }else{
            i <- i+1
          }
        }
        b <- which(tmp1%in%tmps&tmp%in%tmp1s)
        if(length(b)==0){
          b <- 0
        }
      }
      
      if(length(tmp1)!=0&lknown!=nitem){
        
        x <- AddL[which(!AddL%in%y)]
        x <- which(tmp1%in%x)
        
        tmp <- tmp[x]
        tmp1 <- tmp1[x]
        unidim_item_tmp <- unidim_item_tmp[x]
        
        unidim_sign <- rep('-',length(tmp1))
        unidim_sign[b] <- '+'
        
        unidim_item_lav <- paste0(tmp1,' := ', tmp, unidim_sign,unidim_item_tmp,'\n',collapse = '')
      }
      
      mins <- ifelse(length(mins)==0,'',paste0('\n L',mins,':=1', collapse = '\n'))
      
      #Add_param_pre <- paste0(paste0(Add_param,' =~ 0', collapse = '\n'),'\n')
      #Add_param_post <- paste0(Add_param," ~ NA*1 + label('",Add_param,"')*1 +0?1", collapse = '\n')
      #unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
      #conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')
      
      #input <- paste0(input,Add_param,'\n\n',unidim_item,'\n',conL,'\n\n')
      
      #input <- paste0(input,Add_param_pre,'\n\n',Add_param_post,'\n\n',unidim_item,'\n',conL,'\n\n')
      
      #if(unidim_item_lav==''){
      #  input <- paste0(input,'\n\n',mins,'\n\n',conL_neg,'\n\n',unidim_item_lavx,'\n\n',
      #                  conL,'\n\n')
      #}else{
        input <- paste0(input,'\n\n',mins,'\n\n',conL_neg,'\n\n',unidim_item_lav,'\n',unidim_item_lavx,'\n\n',
                        '\n',conL,'\n\n')
      #}
      

      
      #Add_param <- paste0(Add_param,' == -',gsub('L','L_',Add_param), collapse = '\n')
      #unidim_item <- ifelse(length(unidim_item)==0,'',paste0(unidim_item,' == ', sub('_','-',unidim_item),'\n',collapse = ''))
      #conL <- paste0(ifelse(conL=='','',paste0(conL,'\n')), collapse = '')
      

      
      #input <- paste0(input,unidim_item,'\n',Add_param,'\n',conL,'\n\n')
      
      
      
      
      
      
      
      
      
      
      # Fix one loading in each unidim block if this block is not connected with blocks of another factor or of already fixed blocks
      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      input <- paste0(input, '# Fix one loading per uni-dimensional grand block:\n')
      for(i in 1:length(gblocks)){
        if(unidim_gblocks[i]){
          input <- paste0(input, paste0('L',min(gblocks[[i]]),'==1\n', '\n'))
        }
      }
      
      input <- paste0(input,'\n')
      
      # Uniquenesses relating to the same item are equal
      input <- paste0(input, '# Set uniquenesses relation to the same item as equal:\n')
      tmp <- rep(NA,nitem)
      for(i in 1:nitem){
        tmp[i] <- paste0('e',i)%in%APsiA[lower.tri(APsiA)]&paste0('-e',i)%in%APsiA[lower.tri(APsiA)]
        if(tmp[i]){
          input <- paste0(input,paste0('e_',i),' == ',paste0('- e',i),'\n')
        }
      }
      
      input <- paste0(input,'\n')
      
      # Pair's uniqueness is equal to sum of two utility uniquenesses
      input <- paste0(input, '# A pairs uniqueness is equal to the sum of the item uniquenesses:\n')
      if(design=='full'){
        input <- paste0(input,paste0('e',pair_combn[,1],'e',pair_combn[,2],' == e',pair_combn[,1],' + e',pair_combn[,2], collapse = '\n'),'\n\n')
      }else if(design=='block'){
        uniquenesses <- c()
        for(i in 1:nrow(blocks)){
          items <- blocks[i,]
          whitems <- which(pair_combn_b[,1]%in%items&pair_combn_b[,2]%in%items)
          tmp <- pair_combn_b[whitems,1][which(pair_combn_b[whitems,1]%in%pair_combn_b[whitems,2])]
          
          tmp <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2],' == ' ,ifelse(pair_combn_b[whitems,1]%in%tmp,'-e_','e'),pair_combn_b[whitems,1],
                        ifelse(pair_combn_b[whitems,2]%in%tmp,' - e_',' + e'),pair_combn_b[whitems,2])
          tmp0 <- paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2])
          for(i in length(tmp0):1){
            if(any(grepl(tmp0[i],uniquenesses))){
              tmp <- tmp[-i]
            }
          }
          uniquenesses <- c(uniquenesses,tmp)
          ##sapply(paste0('e',pair_combn_b[whitems,1],'e',pair_combn_b[whitems,2]),function(x) grepl(x,uniquenesses))
        }
        uniquenesses <- unique(uniquenesses[order(nchar(uniquenesses),uniquenesses)])
        input <- paste0(input,paste0(uniquenesses, collapse = '\n'),'\n', collapse = '\n')
      }

      # This is not necessary but maybe more helpful for application. Fix the lowest item per block
      for(i in 1:length(gblocks)){
        tmp2 <- min(gblocks[[i]])
        tmp3 <- max(gblocks[[i]])
        if(paste0('e',tmp2)%in%APsiA_under[lower.tri(APsiA)]){
          input <- paste0(input,'\n','e',tmp2,' == 1')
        }else{
          input <- paste0(input,'\n','e_',tmp2,' == -1')
        }
        
      }
    }
    
  }
  if(!is.null(rename_list)){
    tmp <- ceiling(log10(length(rename_list[[1]])))
    elem <- c()
    for(i in 1:tmp){
      tmp0 <- paste0('el3m3nt',paste0(rep('0',tmp-i),collapse = ''),(10^(i-1)):((10^i)-1))
      elem <- c(elem,tmp0)
    }
    elem <- elem[1:length(rename_list[[1]])]
    
    for(i in 1:length(rename_list[[1]])){
      input <- gsub(rename_list[[1]][i],elem[i],input)
    }
    for(i in 1:length(rename_list[[1]])){
      input <- gsub(elem[i],rename_list[[2]][i],input)
    }
  }
  
  return(input)
}

choose.item <- function(tmp,tmp2,i, multidim, itf=NULL, tmp_itf=NULL,warn=0){
  
  if(length(tmp2)==(length(tmp[[i]])-1)){
    tmp3 <- tmp[[i]][-tmp2]
  }else if(length(tmp2)>0&multidim){
    if(length(tmp[[i]])==length(tmp2)){
      tmp2 <- which(itf[tmp[[i]]]%in%tmp_itf)
      if(length(tmp[[i]])==length(tmp2)){
        stop("There are not enough items to choose from for multidimensional blocks. This is most likly due to nfactors < nrank.")
      }else if(length(tmp2)>0){
        tmp3 <- sample(tmp[[i]][-tmp2],size=1)
        warn <- warn+1
      }else{
        tmp3 <- sample(tmp[[i]],size=1)
      }
    }else{
      tmp3 <- sample(c(tmp[[i]][-tmp2]),size=1)
    }
  }else{
    tmp3 <- sample(tmp[[i]],size=1)
  }
  return(list(tmp3,warn))
}

rref <- function (A){
  stopifnot(is.numeric(A))
  if (!is.matrix(A)) 
    stop("Input parameter 'A' must be a matrix.")
  nr <- nrow(A)
  nc <- ncol(A)
  tol <- eps() * max(nr, nc) * max(abs(A))
  r <- 1
  for (i in 1:nc) {
    pivot <- which.max(abs(A[r:nr, i]))
    pivot <- r + pivot - 1
    m <- abs(A[pivot, i])
    if (m <= tol) {
      A[r:nr, i] <- 0
    }
    else {
      A[c(pivot, r), i:nc] <- A[c(r, pivot), i:nc]
      A[r, i:nc] <- A[r, i:nc]/A[r, i]
      if (r == 1) {
        ridx <- c((r + 1):nr)
      }
      else if (r == nr) {
        ridx <- c(1:(r - 1))
      }
      else {
        ridx <- c(1:(r - 1), (r + 1):nr)
      }
      A[ridx, i:nc] <- A[ridx, i:nc] - A[ridx, i, drop = FALSE] %*% 
        A[r, i:nc, drop = FALSE]
      if (r == nr) 
        break
      r <- r + 1
    }
  }
  A[abs(A) < tol] <- 0
  return(A)
}

### Roxygen-izable Documentation ----
#' Creates names for paired comparisons of a given design.
#' 
#' This function creates names for paired comparisons in the ixiy scheme. If
#' items 1 and 2 are compared, this corresponds to i1i2.
#' 
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' 
### Outputs ---- 
#' @return Returns a character vector, containing names for all binary indicators
#' of a design.
#' 
### Examples ----
#' @examples
#' 
#' # Define 30 items divided by ten triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' i.name(blocks)
#'     
#' @export

i.name <- function(blocks){
    pair_combn <- pair.combn(blocks)
    pair_names <- paste0('i',pair_combn[,1],'i',pair_combn[,2])
  return(pair_names)
}

### Roxygen-izable Documentation ----
#' Sorts the blocks in ascending numbering
#' 
#' This function sorts all items in a block into ascending order.
#' 
### Inputs ----
#'
#' @param blocks A matrix defining the blocks of the model. The number of rows 
#' must be the number of blocks, each row represents a block and contains the 
#' item numbers. The number of columns represent the number of items per block.
#' 
### Outputs ---- 
#' @return Returns a matrix consisting of the blocks where all items per blocks
#' are sorted in ascending order.
#' 
### Examples ----
#' @examples
#' 
#' # Define 30 items divided by ten triplets as blocks
#' blocks <- matrix(c(1:30), ncol = 3)
#' 
#' # sort the blocks
#' blocksort(blocks)
#' 
#' @export


### Function definition ----

blocksort <- function(blocks){
  blocks <- t(apply(blocks,1,sort))
  return(blocks)
}

### Roxygen-izable Documentation ----
#' Estimate factor scores based on Genuine Likelihood
#' 
#' This function estimates factor scores based on genuine likelihood (Yousfi, 2019).
#' 
#' 
### Inputs ----
#' @param dat A data.frame or matrix defining holding the binary coded and named
#' response data.
#' @param blocks A matrix defining the blocks of the model. The number of rows
#' must be the number of blocks, each row represents a block and contains the
#' item numbers. The number of columns present the number of items per block.
#' @param itf A vector defining the items-to-factor relation. For example
#' `c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
#' the second three correspond to factor 2. 
#' @param fit The fit object. In case of Mplus, this is the path to the Mplus output file. In case of lavaan this is the lavaan Object.
#' @param alg The algorithm to use for numerical integration. See ?mvtnorm::pmvnorm.
#' @param log logical. Should the log transformed results be returned. Defaults to TRUE.
#' @param mp The means of prior distribution. Defaults to multivariate normal with means 0 and vars 1
#' @param sp The vars of prior distribution. Defaults to multivariate normal with means 0 and vars 1
#' @param serr Logical. Should the standard errors be returned. Defaults to TRUE.
#' @param sv Starting values. Defaults to NULL.
#' @param blocks_ul Similar to `blocks`, but only with the unlinked design. Useful
#' for comparison with other functions based on R.
#' @param mplus Logical. Are results read from mplus? Defaults to FALSE.
#' @param ... other arguments passed by get.scores for optim.
#' 
### Outputs ---- 
#' @return The result is a list with the factor scores and the standard errors per person and factor.
#' 
#' @details The function estimates factor scores based on the Thurstonian IRT model
#' and based on genuine likelihood proposed by Yousfi (2019). The function allows
#' for the estimation of factor scores under all block designs, limited only by 
#' numerical integration procedures. For items per block between 5 and 20 use 
#' `alg=mvtnorm::Miwa()`. Additionally, all model types can be performed with all
#' types of forced choice designs (full, block, partially linked block, linked
#' block). For an overview and review see Jansen and Schulze (2023a,2023b).
#' 
#' The standard naming procedure ixiy, for the comparison of items x and y.
#' 
### Examples ----
#' @examples
#' 
#' 
#' # read and save data set FC
#' data(FC12)
#' 
#' # set seed and define blocks
#' blocks <- matrix(c(5,2,1,4,7,6,3,8,10,12,9,11), ncol = 3)
#' 
#' # define the item-to-factor relation
#' itf <- rep(1:3,4)
#' 
#' # Create and run syntax
#' \donttest{fit <- fit.lavaan(blocks, itf, 'irt', FC, estimator = 'ULSMV')
#' 
#' # get scores for the first two respondents
#' ests <- get.scores(dat=as.matrix(FC)[1:2,],itf=itf, blocks=blocks,
#' fit = fit,mp=rep(0,max(itf)), 
#' sp=diag(1,max(itf)),estimator="MAP",sv=NULL, alg=mvtnorm::Miwa(),log=TRUE, mplus=FALSE)}
#' 
#' @references 
#' Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}
#'
#' Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
#'  
#' Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.
#' 
#' Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 
#' 
#' Yousfi, S. (2019). Person Parameter Estimation for IRT Models of Forced-Choice Data: Merits and Perils of Pseudo-Likelihood Approaches. In: Wiberg, M., Molenaar, D., González, J., Böckenholt, U., Kim, JS. (eds) \emph{Quantitative Psychology}. IMPS 2019. Springer Proceedings in Mathematics & Statistics, vol 322. Springer, Cham. \doi{10.1007/978-3-030-43469-4_3}.
#'
#' @export


get.scores <- function(dat, blocks, itf, fit, alg = mvtnorm::TVPACK(), log = TRUE, mp, sp, serr=TRUE, sv=NULL,blocks_ul=NULL,mplus=FALSE,...){
  if (is.vector(blocks)) {
    blocks <- matrix(blocks, nrow = 1)
  }
  nrank <- dim(blocks)[2]
  
  nfactor <-  max(itf)
  nitem <- max(blocks)
  
  if(mplus==TRUE){
    resul <- read.mplus(blocks,itf,model='irt',fit,intercept=F)
    
    thres <- unlist(lapply(resul$thres,function(x) x[1]))
    names(thres) <- sub('\\$','',sub('1$','',names(thres)))
    names(thres) <- sub('I','i',names(thres))
    
    loadmat <- unlist(lapply(resul$loadings,function(x) x[1]))
    loadmat <- loadmat[-grep('_',names(loadmat))]
    loadmat <- loadmat[order(as.numeric(sub('L','',names(loadmat))))]
    lambda <- matrix(0,nrow=nitem,ncol=nfactor)
    for(i in 1:nfactor){
      lambda[,i] <- ifelse(itf==i,loadmat,0)
    }
    lo <- lambda
    
    unis <- unlist(lapply(resul$resvar,function(x) x[1]))
    names(unis) <- sub('I','i',names(unis))
    
    if(!is.null(blocks_ul)){
      thres <- thres[which(names(thres)%in%i.name(blocks_ul))]
      unis <- unis[which(names(unis)%in%i.name(blocks_ul))]
      blocks <- blocks_ul
    }
    res <- diag(unis)
  }else{
    x <- lavaan::lavInspect(fit, what = "list")
    thres <- unlist(x[x$op=='|','est'])
    names(thres) <- unlist(x[x$op=='|','lhs'])
    thres <- thres[i.name(blocks)]
    
    loadmat <- unlist(x[x$label%in%paste0('L',1:nitem),'est'])
    names(loadmat) <- unlist(x[x$label%in%paste0('L',1:nitem),'label'])
    loadmat <- loadmat[paste0('L',1:nitem)]
    
    lambda <- matrix(0,nrow=nitem,ncol=nfactor)
    for(i in 1:nfactor){
      lambda[,i] <- ifelse(itf==i,loadmat,0)
    }
    lo <- lambda
    
    unis <- unlist(x[x$label%in%gsub('i','e',i.name(blocks)),'est'])
    names(unis) <- i.name(blocks)
    
    if(!is.null(blocks_ul)){
      thres <- thres[which(names(thres)%in%i.name(blocks_ul))]
      unis <- unis[which(names(unis)%in%i.name(blocks_ul))]
      blocks <- blocks_ul
    }
    res <- diag(unis)
  }
  


  
  # Tnb eq 5
  matT <- diag(nrank)
  matT[(1:(nrank-1))*(nrank+1)] <- -1 
  matT[nrank,] <- 1
  
  # create orders of responses from the 01 format
  ord <- lapply(1:nrow(dat), function(y) apply(blocks,1,function(x) pord(dat[y,i.name(x)],x)))
  combs <- cbind(utils::combn(1:nrank,2),utils::combn(1:nrank,2)[2:1,])
  ord_o <- lapply(1:nrow(dat),function(w) apply(ord[[w]],2,function(x) sapply(1:(nrank-1), function(ind, comb) which(apply(comb, 2, function(y) all(y==x[ind:(ind+1)]))), comb=combs)))
  
  # Order intercepts/thresholds once
  thres_bs <- apply(blocks,1,function(x) thres[i.name(x)])
  
  # initialize
  fs <- se <- matrix(NA, nrow(dat), dim(lo)[2])
  if(is.null(sv)) sv <- matrix(0, nrow(dat), dim(lo)[2])
  
  # factor scores for every person
  for(i in 1:nrow(dat)) {
    result <- stats::optim(par=sv[i,],fn=glh, hessian=serr,control = list(fnscale=-1,maxit=2), 
                    lower=rep(-8,dim(lo)[2]),upper=rep(8,dim(lo)[2]),method="L-BFGS-B",
                    dat=dat[i,],nrank=nrank,blocks=blocks,thres_bs=thres_bs,lo=lo,res=res,
                    matT=matT,ord=ord[[i]],ord_o=ord_o[[i]],mp=mp,sp=sp,alg=alg,log=log,
                    ...)
    fs[i,] <- result$par
    if(isTRUE(serr)) se[i,] <- sqrt(diag(MASS::ginv(-result$hessian)))
  }
  return(list("fs"=fs, "se"=se))
}

### Roxygen-izable Documentation ----
#' Count paired comparisons
#' 
#' This function calculates the number of paired comparisons needed to compare a
#'  set of N items.
#' 
#' 
### Inputs ----
#'
#' @param nitem Number of items.
#' 
#' 
### Outputs ---- 
#' @return An integer corresponding to the number of paired comparisons.
#' 
### Details
#' @details 
#' This function is only useful, if the number of paired comparisons of a full
#' design, that is all possible paired comparisons, is of interest. Then the 
#' number is
#' \deqn{\frac{N\times(N-1)}{2}}
#' 
### Examples ----
#' @examples 
#' 
#' # Number if paired comparisons for a set of 15 items = 105.
#' count.combn(15)
#'
#' @export



### Function definition ----
count.combn <- function(nitem){
  return(nitem*(nitem-1)/2)
}

valuesMPout <- function(locator,offset,name,MPoutput, convergence, standardized){
  output <- list()
  for (i in 1:(length(locator))) {
    tmp <- ifelse(standardized==T,max(grep(locator[i],MPoutput)),min(grep(locator[i],MPoutput)))
    tmp <- MPoutput[tmp+offset[i]]
    tmp <- gsub('\\s+',' ',tmp)
    tmp <- strsplit(tmp,' ')[[1]]
    tmp <- tmp[3:length(tmp)] # Estimate,SE,Est/SE, p
    tmp <- ifelse(tmp=="*********",NA,tmp)
    if(convergence==1){
      output[[name[i]]] <- as.numeric(tmp)
    }else{
      output[[name[i]]] <- NA
    }
  }
  return(output)
}

---
title: "Simulating data and estimating Thurstonian IRT and factor models with ThurMod"
author: "Markus Thomas Jansen"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
  toc: yes
vignette: >
  %\VignetteIndexEntry{Simulating data and estimating Thurstonian IRT and factor models with ThurMod}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
params:
  EVAL: !r identical(Sys.getenv("NOT_CRAN"), "true")
editor_options: 
  chunk_output_type: console
---

```{r, SETTINGS-knitr, include=FALSE}
stopifnot(require(knitr))
options(width = 90)
opts_chunk$set(
  comment = NA,
  message = FALSE,
  warning = FALSE
)
```

# Introduction

In this vignette, the basic procedures in data-handling, simulation and estimation with the package `ThurMod` are performed. For the moment, we separate two different model types:
 <ul>
  <li>A Thurstonian factor model</li>
  <li>A Thurstonian IRT model</li>
</ul> 

The Thurstonian factor model was introduced by Maydeu-Olivares \& Böckenholt (2005), the Thurstonian IRT model was introduced by Maydeu-Olivares \& Brown (2010). For a review see Jansen \& Schulze (2023a). For further extensions and discussion about the model types see Jansen \& Schulze (2023b). For example, the factor and IRT model both can be further differentiated. For the differentiation we use the design matrix $A$ which represents all paired comparisons of a design. For a design of four items, this would be
```{r, echo=FALSE}
library(ThurMod)
designA(4)
```

```{r, echo=FALSE}
load("4v.RData")
```


The design matrix can be (Jansen \& Schulze, 2023b):

 <ul>
  <li>A <b>full</b> matrix, that is all possible paired comparisons are considered</li>
  <li>An <b>unlinked block</b> matrix, where only blocks of items and the respective paired comparisons are considered </li>
  <li>A <b>partially linked block</b> matrix, where some of the blocks are linked</li>
  <li>A <b>complete linked block</b> matrix, where all of the blocks are linked</li>
  <li>Anything in between of the other models</li>
</ul> 

# Examples

First, we will load the package required in the vignette.
```{r}
library(ThurMod)
```

The next step is to define the model we are interested in. For this we have to define the following aspects:
 <ul>
  <li>the number of factors of the model</li>
  <li>the number of items of the model</li>
  <li>the item-to-factor relations</li>
</ul> 

We consider a model with four factors (traits), measured by 12 items. The item-to-factor relations are defined so that 
 <ul>
  <li>items 1, 5, 9 measure factor 1</li>
  <li>items 2, 6, 10 measure factor 2</li>
  <li>items 3, 7, 11 measure factor 3</li>
  <li>items 4, 8, 12 measure factor 4.</li>
</ul> 

Further we simulate data of 1000 respondents with loadings between .30 and .95 and latent utility means between -1 and 1. We assume that the data results from rankings, that is that transitivity between responses holds (the variance of the binary indicators is zero). Further, we assume uncorrelated data. We will simulate the data of all paired comparisons possible (see Jansen \& Schulze, 2023b).

Set up the simulation conditions:
```{r}
nfactor <- 4
nitem <- 12
nperson <- 1000
itf <- rep(1:4, 3)
varcov <- diag(1, 4)

# latent utility means
set.seed(69)
lmu <- runif(nitem, -1, 1)
loadings <- runif(nitem, 0.30, 0.95)
```

Next, we simulate a data set based on the true parameter values:
```{r}
data <- sim.data(nfactor = nfactor, nitem = nitem, nperson = nperson, itf = itf,
                 varcov = varcov, lmu = lmu, loadings = loadings)

#save the file
write.table(data, paste0(tempdir(),'/','myDataFile_f.dat'), quote = FALSE, 
            sep = " ", col.names = FALSE, row.names = FALSE)
```

The data set contains all (12 \times 11)/2 = 66 possible paired comparison variables. With this data set we will perform analyses of the full design (IRT, CFA).

## Full design

Full designs include all paired comparisons. The estimation of these designs can take a while, as with categorical data a large correlations matrix must be estimated.

For all functions, the blocks we use must be defined. In a full design, there is only one block with all items:
```{r}
blocks <- matrix(1:nitem, nrow = 1)
```
The `blocks` must be defined as a matrix where the rows correspond to each block. Only for a full design, a vector, for example, `1:12` would work.

### Thurstonian factor models
We will analyse the data with Mplus and `lavaan`. We can do this in two ways: First, in three separate steps, second, directly.

Way 1, step 1: Build syntax
```{r, eval =FALSE}
# Mplus
syntax.mplus(blocks, itf, model = 'lmean', input_path = 'myFC_model_f.inp', data_path = 
               "myDataFile_f.dat")
```
```{r}
#lavaan
modelsyn <- syntax.lavaan(blocks, itf, model = 'lmean')
```
For step 2, now these syntaxes must be run (evaluation will not be performed here):
```{r, eval = FALSE}
# Mplus
system('mplus myFC_model_f.inp', wait = TRUE, show.output.on.console= FALSE)
```
```{r, eval = FALSE}
#lavaan
results_lav1 <- lavaan::lavaan(modelsyn, data = data, ordered = TRUE, auto.fix.first = FALSE,
                      auto.var = TRUE, int.lv.free = TRUE, parameterization = "theta",
                      estimator = 'ULSMV')
```
If you replicate this, be patient, it can take about 20 minutes each, depending on your processing power. 

Step 3: Read results. For Mplus you can either open the output file, or use `read.mplus`
```{r,eval = FALSE}
# Mplus
results_mplus1 <- read.mplus(blocks, itf, model = 'lmean', output_path = "myFC_model_f.out")
```
```{r}
unlist(results_mplus1$fit)
```
```{r, eval = FALSE}
results_lav1 <- lavaan::fitmeasures(results_lav1)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav1
```

The second way is to use `fit_mplus` or `fit_lavaan`, each function does the above steps at once:
```{r, eval = FALSE}
# Mplus
results_mplus2 <- fit.mplus(blocks, itf, model = 'lmean', input_path = 'myFC_model_f.inp',
                            output_path = "myFC_model_f.out", data_path = "myDataFile_f.dat")
```
```{r, eval =  FALSE}
#lavaan
results_lav2 <- fit.lavaan(blocks, itf, model = 'lmean', data = data)
lavaan::fitmeasures(results_lav2)[c('rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                    'rmsea.pvalue.scaled','cfi.scaled')]
```

### Thurstonian IRT models
For IRT models, the procedure is the same, just change `model = 'lmean'` to `model = 'irt'`, for example
```{r, eval = FALSE}
# Mplus
results_mplus2irt <- fit.mplus(blocks, itf, model = 'irt', input_path = 'myFC_model_irt.inp',
                               output_path = "myFC_model_irt.out", data_path = "myDataFile_f.dat")
```
```{r}
unlist(results_mplus2irt$fit)
```
```{r, eval = FALSE}
#lavaan
results_lav2irt <- fit.lavaan(blocks, itf, model = 'irt', data = data)
```
```{r, eval = FALSE}
results_lav2irt <- lavaan::fitmeasures(results_lav2irt)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav2irt
```
```{r, eval = FALSE}
scores_results_lav2irt <- lavaan::lavPredict(results_lav2irt)
```

## Block designs - unlinked
The first block designs were introduced as multidimensional forced-choice blocks (Brown \& Maydeu-Olivares, 2011). However, it was shown that the designs must neither be multidimensional, nor must every paired comparison only be contained once (Jansen \& Schulze, 2023b).

We first simulate a new data set. Set up the simulation conditions:
```{r}
nfactor <- 5
nitem <- 30
nperson <- 1000
itf <- rep(1:5, 6)
varcov <- diag(1, 5)

# latent utility means
set.seed(69)
lmu <- runif(nitem, -1, 1)
loadings <- runif(nitem, 0.30, 0.95)
```

Next, we simulate a data set based on the true parameter values:
```{r}
set.seed(1234)
data <- sim.data(nfactor = nfactor, nitem = nitem, nperson = nperson, itf = itf, varcov = varcov,
                 lmu = lmu, loadings = loadings)
```
```{r, eval = FALSE}
#save the file
write.table(data,'myDataFile.dat', quote = FALSE, sep = " ", col.names = FALSE, row.names = FALSE)
```

Next, we consider unlinked blocks of three items (triplets):
```{r}
blocks <- matrix(sample(1:nitem, nitem), ncol = 3)
```
The blocks are defined by a matrix where the rows are the blocks and the number of columns is the number of items per block. Before we can fit the model, we must ensure that the data fits to the syntax we create. The data simulated before assumes that all items are in ascending order. This is important, as the order of the items determine the coding. We code binary items as
\begin{equation}
   y_{l} =
   \begin{cases}
     1 & \text{if item $i$ is chosen over item $j$} \\
     0 & \text{else} .\\
   \end{cases}
\end{equation}
In a ranking task, all choice alternatives are presented at once, but for each possible comparison, the coding scheme can be used. For example, consider $n = 3$ items which are labeled as {A, B, C}. Then we have the pairs {A, B}, {A, C}, {B, C}. If for {A, B} A is preferred over B then $y_{A,B}=1$ and 0 otherwise. This can be done for every paired comparison and ranking task (Maydeu-Olivares \& Böckenholt, 2005): For {A,C,B} the binary outcomes are $y_{A,B}=1$, $y_{A,C}=1$, and $y_{B,C}=0$. However, if the order of the binary item is changed, then the coding is changed accordingly, e.g. $y_{B,A}=0$, $y_{C,A}=0$, and $y_{C,B}=1$.

We can get an overview over all possible comparisons by the block design, by using `pair.combn`:
```{r}
pair.combn(blocks)
```

See that we have some items, that are not in ascending order. The data simulated assumes, that the first named item (item $i$) has a smaller number e.g., i3i9. The blocks we created however, have some comparisons, where the first items have a larger number e.g. i9i3. There are two ways to fit the syntax to the data: First, we can simply sort the blocks via `blocksort`:

```{r}
blocks_sorted <- blocksort(blocks)
pair.combn(blocks_sorted)
```

Now all paired comparisons that can be derived are in ascending order. 

The second way is to recode the corresponding binary indicators in the data, as if we flip the coding schema. We can just recode the variables:
```{r}
# Get names of binary indicators that have non-ascending names
tmp <- which(pair.combn(blocks)[,1] > pair.combn(blocks)[,2])

# get names
pair_names_b <- i.name(blocks)
pair_names_ori <- i.name(1:nitem)

# Rename
pair_names <- i.name(1:nitem)
if(length(tmp) != 0){
  tmp1 <- pair_names_b[tmp]
  tmp2 <- sub('^i.+i','i', tmp1)
  tmp3 <- tmp1
  for(j in 1:length(tmp)){
    tmp3 <- paste0(tmp2[j], sub(paste0(tmp2[j],'$'), '', tmp1[j]))
    pair_names[which(pair_names %in% tmp3)] <- pair_names_b[tmp[j]]
  }
}

tmp <- which(!names(data) %in% pair_names)
# Clone data
data_recoded <- data
# Recode and rename
data_recoded[,tmp] <- abs(data[,tmp]-1)
names(data_recoded) <- pair_names

```
```{r, eval = FALSE}
# Save data
write.table(data_recoded, 'myDataFile_rec.dat', quote = FALSE, sep = " ", col.names = FALSE,
            row.names = FALSE)
```

Both ways yield the same results. We have to add the argument `data_full = TRUE`, as we simulated all data in the data file, but only use some of the data. We demonstrate only the `model = 'irt'` case, however, also for the CFA model types, these analyses can be performed (`model = 'lmean'`, `model = 'uc'`, or `model = 'simple'`).
```{r, eval = FALSE}
# Blocks_sorted
# Mplus
results_mplus_b <- fit.mplus(blocks_sorted, itf, model = 'irt', input_path = 'myFC_model_bu.inp',
                             output_path = "myFC_model_bu.out", data_path = "myDataFile.dat",
                             data_full = TRUE)
```
```{r}
unlist(results_mplus_b$fit)
```
```{r, eval = FALSE}
#lavaan
results_lav_b <- fit.lavaan(blocks_sorted, itf, model = 'irt', data = data)
results_lav_b_fm <- lavaan::fitmeasures(results_lav_b)
```
```{r, eval = FALSE}
results_lav_b <- lavaan::fitmeasures(results_lav_b)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav_b
```
```{r, eval = FALSE}
scores_results_lav_b <- lavaan::lavPredict(results_lav_b)
```

```{r eval = FALSE}
# Recoded data
# Mplus
results_mplus_brec <- fit.mplus(blocks, itf, model = 'irt', input_path = 'myFC_model_brecu.inp',
                                output_path = "myFC_model_brecu.out", data_path = 
                                  "myDataFile_rec.dat", data_full = TRUE)
```
```{r}
unlist(results_mplus_brec$fit)
```
```{r, eval = FALSE}
#lavaan
results_lav_brec <- fit.lavaan(blocks, itf, model = 'irt', data = data_recoded)
```
```{r, eval = FALSE}
results_lav_brec <- lavaan::fitmeasures(results_lav_brec)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav_brec
```
```{r, eval = FALSE}
scores_results_lav_brec <- lavaan::lavPredict(results_lav_brec)
```

In the case of blocks, we must correct the fit indices, as there are redundancies among the thresholds and tetrachoric correlations. The redundancies can be determined with the function `redundancies`. We can also directly correct the fit with `fit.correct`:
```{r}
#save fit measures
tmp <- results_lav_b_fm
fit.correct(1000, blocks, tmp['chisq.scaled'], tmp['df.scaled'], tmp['baseline.chisq.scaled'],
            tmp['baseline.df.scaled'])
```


## Block designs - linked
Now, we consider linked block designs (Jansen \& Schulze, 2023b). The simplest way to construct these designs is to take the original unlinked design and link all blocks together (rank of the design $r_D=N-1$, where $N$ is the number of items; Jansen \& Schulze, 2023b). The original blocks are:
```{r}
blocks
```

To get a linked design we need extra blocks. The number can be determined with `count.xblocks`
```{r}
count.xblocks(blocks)
```
Hence, we need five extra triplets in this case. We add for example the following linking blocks:
 <ul>
  <li>block 1: 23,14,8</li>
  <li>block 2: 24,28,4</li>
  <li>block 3: 25,16,29</li>
  <li>block 4: 22,26,13</li>
  <li>block 5: 1,21,30</li>
</ul> 

```{r}
blocks_extra <- matrix(c(23,14,8,24,28,4,25,16,29,22,26,13,1,21,30), ncol = 3, byrow = TRUE)
blocks_con <- rbind(blocks, blocks_extra)
```

We can determine the rank of the design matrix with `rankA`:
```{r}
rankA(blocks_con)
```
The rank is 30-1=29. The function `metablock` also gives a overview over which blocks are linked.
```{r}
metablock(blocks_con)
```
There is only one meta block, therefore all items are linked. A counter example would be if we would not add block 4:

```{r}
blocks_extra <- matrix(c(23,14,8,24,28,4,25,16,29,1,21,30), ncol = 3, byrow = TRUE)
blocks_con <- rbind(blocks, blocks_extra)
```

Then the rank is
```{r}
rankA(blocks_con)
```
The rank is 30-1=29$\neq$ 28. And we have two "meta" blocks.
```{r}
metablock(blocks_con)
```

Instead of manually constructing the blocks, `ThurMod` enables the user to get extra blocks via a function `get.xblocks`. We have to define if the blocks should be multidimensional (else `multidim = FALSE`), and if items should not be considered (e.g. because they are negatively keyed):
```{r}
blocks_extra <- get.xblocks(blocks, itf, multidim = TRUE, item_not = NULL)
blocks_con <- rbind(blocks, blocks_extra)
blocks_con
blocks_con_sorted <- blocksort(blocks_con)
```

If we take the blocks as is, we need to recode the data set:
```{r}
# Get names of binary indicators that have non-ascending names
tmp <- which(pair.combn(blocks_con)[,1] > pair.combn(blocks_con)[,2])

# get names
pair_names_b <- i.name(blocks_con)
pair_names_ori <- i.name(1:nitem)

# Rename
pair_names <- i.name(1:nitem)
if(length(tmp)!=0){
  tmp1 <- pair_names_b[tmp]
  tmp2 <- sub('^i.+i','i', tmp1)
  tmp3 <- tmp1
  for(j in 1:length(tmp)){
    tmp3 <- paste0(tmp2[j], sub(paste0(tmp2[j],'$'), '', tmp1[j]))
    pair_names[which(pair_names %in% tmp3)] <- pair_names_b[tmp[j]]
  }
}

tmp <- which(!names(data) %in% pair_names)
# Clone data
data_recoded <- data
# Recode and rename
data_recoded[,tmp] <- abs(data[,tmp]-1)
names(data_recoded) <- pair_names
``` 
```{r, eval = FALSE}
# Save data
write.table(data_recoded, 'myDataFile_rec.dat', quote = FALSE, sep = " ", col.names = FALSE,
            row.names = FALSE)
```

The estimation of linked designs is done equivalent via
```{r, eval = FALSE}
# Blocks_sorted
# Mplus
results_mplus_bc <- fit.mplus(blocks_con_sorted,itf,model='irt',input_path='myFC_model_b_con.inp',
                             output_path="myFC_model_b_con.out",data_path="myDataFile.dat",
                             data_full = TRUE)
```
```{r}
unlist(results_mplus_bc$fit)
```
```{r, eval = FALSE}
#lavaan
results_lav_bc <- fit.lavaan(blocks_con_sorted, itf, model = 'irt', data = data)
results_lav_bc_fm <- lavaan::fitmeasures(results_lav_bc)
```
```{r, eval = FALSE}
results_lav_bc <- lavaan::fitmeasures(results_lav_bc)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav_bc
```
```{r, eval = FALSE}
scores_results_lav_bc <- lavaan::lavPredict(results_lav_bc)
```

```{r eval = FALSE}
# Recoded data
# Mplus
results_mplus_bcrec <- fit.mplus(blocks_con, itf, model = 'irt', input_path = 'myFC_model_brec.inp',
                                output_path = "myFC_model_brec.out",data_path =
                                  "myDataFile_rec.dat",data_full = TRUE, byblock = FALSE)
```
```{r}
unlist(results_mplus_bcrec$fit)
```
```{r, eval = FALSE}
#lavaan
results_lav_bcrec <- fit.lavaan(blocks_con, itf, model = 'irt', data = data_recoded)
```
```{r, eval = FALSE}
results_lav_bcrec <- lavaan::fitmeasures(results_lav_bcrec)[c('chisq.scaled','df.scaled','pvalue.scaled',
                                     'rmsea.scaled','rmsea.ci.lower.scaled','rmsea.ci.upper.scaled',
                                     'rmsea.pvalue.scaled','cfi.scaled')]
```
```{r}
results_lav_bcrec
```
```{r, eval = FALSE}
scores_results_lav_bcrec <- lavaan::lavPredict(results_lav_bcrec)
```

Here, we also must correct the fit indices, as there are redundancies among the thresholds and tetrachoric correlations. Again, the redundancies can be determined with the function `redundancies`. We directly correct the fit with `fit.correct`:
```{r}
#save fit measures
tmp <- results_lav_bc_fm
fit.correct(1000, blocks_con, tmp['chisq.scaled'], tmp['df.scaled'], tmp['baseline.chisq.scaled'],
            tmp['baseline.df.scaled'])
```

## Simulating only relevant data

The estimation of a full design will most likely seldom be of interest. More likely, (linked) block designs are of interest. Especially for simulations with many items (more than 50), many binary variables are simulated and saved. While the simulation of all items is important (see Jansen \& Schulze, 2023b), saving all items is not. `sim.data` can also return only items that are of relevance. For that we need to specify the argument `variables`. Assume we have the linked block design we specified before. We only have to give the item names of the sorted blocks and define it as the `variables` argument:
```{r}
#Get the relevant names
tmp_names <- i.name(blocks_con_sorted)

#same example as before
set.seed(1234)
data <- sim.data(nfactor = nfactor, nitem = nitem, nperson = nperson, itf = itf, varcov = varcov,
                 lmu = lmu, loadings = loadings, variables = tmp_names)
```
```{r, eval = FALSE}
#save the file
write.table(data, 'myDataFile.dat', quote = FALSE, sep = " ", col.names = FALSE,
            row.names = FALSE)
```
For the Mplus analysis, the argument `data_full` must then be set to `FALSE`.

Taken together, we have demonstrated the basic steps on how to use the functions given by `ThurMod`. Please report any bugs. If you miss functions that could be helpful for the analysis of Thurstonian forced-choice data, please let me know.

# References
Brown, A., \& Maydeu-Olivares, A. (2011). Item response modeling of forced-choice questionnaires. <i>Educational and Psychological Measurement</i>, <i>71</i>(3), 460-502. <a href="https://doi.org/10.1177/0013164410375112">https://doi.org/10.1177/0013164410375112</a>

Jansen, M. T., \& Schulze, R. (2023a). <i>Item scaling of social desirability using conjoint measurement: A comparison of ratings, paired comparisons, and rankings</i>. Manuscript in preparation.

Jansen, M. T., \& Schulze, R. (in press). <i>The Thurstonian linked bock design: Improving Thurstonian modeling for paired comparison and ranking data</i>. Educational and Psychological Measurement.

Maydeu-Olivares, A., \& Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. <i>Psychological Methods</i>, <i>10</i>(3), 285–304. <a href="https://doi.org/10.1037/1082-989X.10.3.285">https://doi.org/10.1037/1082-989X.10.3.285</a>

Maydeu-Olivares, A., \& Brown, A. (2010). Item response modeling of paired comparison and ranking data. <i>Multivariate Behavioural Research</i>, <i>45</i>(6), 935-974. <a href="https://doi.org/10.1080/00273171.2010.531231">https://doi.org/10.1080/00273171.2010.531231</a>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/redundancies.R
\name{redundancies}
\alias{redundancies}
\title{Determine the number of redundancies}
\usage{
redundancies(blocks, warn = TRUE)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{warn}{Logical. Should warning messages be returned? Defaults to `TRUE`.}
}
\value{
Returns an integer of the number of redundancies.
}
\description{
This function determines the number of redundancies among the tetrachoric
correlations and thresholds.
}
\details{
If a ranking design is used (variances of binary indicators is zero)
there are redundancies among the thresholds and tetrachoric correlations to 
be estimated. This is the case, whenever the number of items per block is 
larger than two. In these cases the degrees of freedom must be corrected by
subtracting the redundancies. For more details see Jansen and Schulze (2023)
and Maydeu-Olivares (1999).
}
\examples{

# Define 30 items divided by ten triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,10)

# Determine the redundancies
redundancies(blocks)

}
\references{
Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.

Maydeu-Olivares, A. (1999). Thurstonian modeling of ranking data via mean and covariance structure analysis. \emph{Psychometrika}, \emph{64}(3), 325-340. \doi{10.1007/BF02294299}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blocksort.R
\name{blocksort}
\alias{blocksort}
\title{Sorts the blocks in ascending numbering}
\usage{
blocksort(blocks)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows 
must be the number of blocks, each row represents a block and contains the 
item numbers. The number of columns represent the number of items per block.}
}
\value{
Returns a matrix consisting of the blocks where all items per blocks
are sorted in ascending order.
}
\description{
This function sorts all items in a block into ascending order.
}
\examples{

# Define 30 items divided by ten triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

# sort the blocks
blocksort(blocks)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count.xblocks.R
\name{count.xblocks}
\alias{count.xblocks}
\title{Determine the number of extra blocks}
\usage{
count.xblocks(blocks)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}
}
\value{
An integer corresponding to the number of extra blocks needed.
}
\description{
This function determines the minimal number of extra blocks needed in order
to link all blocks.
}
\details{
The matrix of blocks must be constructed so that the number of
columns corresponds to the number of items per block. The number of rows
corresponds to the number of blocks for the specific measure. If $p$ is the
number of blocks, and $k$ is the number of items per block (e.g. $k=3$ for
triplets), then the number of extra blocks can be determined by (see also
Jansen & Schulze, 2023)
\deqn{\lceil\frac{p-1}{k-1}\rceil}
}
\examples{

# Define a matrix of blocks
blocks <- matrix(1:15,ncol=3, byrow=TRUE)

# Determine the number of extra blocks needed
count.xblocks(blocks)

}
\references{
Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_FC_raw.R
\docType{data}
\name{FC_raw}
\alias{FC_raw}
\title{Raw ranking data of $N=15$ items from three factors/traits (Thurstonian modeling)}
\format{
A data frame with 1000 observations on 15 variables.
}
\usage{
data(FC_raw)
}
\description{
This data set contains synthetic raw data of 1000 participants on a ranking
task on 15 items.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankA.R
\name{rankA}
\alias{rankA}
\title{Determine the rank of the design matrix defined by the blocks.}
\usage{
rankA(blocks)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}
}
\value{
Returns the rank of the design matrix as an integer.
}
\description{
This function determines the rank of the fundamental design matrix defined
by the blocks.
}
\examples{

# Define nine items divided by three triplets as blocks
blocks <- matrix(c(1:9), ncol = 3)

# Determine the rank of the design matrix
rankA(blocks)
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_FC.R
\docType{data}
\name{FC}
\alias{FC}
\title{Paired comparisons of $N=15$ items from one factor/trait (Thurstonian modeling)}
\format{
A data frame with 1000 observations on 105 variables. For a variable
ixiy, the result is the response preferences between item x and item y. It is
coded a 1, if item x is preferred over item y, and 0 otherwise.
}
\usage{
data(FC)
}
\description{
This data set contains synthetic data of 1000 participants on all binary
indicators of 15 items. For each paired comparison, participants had to rank
the two alternative items according to their preference. It is assumed that 
transitivity holds (that is, the data comes from a ranking task). More
details can be found in Brown and Maydeu-Olivares (2011), Jansen and Schulze
(2023) and Maydeu-Olivares and Böckenholt (2005).
}
\references{
Brown, A, & Maydeu-Olivares, A. (2011). Item response modeling of forced-choice questionnaires. \emph{Educational and Psychological Measurement}, \emph{71}(3), 460-502. \doi{10.1177/0013164410375112}

Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.

Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/i.name.R
\name{i.name}
\alias{i.name}
\title{Creates names for paired comparisons of a given design.}
\usage{
i.name(blocks)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}
}
\value{
Returns a character vector, containing names for all binary indicators
of a design.
}
\description{
This function creates names for paired comparisons in the ixiy scheme. If
items 1 and 2 are compared, this corresponds to i1i2.
}
\examples{

# Define 30 items divided by ten triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

i.name(blocks)
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.correct.R
\name{fit.correct}
\alias{fit.correct}
\title{Correct degree of freedom and fit indices in Thurstonian block models}
\usage{
fit.correct(n, blocks, chi2_mod, df_mod, chi2_base, df_base)
}
\arguments{
\item{n}{The number of respondents.}

\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{chi2_mod}{The \eqn{\chi^2}{chi^2} value of the estimated model.}

\item{df_mod}{The degrees of freedom of the estimated model.}

\item{chi2_base}{The \eqn{\chi^2}{chi^2} value of the baseline model.}

\item{df_base}{The degrees of freedom of the baseline model.}
}
\value{
Returns a vector containing corrected degrees of freedom, and the
corrected RMSEA and CFI values.
}
\description{
Correct fit indices (RMSEA and CFI) by correcting the degrees of freedom
after estimation a Thurstonian model.
}
\details{
If a ranking design is used (variances of binary indicators is zero)
there are redundancies among the thresholds and tetrachoric correlations to 
be estimated. This is the case, whenever the number of items per block is 
larger than two. In these cases the degrees of freedom must be corrected by
subtracting the redundancies. For more details see Jansen and Schulze (2023)
and Maydeu-Olivares (1999).
}
\examples{

# Define 30 items divided by ten triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

# Assume the model yield the following fit, with 426 respondents
# chi2_mod = 224.456, df_mod = 59, chi2_base = 1056.566, df_base = 90

fit.correct(426,blocks,224.456,59,1056.566,90)

# The corrected values are rmsea = 0.0917892; cfi =  0.8184749

}
\references{
Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.

Maydeu-Olivares, A. (1999). Thurstonian modeling of ranking data via mean and covariance structure analysis. \emph{Psychometrika}, \emph{64}(3), 325-340. \doi{10.1007/BF02294299}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/designA.R
\name{designA}
\alias{designA}
\title{Create the Thurstonian design matrix for paired comparison and ranking data.}
\usage{
designA(blocks = NULL, nitems = max(unique(blocks)))
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{nitems}{The number of items that are included in the design.}
}
\value{
Returns a design matrix which includes all paired comparisons
derivable from the blocks.
}
\description{
Creates the Thurstonian design matrix for paired comparison and ranking data,
given by blocks or the number of items.
}
\details{
Each Thurstonian design can be defined by blocks of at least two
items. The function determines the fundamental design matrix A of the
Thurstonian design, including all possible paired comparisons that can be
derived by the blocks. For further information of the importance of the
design matrix, see Jansen and Schulze (2023a,2023b).
}
\examples{

# Define a matrix of blocks
blocks <- matrix(1:15,ncol=3, byrow=TRUE)

# Get the design matrix
loading_Matrix <- designA(blocks)

}
\references{
Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Manuscript submitted.

Jansen, M. T., & Schulze, R. (2023b). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/syntax.mplus.R
\name{syntax.mplus}
\alias{syntax.mplus}
\title{Create Mplus syntax for Thurstonian forced choice designs.}
\usage{
syntax.mplus(blocks, itf, model, input_path = "myFC_model.inp",
  data_path = "myDataFile.dat", fscore_path = "myFactorScores.dat",
  title = "myFC_model", ID = FALSE, byblock = TRUE,
  estimator = "ULSMV", data_full = FALSE, standardized = TRUE,
  rename_list = NULL)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{model}{A descriptor for the model. Can be one of `'lmean'`,
`'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
the `'simple'` statement defines the Thurstone case.}

\item{input_path}{Path to save the Mplus input file. Defaults to `'myFC_model.inp'`.}

\item{data_path}{Path of the data file for Mplus. Defaults to `'myDataFile.dat'`.}

\item{fscore_path}{Path to save the file of factor scores. Defaults to
`'myFactorScores.dat'`.}

\item{title}{Title of the Mplus model. Defaults to `'myFC_model'`.}

\item{ID}{Logical. Should a ID variable be included? The ID must be the first 
variable in the data set. Defaults to `FALSE`.}

\item{byblock}{Logical. Should the order in Mplus variable statement be the same
as in the blocks. Defaults to `TRUE`.}

\item{estimator}{Which estimator should be used? All Estimators that are
available in Mplus can be used. Defaults to `'ULSMV'`.}

\item{data_full}{Logical. Are the data considered to be from a full design?
Defaults to `FALSE`.}

\item{standardized}{Logical. Should standardized values be computed? Defaults
to `TRUE`.}

\item{rename_list}{A list with two vectors to rename the objects in the syntax.
Vector one is the original names, vector two the new names. Defaults to `NULL`.}
}
\value{
Returns a description of the user-specified model. Typically, the
model is described using the lavaan model syntax. See `lavaan::model.syntax`
for more information.

Saves a list Mplus input file at 'input_path'.
}
\description{
This function writes and saves the Mplus syntax given the specifications of a
Thurstonian forced choice design.
}
\details{
The syntax currently is able to perform model analysis for the
latent utility models (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
model types can be performed with all types of forced choice designs (full,
block, partially linked block, linked block). For an overview and review see
Jansen and Schulze (2023a,2023b).

The function writes and saves the Mplus input files.

The standard naming procedure ixiy, for the comparison of items x and y, 
can be changed by specifying the `rename_list` argument. The first vector of
the schould be the vector of original names, for example 
`c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
the second vector should contain the new names, for example 
`c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.

### Outputs ----
}
\examples{

# read and save data set FC
data(FC)
write.table(FC,paste0(tempdir(),'/','my_data.dat'),quote=FALSE, sep=" ",
col.names = FALSE, row.names = FALSE)

# set seed and define blocks
set.seed(1)
blocks <- matrix(sample(1:15,15), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,5)

# Create and save Mplus syntax
syntax.mplus(blocks,itf,'lmean',data_path = 'my_data.dat', data_full = TRUE,
input_path = paste0(tempdir(),'/','myFC_model'))

}
\references{
Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}

Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
 
Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.

Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim.data.R
\name{sim.data}
\alias{sim.data}
\title{Create data based on Thurstonian model equations}
\usage{
sim.data(nfactor = 1, nitem, nperson, itf, model = "factor",
  variables = NULL, ints = NULL, lmu = NULL, ivarcov = NULL,
  loadings = NULL, varcov = NULL, graded = FALSE, ncat = NULL,
  thres = NULL, transitive = TRUE, var = 0, eta = NULL,
  fvalues = FALSE, sim = TRUE)
}
\arguments{
\item{nfactor}{The number of factors. Defaults to 1.}

\item{nitem}{The number of items.}

\item{nperson}{The number of data points (= respondents) to simulate.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{model}{The model class from which to simulate. Options are 'simple', 'factor', 'uc' and 'irt'. Defaults to 'factor'.}

\item{variables}{A vector containing the names of paired comparison variables
to return. If `NULL` (default), all variables are returned.}

\item{ints}{A vector defining the latent intercepts of item pairs.}

\item{lmu}{A vector defining the latent means of items.}

\item{ivarcov}{A matrix defining the variance-covariance matrix of the items.}

\item{loadings}{A vector defining the loadings of items.}

\item{varcov}{A matrix defining the variance-covariance matrix of the traits.}

\item{graded}{Logical. Should a graded preference model be simulated? Defaults to `FALSE`.}

\item{ncat}{Numerical. The number of categories to specify for graded preference models. If graded = `FALSE`, ncat is 2.}

\item{thres}{A vector of thresholds to categorize the latent difference response. If no thresholds are given, they are simulated from the distribution of the latent items. If graded = `FALSE`, the thresholds are all 0.}

\item{transitive}{Logical. Should the data be transitive? If `TRUE`, ranking
data is simulated, else paired comparison data is simulated. Defaults to `TRUE`.}

\item{var}{A vector containing the variances for each paired comparison. Defaults to 0.}

\item{eta}{A nfactor-dimensional matrix with factor scores. Defaults to `NULL`. If `NULL` factor scores are simulated from multivariate normal distribution.}

\item{fvalues}{Logical. Should simulated factor values be returned? Defaults to `FALSE`.}

\item{sim}{Logical. Should the simulated data be returned? Defaults to `TRUE`.}
}
\value{
Returns a list containing the true factor scores and the data, or a
matrix containing the data.
}
\description{
Simulates a data set of paired comparisons or ranking data based a Thurstonian
latent utility model.
}
\details{
The syntax currently is able to simulate data from the
latent utility model (`'simple'` and `'factor'`; Maydeu-Olivares & Böckenholt, 2005) the 
unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010).
}
\examples{

nfactor <- 3
nitem <- 15
nperson <- 1000
itf <- rep(1:3,5)
varcov <- diag(1,3)

# latent utility means
set.seed(69)
lmu <- runif(nitem, -1, 1)
loadings <- runif(nitem, 0.30, 0.95)

FC <- sim.data(nfactor=nfactor, nitem=nitem, nperson=nperson, itf=itf, 
varcov = varcov, lmu = lmu, loadings=loadings)
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/syntax.lavaan.R
\name{syntax.lavaan}
\alias{syntax.lavaan}
\title{Create lavaan syntax for Thurstonian forced choice analysis}
\usage{
syntax.lavaan(blocks, itf, model, rename_list = NULL)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{model}{A descriptor for the model. Can be one of `'lmean'`,
`'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
the `'simple'` statement defines the Thurstone case.}

\item{rename_list}{A list with two vectors to rename the objects in the syntax.
Vector one is the original names, vector two the new names. Defaults to `NULL`.}
}
\value{
Returns a description of the user-specified model. Typically, the
model is described using the lavaan model syntax. See `lavaan::model.syntax`
for more information.
}
\description{
This function writes a lavaan syntax given the specifications of the
Thurstonian forced choice model.
}
\details{
The syntax currently is able to perform model analysis for the
latent utility models (`'simple'` and `'lmean'`; Maydeu-Olivares & Böckenholt, 2005) the 
unconstrained factor model (`'uc'`; Maydeu-Olivares & Böckenholt, 2005) and 
the IRT model(`'irt'`; Maydeu-Olivares & Brown, 2010). Additionally, all
model types can be performed with all types of forced choice designs (full,
block, partially linked block, linked block). For an overview and review see
Jansen and Schulze (2023a,2023b).

The standard naming procedure ixiy, for the comparison of items x and y, 
can be changed by specifying the `rename_list` argument. The first vector of
the schould be the vector of original names, for example 
`c('i1i2','i1i3','i2i3','Trait1','Trait2','Trait3')`
the second vector should contain the new names, for example 
`c('A01E12','A01C13','E01C23','Agree','Extra','Consc')`.
}
\examples{

# read data set FC
data(FC)

# set seed and define blocks
set.seed(1)
blocks <- matrix(sample(1:15,15), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,5)

# Create lavaan model syntax
syntax.lavaan(blocks,itf,'irt')


}
\references{
Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}

Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
 
Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.

Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.scores.R
\name{get.scores}
\alias{get.scores}
\title{Estimate factor scores based on Genuine Likelihood}
\usage{
get.scores(dat, blocks, itf, fit, alg = mvtnorm::TVPACK(), log = TRUE, mp,
  sp, serr = TRUE, sv = NULL, blocks_ul = NULL, mplus = FALSE, ...)
}
\arguments{
\item{dat}{A data.frame or matrix defining holding the binary coded and named
response data.}

\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{fit}{The fit object. In case of Mplus, this is the path to the Mplus output file. In case of lavaan this is the lavaan Object.}

\item{alg}{The algorithm to use for numerical integration. See ?mvtnorm::pmvnorm.}

\item{log}{logical. Should the log transformed results be returned. Defaults to TRUE.}

\item{mp}{The means of prior distribution. Defaults to multivariate normal with means 0 and vars 1}

\item{sp}{The vars of prior distribution. Defaults to multivariate normal with means 0 and vars 1}

\item{serr}{Logical. Should the standard errors be returned. Defaults to TRUE.}

\item{sv}{Starting values. Defaults to NULL.}

\item{blocks_ul}{Similar to `blocks`, but only with the unlinked design. Useful
for comparison with other functions based on R.}

\item{mplus}{Logical. Are results read from mplus? Defaults to FALSE.}

\item{...}{other arguments passed by get.scores for optim.}
}
\value{
The result is a list with the factor scores and the standard errors per person and factor.
}
\description{
This function estimates factor scores based on genuine likelihood (Yousfi, 2019).
}
\details{
The function estimates factor scores based on the Thurstonian IRT model
and based on genuine likelihood proposed by Yousfi (2019). The function allows
for the estimation of factor scores under all block designs, limited only by 
numerical integration procedures. For items per block between 5 and 20 use 
`alg=mvtnorm::Miwa()`. Additionally, all model types can be performed with all
types of forced choice designs (full, block, partially linked block, linked
block). For an overview and review see Jansen and Schulze (2023a,2023b).

The standard naming procedure ixiy, for the comparison of items x and y.
}
\examples{


# read and save data set FC
data(FC12)

# set seed and define blocks
blocks <- matrix(c(5,2,1,4,7,6,3,8,10,12,9,11), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,4)

# Create and run syntax
\donttest{fit <- fit.lavaan(blocks, itf, 'irt', FC, estimator = 'ULSMV')

# get scores for the first two respondents
ests <- get.scores(dat=as.matrix(FC)[1:2,],itf=itf, blocks=blocks,
fit = fit,mp=rep(0,max(itf)), 
sp=diag(1,max(itf)),estimator="MAP",sv=NULL, alg=mvtnorm::Miwa(),log=TRUE, mplus=FALSE)}

}
\references{
Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}

Maydeu-Olivares, A., & Brown, A. (2010). Item response modeling of paired comparison and ranking data. \emph{Multivariate Behavioural Research}, \emph{45}(6), 935-974. \doi{10.1080/00273171.2010.531231}
 
Jansen, M. T., & Schulze, R. (2023a). \emph{Linear factor analytic Thurstonian forced-choice models: Current status and issues}. Educational and Psychological Measurement.

Jansen, M. T., & Schulze, R. (2023b, in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. 

Yousfi, S. (2019). Person Parameter Estimation for IRT Models of Forced-Choice Data: Merits and Perils of Pseudo-Likelihood Approaches. In: Wiberg, M., Molenaar, D., González, J., Böckenholt, U., Kim, JS. (eds) \emph{Quantitative Psychology}. IMPS 2019. Springer Proceedings in Mathematics & Statistics, vol 322. Springer, Cham. \doi{10.1007/978-3-030-43469-4_3}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count.combn.R
\name{count.combn}
\alias{count.combn}
\title{Count paired comparisons}
\usage{
count.combn(nitem)
}
\arguments{
\item{nitem}{Number of items.}
}
\value{
An integer corresponding to the number of paired comparisons.
}
\description{
This function calculates the number of paired comparisons needed to compare a
 set of N items.
}
\details{
This function is only useful, if the number of paired comparisons of a full
design, that is all possible paired comparisons, is of interest. Then the 
number is
\deqn{\frac{N\times(N-1)}{2}}
}
\examples{

# Number if paired comparisons for a set of 15 items = 105.
count.combn(15)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recode.R
\name{recode}
\alias{recode}
\title{Recode variables.}
\usage{
recode(var = var, vals = c(1, 2, 3, 4, 5), ct = c(5, 4, 3, 2, 1),
  cat = FALSE)
}
\arguments{
\item{var}{A variable/column of a data matrix/ data frame containing data to be recoded.}

\item{vals}{A vector containing the original scores. If cat is TRUE, these are thresholds.}

\item{ct}{A vector containing the desired scores.}

\item{cat}{Logical. should the data be categorized?

### Outputs ----}
}
\value{
Returns a vector with the recoded input values.
}
\description{
Recode variables.
}
\details{
This function is a simple recode function. It is possible to recode any value to any other value, as long as the class of values within a vector are equal. If an interval of numerical values should be categorized, for N categories, N-1 thresholds must be given. The first N-1 categories are constructed by using all values that are smaller or equal (<=) than the thresholds, the last category is constructed to be all values larger than the last threshold.
}
\examples{
#numerical
v <- rep(c(1:5),10)
v_r <- recode(v, c(1:5), c(5:1))

# character
v <- rep(c('a','b','d','e'),10)
v_r <- recode(v, c('a','b','d','e'), c('apple','1','dummy',TRUE))

# interval
set.seed(1)
v <- sample(runif(20,0,6))

# recode x <= 1.3 into 1
# recode x <= 2.6 into 2
# recode x <= 3.9 into 3
# recode x <= 5.2 into 4
# recode x > 5.1 into 5
v_r <- recode(v,c(1.3,2.6,3.9,5.2),c(1:5),cat=TRUE)

}
\author{
Markus Thomas Jansen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pair.combn.R
\name{pair.combn}
\alias{pair.combn}
\title{Determine all paired comparisons}
\usage{
pair.combn(blocks, unique = TRUE)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{unique}{Logical. Should only unique paired comparisons be returned?}
}
\value{
Returns a matrix with all paired comparisons defined by a design.
}
\description{
This function returns a matrix containing all paired comparisons defined by
a design.
}
\examples{

#' # Define 30 items divided by three triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

# Get all blocks
pair.combn(blocks)
    
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.xblocks.R
\name{get.xblocks}
\alias{get.xblocks}
\title{Get extra blocks in a Thurstonian design, that links all blocks.}
\usage{
get.xblocks(blocks, itf, multidim, item_not = NULL, min = FALSE,
  show.warnings = FALSE)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{multidim}{Logical. Should the items within each linking block be
forced to be multidimensional?}

\item{item_not}{The items that are differently keyed compared to the majority
of items.}

\item{min}{Logical. Should a minimal number of blocks contain mixed keyed items?}

\item{show.warnings}{Logical. Should warnings be shown?}
}
\value{
The result is a matrix where the rows correspond to the specific
extra blocks.
}
\description{
The function creates extra blocks for a Thurstonian design, that links all
initial blocks with as few extra blocks as possible. The number of extra
blocks is determined by `count.xblocks` (see Jansen & Schulze, 2023).
}
\examples{

# Define a matrix of blocks
blocks <- matrix(1:15,ncol=3, byrow=TRUE)

# define the item-to-factor relation
itf <- rep(1:3,5)
 
# Get the extra blocks for a completely linked design
get.xblocks(blocks, itf, FALSE)

}
\references{
Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_FC_scores.R
\docType{data}
\name{FC_scores}
\alias{FC_scores}
\title{Scores of the data set 'FC' from Mplus.}
\format{
A data frame with 1000 observations on 111 variables. For a variable
ixiy, the result is the response preferences between item x and item y. It is
coded a 1, if item x is preferred over item y, and 0 otherwise. The last six
variables are the resulting factor scores and the standard error of factor 
scores for three factors.
}
\usage{
data(FC_scores)
}
\description{
This data set contains synthetic data of 1000 participants on all binary
indicators of 15 items and their factor scores. For each paired comparison, 
participants had to rank the two alternative items according to their
preference. It is assumed that transitivity holds (that is, the data comes
from a ranking task). More details can be found in Brown and Maydeu-Olivares
(2011), Jansen and Schulze (2023) and Maydeu-Olivares and Böckenholt (2005).
}
\references{
Brown, A, & Maydeu-Olivares, A. (2011). Item response modeling of forced-choice questionnaires. \emph{Educational and Psychological Measurement}, \emph{71}(3), 460-502. \doi{10.1177/0013164410375112}

Jansen, M. T., & Schulze, R. (in review). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}.

Maydeu-Olivares, A., & Böckenholt, U. (2005). Structural equation modeling of paired-comparison and ranking data. \emph{Psychological Methods}, \emph{10}(3), 285-304. \doi{10.1037/1082-989X.10.3.285}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliabiltyFS.R
\name{reliabiltyFS}
\alias{reliabiltyFS}
\title{Calculate reliability estimate for factor scores.}
\usage{
reliabiltyFS(scores, scores_se, method = "sub")
}
\arguments{
\item{scores}{A matrix with factor scores. Rows are respondents, columns are
factors/traits.}

\item{scores_se}{A matrix with the standard errors of the factor scores. 
Rows are respondents, columns are factors/traits. Factors/traits must be in
the same order as for `scores`.}

\item{method}{Can be one of 'div' or 'sub'. See details. Defaults to 'sub'.}
}
\value{
Returns the reliability value. If method = 'irt', the a plot with the
reliability depending on the factorscore is returned.
}
\description{
This function returns a reliability estimate for factor scores estimated via
the Thurstonian IRT models.
}
\details{
The function returns the empirical reliability of factor scores. If
\eqn{\sigma^2} is the sample variance of the estimated scores and \eqn{\bar{sigma}^2_{error}}
is the average if the squared scores, that is

\deqn{\bar{sigma}^2_{error}=\frac{1}{N}\sum^N_i=1 se_{scores}^2}

then the subtraction method, a classical reliability estimate similar to
classical test theory is returned using 'sub' yields
\deqn{\frac{\sigma^2-\bar{sigma}^2_{error}}{\sigma^2}}
for the reliability of the scores. If 'div' is chosen, and alternative
division based approach is used.
\deqn{\frac{\sigma^2}{\sigma^2+\bar{sigma}^2_{error}}}.

If 'irt' is chosen, a plot returning the standard error of the scores with
the scores is returned per factor.
}
\examples{

# read and save data set FC
data(FC_scores)

# get reliability
reliabiltyFS(FC_scores[,c(106,108,110)],FC_scores[,c(107,109,111)])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metablock.R
\name{metablock}
\alias{metablock}
\title{Find all general blocks}
\usage{
metablock(blocks)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}
}
\value{
Returns a list of items that form meta interlinked blocks.
}
\description{
This function creates meta interlinked blocks within a Thurstonian
design.
}
\details{
This function creates meta interlinked blocks of a block design.
These are blocks, that have at least one link from each of its items to
any other of its items. If there is not such a link between every item,
there are at least two meta blocks.
}
\examples{

# Define 30 items divided by ten triplets as blocks
blocks <- matrix(c(1:30), ncol = 3)

# Add one block to link the first three blocks.
blocks <- rbind(blocks,c(1,2,3))

# Find mata blocks
metablock(blocks)
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.mplus.R
\name{read.mplus}
\alias{read.mplus}
\title{Reads results from Mplus output file.}
\usage{
read.mplus(blocks, itf, model, output_path, convergence = TRUE,
  fit.stat = TRUE, loading = TRUE, cor = TRUE, intercept = TRUE,
  threshold = TRUE, resvar = TRUE, standardized = FALSE)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{model}{A descriptor for the model. Can be one of `'lmean'`,
`'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
the `'simple'` statement defines the Thurstone case.}

\item{output_path}{Path to the Mplus output file. Defaults to `'myFC_model.out'`.}

\item{convergence}{Logical. Should a message for convergence be returned?
Defaults to `TRUE`.}

\item{fit.stat}{Logical. Should fit statistics be returned? Defaults to `TRUE`.}

\item{loading}{Logical. Should loading estimates be returned? Defaults to `TRUE`.}

\item{cor}{Logical. Should latent correlation estimates be returned?
Defaults to `TRUE`.}

\item{intercept}{Logical. Should intercepts be returned? Does only work for
`model = 'lmean'`. Defaults to `TRUE`.}

\item{threshold}{Logical. Should thresholds be returned? Does only work for
`model = 'uc'` or `'irt'`. Defaults to `TRUE`.}

\item{resvar}{Logical. Should residual variances be returned? Defaults to `TRUE`.}

\item{standardized}{Logical. Should standardized values be returned? Defaults
to `FALSE`.}
}
\value{
Returns a list containing the specified results, after model analysis, by reading
the results from the 'output_path'.
}
\description{
This function reads and returns results from an Mplus output file.
}
\examples{

# read and save data set FC
data(FC)

write.table(FC,paste0(tempdir(),'/','my_data.dat'),quote=FALSE, sep=" ",
col.names = FALSE, row.names = FALSE)


# set seed and define blocks
set.seed(1)
blocks <- matrix(sample(1:15,15), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,5)

# perform analysis 
\dontrun{
fit.mplus(blocksort(blocks),itf,'irt',data_path = 'mydata.dat', data_full = TRUE,
input_path = paste0(tempdir(),'/','myFC_model'))


# After estimation
read.mplus(blocks,itf,'irt',output_path = paste0(tempdir(),'/','myFC_model.out'))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.xblocks.any.R
\name{get.xblocks.any}
\alias{get.xblocks.any}
\title{Get extra blocks in a Thurstonian design, that links as few blocks as possible.}
\usage{
get.xblocks.any(blocks, itf, multidim)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{multidim}{Logical. Should the items within each linking block be
forced to be multidimensional?}
}
\value{
The result is a matrix where the rows correspond to the specific
extra blocks.
}
\description{
The function creates extra blocks in a Thurstonian design, that links as few
initial blocks as possible, with the number of blocks determined by
`count.xblocks`. This is only useful for comparisons between linked and
partially linked block designs (Jansen & Schulze, 2023).
}
\details{
The main strategy of the function is to create extra blocks that
link as few blocks as possible, with the number of blocks determined by
`count.xblocks`. Therefore, first all combinations of additional blocks with
the first two blocks are created. If more extra blocks are needed the
function uses block three, four, etc..
}
\examples{

# Define a matrix of blocks
blocks <- matrix(1:15,ncol=3, byrow=TRUE)

# define the item-to-factor relation
itf <- rep(1:3,5)

# Get the extra blocks for a partially linked design
get.xblocks.any(blocks, itf, FALSE)

}
\references{
Jansen, M. T., & Schulze, R. (2023). \emph{The Thurstonian linked block design: Improving Thurstonian modeling for paired comparison and ranking data}. Manuscript submitted.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod.matrices.R
\name{mod.matrices}
\alias{mod.matrices}
\title{Create model matrices for Thurstonian modeling}
\usage{
mod.matrices(blocks, itf, model)
}
\arguments{
\item{blocks}{A matrix defining the blocks of the model. The number of rows
must be the number of blocks, each row represents a block and contains the
item numbers. The number of columns present the number of items per block.}

\item{itf}{A vector defining the items-to-factor relation. For example
`c(1,1,1,2,2,2)` defines six items, the first three correspond to factor 1,
the second three correspond to factor 2.}

\item{model}{A descriptor for the model. Can be one of `'lmean'`,
`'uc'`, `'irt'` or  `'simple2'`, `'simple3'` or `'simple5'`. The Number behind 
the `'simple'` statement defines the Thurstone case.}
}
\value{
Returns a list of elements containing model matrix information.
}
\description{
This function creates and returns model matrices of Thurstonian model equations.
}
\examples{

# set seed and define blocks
set.seed(1)
blocks <- matrix(sample(1:15,15), ncol = 3)

# define the item-to-factor relation
itf <- rep(1:3,5)

mod.matrices(blocks,itf,'irt')
    
}
